! PROGRAM WEF1041
! ------------------------------------------------------------------------------
! PURPOSE - 
!   TAKEOFF AND LANDING PERFORMANCE CAPABILITIES OF TRANSPORT CATEGORY AIRCRAFT

!        WEF1041 COMPUTES TAKE-OFF PERFORMANCE WITH OPTIONS
!        FOR F.A.R. BALANCED FIELD LENGTH.

! ( NASA Langley Research Center )

! One of the most important considerations in the design of a commercial 
! transport aircraft is the aircraft's performance during takeoff and landing 
! operations. The aircraft must be designed to meet field length constraints in 
! accordance with airworthiness standards specified in the Federal Aviation 
! Regulations. In addition, the noise levels generated during these operations 
! must be within acceptable limits. This computer program provides for the 
! detailed analysis of the takeoff and landing performance capabilities of 
! transport category aircraft. The program calculates aircraft performance in 
! accordance with the airworthiness standards of the Federal Aviation 
! Regulations. The aircraft and flight constraints are represented in sufficient 
! detail to permit realistic sensitivity studies in terms of either 
! configuration modifications or changes in operational procedures.

! This program provides for the detailed performance analysis of the takeoff and 
! landing capabilities of specific aircraft designs and allows for sensitivity 
! studies. The program is not designed to synthesize configurations or to 
! generate aerodynamic, propulsion, or structural characteristics. This type of 
! information must be generated externally to the program and then input as 
! data. The program's representation of the aircraft data is extensive and 
! includes realistic limits on engine and aircraft operational boundaries and 
! maximum attainable lift coefficients. The takeoff and climbout flight path is 
! generated by a stepwise integration of the equation of motion. Special 
! features include options for nonstandard-day operation, for balanced field 
! length, for derated throttle to meet a given field length for off-loaded 
! aircraft, and for throttle cutback during climbout for community noise 
! alleviation. Advanced takeoff procedures for noise alleviation such as 
! programmed throttle and control flaps may be investigated with the program. 
! Approach profiles may incorporate advanced procedures such as two segment 
! approaches and decelerating approaches. The landing performance considers the 
! application of wheel brakes, spoilers, and thrust reversers.

!
!

! AUTHORS - Willard E. Foss, NASA Langley Research Center
!           Ralph L. Carmichael, Public Domain Aeronautical Software
!
! REVISION HISTORY
!   DATE  VERS RERSON  STATEMENT OF CHANGES
! xxNov76  0.1   RLC   Approach calcs. completed
! xxNov76  0.3   RLC   New aero. format, CD=F(CL,FLAP,G.E.)
! xxFeb78  0.5   WEF   Throttle schedule logic installed
! xxMar79  0.8   WEF   Flap schedule logic installed
! 31Dec79  1.0   WEF   Release to COSMIC as program LAR-13086
! xxMay82  1.1   WEF   Noise equations and engine param. removed.
! xxMay82  1.2   WEF   Thrust reversal added to refused t.o. calc.
! xxMay82  1.3   WEF   Ground effect option added, (vs.  h/b  squared )
! xxJun82  1.4   WEF   Gamma limit logic for liftoff to hobs.
! xxJul82  1.5   WEF   Option for 1st seg. alpha (=alfrot or alflof).
!   1996   1.6   RLC   Program acquired from COSMIC
! 01Dec08  1.7   RLC   Converted to Fortran 95 free format, updated
!                         obsolescent features, compiled with gfortran
! ???      1.71  RLC   Acquired and scanned NASA TM 80120 by Willard Foss 
! 01Jun09  1.8   RLC   Reorganized into modules
! 19Jul09  1.85  RLC   All calls to FTLUP2 now have LOC as a variable
! 24Jul09  1.9   RLC   Removed blank common and made module variables
! 05Aug09  1.95  RLC   All common variables to one module
! 28Aug09  1.96  RLC   Procedures in module TolProcedures in alphabetic order


!+
MODULE TolConstants
! ------------------------------------------------------------------------------
! PURPOSE - Collect a number of constants used in the takeoff and landing program
IMPLICIT NONE

  INTEGER,PARAMETER:: SP = SELECTED_REAL_KIND(6)   ! single precision
  INTEGER,PARAMETER:: DP = SELECTED_REAL_KIND(14)   ! double precision
  INTEGER,PARAMETER:: WP = SP   ! working precision

  REAL(WP),PARAMETER:: PI = 3.141592653589793238462643383279502884197_WP
  REAL(WP),PARAMETER:: HALFPI=PI/2, TWOPI=PI+PI
  REAL(WP),PARAMETER:: ZERO = 0.0_WP, HALF = 0.5_WP, ONE=1.0_WP, TWO=2.0_WP
  REAL(WP),PARAMETER:: DEG2RAD = PI/180,   RAD2DEG = 180/PI
  
  REAL(WP),PARAMETER:: FT2METERS = 0.3048              ! mult. ft. to get meters
  REAL(WP),PARAMETER:: SQFT2SQM = FT2METERS**2          ! mult sq.ft to get sq.m
  REAL(WP),PARAMETER:: KELVIN2RANKINE = 1.8            ! mult deg K to get deg R
  REAL(WP),PARAMETER:: PSF2NSM = 47.880258         ! mult lb/sq.ft to get N/sq.m
  REAL(WP),PARAMETER:: SCF2KCM = 515.379        ! mult slug/cu.ft to get kg/cu.m
  REAL(WP),PARAMETER:: LBS2NEWTONS = 4.44822       ! mult. pounds to get newtons
  REAL(WP),PARAMETER:: KNOTS2MPS = 0.51444              ! mult. knots to get m/s
  REAL(WP),PARAMETER:: TZERO = 288.15           ! temperature at sealevel, deg K
  REAL(WP),PARAMETER:: PZERO = 101325.0           ! pressure at sealevel, N/sq.m
  REAL(WP),PARAMETER:: RHOZERO = 1.2250           ! density at sealevel, kg/cu.m
  REAL(WP),PARAMETER:: ASOUNDZERO = 340.294    ! speed of sound at sealevel, m/s


  INTEGER,PARAMETER:: IN=1, OUT=2, DBG=3   ! input/output unit numbers
  
!-------------------------------------------------------------------------------

END Module TolConstants  




!+
MODULE TolNonCommonProcedures
! ------------------------------------------------------------------------------
! PURPOSE - Collect all of the procedures that do not use COMMON variables
!  form the field length program WEF1041

!-------------------------------------------------------------------------------

CONTAINS



!+
SUBROUTINE AT76SP(ZFT,ANS,DTEMP,PBTAB)
! ------------------------------------------------------------------------------
! PURPOSE -  Determine atmosphere properties. Units are as desired by WEF1041

  REAL,INTENT(IN):: zft
  REAL,INTENT(OUT),DIMENSION(5):: ans ! ans(1)=density, slugs/cu.ft.
                                      ! ans(2)=pressure, psf
                                      ! ans(3)=temperature, kelvins
                                      ! ans(4)=speed of sound, ft/s
                                      ! ans(5)=(unit Reynolds)/Mach (1/ft)
                                      !       = speed of sound/kinematic viscosity
  REAL,INTENT(IN):: dtemp             ! temperature increment (hot/cold day) kelvins
  REAL,INTENT(IN),DIMENSION(8):: pbtab  ! pressure (Pascals) at atmosphere layers
                                        ! not used

  REAL,PARAMETER:: FT2M = 0.3048, FT2KM = FT2M/1000
  REAL,PARAMETER:: SLUG2KG = 14.5939029
  REAL,PARAMETER:: SLUGCUFT2KGCUM = SLUG2KG/FT2M**3
  REAL,PARAMETER:: LBSQFT2NSQM = 47.880258
  
! z=0 values of temp,pressure,density (SI)  kelvins,pascals,kg/m**3
  REAL,PARAMETER:: TZERO=288.15, PZERO=101325.0, RHOZERO=1.225
! speed of sound at sea level in ft/s
  REAL,PARAMETER:: CZERO=340.294/FT2M 

  REAL:: mu
  REAL:: sigma,delta,theta
  REAL:: z
!-------------------------------------------------------------------------------
  z=FT2KM*zft   ! z in kilometers
  ans=1.0
  CALL Atmosphere(z,sigma,delta,theta,dtemp)
  mu=Viscosity(theta)
  ans(1)=RHOZERO*sigma/SLUGCUFT2KGCUM
  ans(2)=PZERO*delta/LBSQFT2NSQM
  ans(3)=TZERO*theta
  ans(4)=CZERO*SQRT(theta)
  ans(5)=RHOZERO*sigma*ans(4)/mu

  RETURN
END Subroutine At76SP   ! ------------------------------------------------------



!+
SUBROUTINE Atmosphere(alt, sigma, delta, theta, deltaT)
!   -------------------------------------------------------------------------
! PURPOSE - Compute the properties of the 1976 standard atmosphere to 86 km.
!  allowing for a user-specified increment to the temperature to account for
!  weather variations (hot day, cold day, etc.)

  IMPLICIT NONE
  REAL,INTENT(IN)::  alt    ! geometric altitude, km.                        
  REAL,INTENT(OUT):: sigma  ! density/sea-level standard density              
  REAL,INTENT(OUT):: delta  ! pressure/sea-level standard pressure           
  REAL,INTENT(OUT):: theta  ! temperature/sea-level standard temperature
  REAL,INTENT(IN),OPTIONAL:: deltaT ! temp increment, kelvins

  REAL,PARAMETER:: REARTH = 6378.0                 ! radius of the Earth (km)
  REAL,PARAMETER:: GMR = 34.163195         ! hydrostatic constant, kelvins/km
  INTEGER,PARAMETER:: NTAB=8       ! number of entries in the defining tables

  INTEGER:: i,j,k                                                  ! counters
  REAL:: h                                       ! geopotential altitude (km)
  REAL:: tgrad, tbase      ! temperature gradient and base temp of this layer
  REAL:: tlocal                                           ! local temperature
  REAL:: deltah                             ! height above base of this layer

  REAL,DIMENSION(NTAB),PARAMETER:: htab= &   ! kilometers
                          (/0.0, 11.0, 20.0, 32.0, 47.0, 51.0, 71.0, 84.852/)
  REAL,DIMENSION(NTAB),PARAMETER:: ttab= &   ! kelvins
          (/288.15, 216.65, 216.65, 228.65, 270.65, 270.65, 214.65, 186.946/)
  REAL,DIMENSION(NTAB),PARAMETER:: ptab= &   ! non-dimensional
               (/1.0, 2.233611E-1, 5.403295E-2, 8.5666784E-3, 1.0945601E-3, &
                                     6.6063531E-4, 3.9046834E-5, 3.68501E-6/)
  REAL,DIMENSION(NTAB),PARAMETER:: gtab= &   ! kelvins/km
                                (/-6.5, 0.0, 1.0, 2.8, 0.0, -2.8, -2.0, 0.0/)
!----------------------------------------------------------------------------
  h=alt*REARTH/(alt+REARTH)      ! convert geometric to geopotential altitude

  i=1 
  j=NTAB                                  ! setting up for binary search
  DO
    k=(i+j)/2                                              ! integer division
    IF (h < htab(k)) THEN
      j=k
    ELSE
      i=k
    END IF   
    IF (j <= i+1) EXIT
  END DO

  tgrad=gtab(i)                                     ! i will be in 1...NTAB-1
  tbase=ttab(i)
  deltah=h-htab(i)
  tlocal=tbase+tgrad*deltah
  theta=tlocal/ttab(1)                                    ! temperature ratio

  IF (tgrad == 0.0) THEN                                     ! pressure ratio
    delta=ptab(i)*EXP(-GMR*deltah/tbase)
  ELSE
    delta=ptab(i)*(tbase/tlocal)**(GMR/tgrad)
  END IF
  
  IF (Present(deltaT)) THEN
    theta=(tlocal+deltaT)/ttab(1)                     ! temperature increment
  END IF

  sigma=delta/theta                                           ! density ratio
  RETURN
END Subroutine Atmosphere   ! -----------------------------------------------




!+
FUNCTION GetDateTimeStr() RESULT (s)
!   ---------------------------------------------------------------------------
! PURPOSE - Return a string with the current date and time
  IMPLICIT NONE
  CHARACTER(LEN=*),PARAMETER:: MONTH='JanFebMarAprMayJunJulAugSepOctNovDec'
  CHARACTER(LEN=*),PARAMETER:: FMT='(I2.2,A1,I2.2,I3,A3,I4)'
  CHARACTER(LEN=15):: s
  INTEGER,DIMENSION(8):: v
  INTRINSIC:: DATE_AND_TIME
!------------------------------------------------------------------------------
  CALL DATE_AND_TIME(VALUES=v)
  WRITE(s,FMT) v(5), ':', v(6), v(3), MONTH(3*v(2)-2:3*v(2)), v(1)
  RETURN
END FUNCTION GetDateTimeStr   ! ===============================================

!+
      SUBROUTINE GREFCT(NTOP,J,AR,SPAN,ALFMIN,CLMIN,CDMIN,HG,TALPHTO,   &
     & TCLTO,TCDTO,TALPTOG,TCLTOG,TCDTOG,CLMING)
! ------------------------------------------------------------------------------
! PURPOSE - Ground effect calculations
IMPLICIT NONE
!
!  AS OF 12/78, BEING USED TO CALC. FULL G.E.POLARS, DEYOUNG/AST
  INTEGER,INTENT(IN):: ntop
  INTEGER,INTENT(IN):: j
  REAL,INTENT(IN):: ar ! aspect ratio
  REAL,INTENT(IN):: span  ! wing span
  REAL,INTENT(IN):: alfmin,clmin,cdmin
  REAL,INTENT(IN):: hg
  REAL,INTENT(IN),DIMENSION(15,4):: talphto,tclto,tcdto
  REAL,INTENT(OUT),DIMENSION(15,4):: talptog,tcltog,tcdtog
  REAL,INTENT(OUT):: clming

!!!      DIMENSION TALPHTO(15,4),TCLTO(15,4),TCDTO(15,4),TALPTOG(15,4),    &
!!!     & TCLTOG(15,4),TCDTOG(15,4)

  INTEGER:: i
  REAL:: rad,a,hf,b,dnom,clgocl,phi,alf,d,cl,cd1,cd2,cdgocd
!-------------------------------------------------------------------------------
      rad=3.1416/180.
      a=(6.+ar)**2/(36.+ar)
      hf=hg/span
      b=32.*(hf*a)**2.+1.
      dnom=b-.5+4.*hf*a*b**.5
      clgocl=1.+1./dnom
      b=1.+32.*hf**2.
      dnom=4.*hf*b**.5+b
      phi=1.-1./dnom

      DO i=1,ntop
      alf=talphto(i,j)
      talptog(i,j)=alf
      d=(alf-alfmin)*rad
      cl=tclto(i,j)
      cd1=cdmin+d*(cl-clmin)
      cd2=cdmin+clgocl*phi*(cd1-cdmin)+d*cl*(clgocl-1.)
      cdgocd=cd2/cd1
      tcdtog(i,j)=cdgocd*tcdto(i,j)
      tcltog(i,j)=clgocl*tclto(i,j)
   END DO

      clming=clmin*clgocl

      RETURN
END Subroutine Grefct   ! ------------------------------------------------------


!+
      SUBROUTINE AT62SP(ZFT,ANS,DTEMP,PBTAB)
! ------------------------------------------------------------------------------
! PURPOSE  DETERMINE ATMOSPHERE PROPERTIES

  REAL,INTENT(IN):: zft
  REAL,INTENT(OUT),DIMENSION(5):: ans 
  REAL,INTENT(IN):: dtemp
  REAL,INTENT(IN),DIMENSION(8):: pbtab
  
      DIMENSION HMTAB(9),HBTAB(8),ELMTAB(8),TMBTAB(8)
!!!      DIMENSION PBTAB(8),ZDPR(1),HDPR(1)
!!!      REAL,DIMENSION(8):: pbtab
      REAL:: zdpr,hdpr
 1112 DATA(HMTAB(I),I=1,9),(HBTAB(I),I=1,8),(ELMTAB(I),I=1,8),          &
     &(TMBTAB(I),I=1,8)/                                                &
     &-99999999.,11000.,20000.,32000.,47000.,52000.,61000.,             &
     &79000.,99999999.,                                                 &
     &0.,11000.,20000.,32000.,47000.,52000.,61000.,79000.,              &
     &-.0065,.0,.001,.0028,.0,-.002,-.004,.0,                           &
     &288.15,216.65,216.65,228.65,270.65,270.65,252.65,180.65/
      EMOR=.00348368
      GAM=1.4
      GO=9.80665
      CONV1=.3048
      CONV2=.45359237
!-------------------------------------------------------------------------------      
!     CONVERT Z FT. TO Z METERS
    7 Z   =ZFT   *CONV1
 1111 ZDPR=Z
      Z6=ZDPR*.000001
  210 Z6P=Z6+6.3675708
  211 TEMP2=.0010559179*Z6+1.5640943
  212 TEM2=Z6P*Z6P
  213 TEM3=TEM2*Z6P
  214 TEM4=TEM2*TEM2
  215 TEM5=TEM2*TEM3
  216 TEM9=TEM4*TEM5
      SINZ=SIN(.00052795893*Z6+.78204713)
  217 HDPR   =-40648635./(Z6P)-894899.28/TEM3-16302410./TEM9-7408.3724  &
     & /TEM4-135.55760*Z6*Z6-1724.8490 *Z6+(2684697.8/TEM3)*(SINZ*SINZ) &
     & +(708.70509/TEM2-256757.85 *Z6-1634923.8+.00079017959*ALOG(zdpr  &
     &  +6367570.8))*( SIN(TEMP2))+(.74833436/(Z6P)-243160820.)*        &
     & ( COS(TEMP2))+9646530.7284232
      H=HDPR
   16 IF (Z   -90000.0)70,70,500
!     Z LESS THAN 90000.M FIND H IN HMTAB TABLE
   70 N=1
  702 IF (H-HMTAB(N))703,701,700
  700 N=N+1
      GO TO 702
  701 J=N
      GO TO 704
  703 J=N-1
!     FIND HB,PB,ELM,TMB,TM
  704 HB=HBTAB(J)
      PB=PBTAB(J)
      ELM=ELMTAB(J)
      TMB=TMBTAB(J)+DTEMP
      TM=TMB+ELM*(H-HB)
!        TM IN KELVIN
      IF (ABS(PBTAB(2)-22632.029).GE.0.002) GO TO 248
  247 DTSAVE=TMB-TMBTAB(J)
      TM=TM-DTSAVE
      TMB=TMB-DTSAVE
  248 IF (ELM)72,73,72
!     IF ELM NOT =0 COMPUTE P
   72 P   =EXP(ALOG(PB)-(GO*EMOR/ELM)*ALOG(TM/TMB))
  249 GO TO 89
!     IF ELM=0 COMPUTE P
   73 P=EXP(ALOG(PB)-(GO*EMOR)*((H-HB)/TMB))
!     CONVERT P  NT/M2 TO PSF
   89 PRESS=P/47.880183
!     T DEGREES KELVIN
      IF (ABS(PBTAB(2)-22632.029).GE.0.002) GO TO 266
      TM=TM+DTSAVE
  266 T=TM
!     RHO KG/M3
  269 RHO   =EMOR*(P   /TM   )
!     CS M/SEC
   18 CS   =SQRT(GAM*TM   /EMOR)
!     CONVERT RHO KG/M3 TO LB/FT3
  291 RHO   =RHO   *CONV1*CONV1*CONV1/CONV2
!     CS FT/SEC
   22 CS   =CS   /CONV1
!     VISCOS IN LB/FT-SEC
      CONST=(CONV1/CONV2)*1.458/10.**6.
      VISCOS=CONST*T**1.5/(T+110.4)
!     RNOML IN RN/MACH-FT
      RNOML=RHO*CS/VISCOS
!     CONVERT RHO LB/FT3 TO SLUG/FT3
      RHO   =RHO   /.321740485E+02
    5 ANS(1)=RHO
      ANS(2)=PRESS
      ANS(3)=T
      ANS(4)=CS
      ANS(5)=RNOML
      H=ZFT
      RETURN
  500 DO 600 JJ=1,5
      ANS(JJ)=0.0
  600 END DO
  
      RETURN
END Subroutine At62SP   ! ------------------------------------------------------

!+
SUBROUTINE Ftlup2(x,y,m,n,vari,vard,loc)
! ------------------------------------------------------------------------------
! PURPOSE  -  1-D Table Interpolation;     Modified version of old FTLUP.
! NOTE - Called by Wef1041,Aero41,Approach,Qant4,Refused
IMPLICIT NONE
  REAL,INTENT(IN):: x 
  REAL,INTENT(OUT):: y
  INTEGER,INTENT(IN):: m,n 
  REAL,INTENT(IN),DIMENSION(:):: vari,vard
  INTEGER,INTENT(IN OUT):: loc 
  
  INTEGER:: i,if, iyy,j,k
  INTEGER:: npt
  REAL,DIMENSION(3):: v 
  REAL,DIMENSION(2):: yy 
!-------------------------------------------------------------------------------   
!!!      DIMENSION VARI(1),VARD(1),V(3),YY(2)

      IF (LOC.EQ.0) LOC=99  
      NPT=N
      IF (M.EQ.0.AND.N.EQ.0) GO TO 1
      IF (M.EQ.0.AND.N.NE.0)GO TO 97
      IF (N.LE.IABS(M)) GO TO 97
      IF (M.GT.0)GO TO 31
      
      
!             M.LT.0
      DO 44 IYY=1,N
      I=IYY
      IF (vari(i) < x) GO TO 800
      IF (vari(i)==x) GO TO 119
!!!      IF (VARI(I)-X)800,119,44
   44 END DO
      I=N+M
!          IF X.LT.X(N),EXTRAPOLATE
      IF (M.EQ.-1)GO TO 801
      GO TO 802
!          IF X.GT.X(1),EXTRAPOLATE
  800 IF (I.EQ.1.AND.M.EQ.-1)GO TO 801
      IF (I.EQ.1.AND.M.EQ.-2)GO TO 802
      IF (M.NE.-1)GO TO 622
!          M=-1
      I=I-1
  801 IF (VARI(I).LE.VARI(I+1)) GO TO 97
      GO TO 1701
!          M=-2
  622 IF (I.NE.N)GO TO 1622
      I=N-2
      GO TO 802
!          COMPARE WITH NEXT
 1622 IF (VARI(I+1)-X)803,97,97
  803 I=I-1
      IF (I.EQ.1)GO TO 802
!          SEE WHICH THREE
      IF ((VARI(I-1)-X).LT.(X-VARI(I+2)))I=I-1
  802 IF (VARI(I).LE.VARI(I+1).OR.VARI(I+1).LE.VARI(I+2))GO TO 97
      GO TO 1702
!             M.GT.0
   31 DO 4 IYY=1,N
      I=IYY
!!!      IF (X-VARI(I))700,119,4
        IF (x < vari(i)) GO TO 700
        IF (x==vari(i)) GO TO 119
    4 END DO
      I=N-M
!          IF X.GT.X(N),EXTRAPOLATE
      IF (M.EQ.1)GO TO 701
      GO TO 702
!          IF X.LT.X(1),EXTRAPOLATE
  700 IF (I.EQ.1.AND.M.EQ.1)GO TO 701
      IF (I.EQ.1.AND.M.EQ.2)GO TO 702
      IF (M.NE.1)GO TO 222
!          M=1
      I=I-1
  701 IF (VARI(I+1).LE.VARI(I))GO TO 97
!          LINEAR
 1701 Y=(VARD(I)*(VARI(I+1)-X)-VARD(I+1)*(VARI(I)-X))/(VARI(I+1)-VARI(I))
      RETURN
!          M=2
  222 IF (I.NE.N)GO TO 1222
      I=N-2
      GO TO 702
!          COMPARE WITH NEXT
 1222 IF (X-VARI(I+1))703,97,97
  703 I=I-1
      IF (I.EQ.1)GO TO 702
!          SEE WHICH THREE
      IF ((X-VARI(I-1)).LT.(VARI(I+2)-X))I=I-1
  702 IF (VARI(I+1).LE.VARI(I).OR.VARI(I+2).LE.VARI(I+1))GO TO 97

!          SECOND ORDER
 1702 V(1)=VARI(I)-X
      V(2)=VARI(I+1)-X
      V(3)=VARI(I+2)-X
      K=I
      DO 704 J=1,2
      YY(J)=(VARD(K)*V(J+1)-VARD(K+1)*V(J))/(VARI(K+1)-VARI(K))
  704 K=K+1
      Y=(YY(1)*V(3)-YY(2)*V(1))/(VARI(I+2)-VARI(I))
      RETURN

    1 Y =VARD(1)   !             ZERO ORDER(Y=Y(1))
      RETURN

  119 Y=VARD(I)   !             Y=Y(I)
      RETURN
!
!          ERROR PRINT  (FATAL ERROR, NO RETURN)
   97 PRINT 103,LOC
  103 FORMAT(/" ERROR IN FTLUP2 FOR LOC=",I5)
      PRINT 104, M,N,X,(VARI(IF),IF=1,NPT)
      PRINT 105, (VARD(IF),IF=1,NPT)
  104 FORMAT(2X,"*****  M=",I5,2X,"N=",I5,2X,"X=",E18.6/                   &
     &5X,"VARI(ALL)=",5E18.6/(15X,5E18.6/))
  105 FORMAT(5X,"VARD(ALL)=",5E18.6/(15X,5E18.6/))
      IF (N==0) STOP
      IF (M==0) STOP
      WRITE(6,*) "TABLE OUT OF ORDER"
      WRITE(*,*) "TABLE OUT OF ORDER"
      IF (LOC==99) STOP
      LOC=1000
      RETURN
END Subroutine FtLup2   !-------------------------------------------------------

!+
      SUBROUTINE Bilupv(nia,nja,tabi,tabj,tabij,tacij,ni,nj,vali,valj,  &
     &bval1,cval1,bval2,valimx,valimn)
! ------------------------------------------------------------------------------     
! PURPOSE - A two dimensional table look-up for two variables. The input tables 
!  are - tabij(i,j) and tacij(i,j) as functions of tabi(i) and tabj(j).  
!  The two dependent variables are linearly interpolated simultaneously for 
!  input values of vali and valj resulting in answers bval1 and cval1.  
!  tabij(i,j) is also done on a square basis with respect to tabi(i), 
!  resulting in bval2.  Error signals are generated when the tabj(j) table is 
!  extrapolated. The minimum and maximum limits of tabi are also interpolated
!  resulting in valimn and valimx.
! NOTE - Called by Wef1041,Aero41,Cutback,Qant4
IMPLICIT NONE

  INTEGER,INTENT(IN):: nia,nja
  REAL,INTENT(IN),DIMENSION(nia,nja):: tabi 
  REAL,INTENT(IN),DIMENSION(nja):: tabj
  REAL,INTENT(IN),DIMENSION(nia,nja):: tabij,tacij
  INTEGER,INTENT(IN):: ni,nj 
  REAL,INTENT(IN):: vali,valj
  REAL,INTENT(OUT):: bval1,cval1
  REAL,INTENT(OUT):: bval2
  REAL,INTENT(OUT):: valimx,valimn
!!!      DIMENSION TABJ(NJA),TABI(NIA,NJA),TABIJ(NIA,NJA),TACIJ(NIA,NJA)
!!!      DIMENSION TBISL(2),TBI2SL(2),TBIJ1(2),TCIJ1(2),TBIJ2(2),TBIMX(2)
!!!      DIMENSION TBIMN(2)
      
  REAL,DIMENSION(2):: tbisl,tbi2sl,tbij1,tcij1,tbij2,tbimx,tbimn      
  
  INTEGER:: i,j 
  INTEGER:: ii, isave1,isave2, kk, l, lim
  REAL:: tabcl, tbjsl
!-------------------------------------------------------------------------------      
      TBJSL=0.0
      KK=2
      IF (TABJ(1).LT.TABJ(2)) GO TO 1
      DO 10 J=1,NJ
!!!      IF (VALJ-TABJ(J)) 10,9,11
        IF (valj > tabj(j)) GO TO 11
        IF (valj==tabj(j)) THEN
          tbjsl=1.0
          GO TO 300
        END IF 
!!!    9 TBJSL=1.0
!!!      GO TO 300
   10 END DO
      J=NJ
!     WRITE(6,100)
! 100 FORMAT(/20X,"HIGH J EXTRAPOLATION")
   11 IF (J.GT.1) GO TO 300
      J=2
!     WRITE(6,101)
! 101 FORMAT(/20X,"LOW J EXTRAPOLATION")
      GO TO 300
      
    1 DO 2 J=1,NJ
!!!      IF (VALJ-TABJ(J)) 3,4,2
!!!    4 TBJSL=1.0
!!!      GO TO 300
        IF (valj < tabj(j)) GO TO 3
        IF (valj==tabj(j)) THEN
          tbjsl=1.0
          GO TO 300
        END IF
    2 END DO
      J=NJ
!     WRITE(6,100)
    3 IF (J.GT.1) GO TO 300
!     WRITE(6,101)
      J=2
      
  300 LIM=1
      DO 320 I=1,NI
      IF (LIM.GT.1) GO TO 314
      IF (TABIJ(I,J).EQ.0.0) CYCLE   !!! GO TO 320
      ISAVE1=I+1
      LIM=2
      CYCLE !!!GO TO 320
  314 IF (TABIJ(I,J).GT.0.0) CYCLE !!!GO TO 320
      ISAVE2=I-1
      GO TO 12
  320 END DO
  
      ISAVE2=NI
   12 IF (TABI(ISAVE1-1,J).GT.TABI(ISAVE1,J)) GO TO 5
      IF (VALI.LT.TABI(ISAVE1-1,J)) GO TO 21
      IF (VALI.GT.TABI(ISAVE2,J)) GO TO 24
      II=ISAVE1-1
      DO 20 I=II,ISAVE2
!!!      IF (VALI-TABI(I,J)) 22,28,20
        IF (vali < tabi(i,j)) GO TO 22
        IF (vali==tabi(i,j)) GO TO 28
   20 END DO
   
   21 I=ISAVE1
      GO TO 22
   24 I=ISAVE2
      GO TO 22
    5 IF (VALI.GT.TABI(ISAVE1-1,J)) GO TO 6
      IF (VALI.LT.TABI(ISAVE2,J)) GO TO 7
      II=ISAVE1-1
      DO 8 I=II,ISAVE2
!!!      IF (VALI-TABI(I,J)) 8,28,22
        IF (vali > tabi(i,j)) GO TO 22
        IF (vali==tabi(i,j)) GO TO 28
    8 END DO
    6 I=ISAVE1
      GO TO 22
    7 I=ISAVE2
      GO TO 22
   28 TBISL(KK)=0.0
      TBI2SL(KK)=0.0
      TBIJ1(KK)=TABIJ(I,J)
      TCIJ1(KK)=TACIJ(I,J)
      TBIJ2(KK)=TABIJ(I,J)
      GO TO 29
!        STATEMENTS 22 TO 33 DETERMINE CL FOR MINIMUM CD.
   22 IF (TABIJ(ISAVE1,J).LT.TABIJ(ISAVE1-1,J)) GO TO 30
!     TABCL=0.     CHANGED 4-77
      TABCL=TABI(ISAVE1-1,J)
      GO TO 33
   30 DO 31 L=ISAVE1,ISAVE2
      IF (TABIJ(L,J).LT.TABIJ(L-1,J)) CYCLE   !!! GO TO 31
      GO TO 32
   31 END DO
      L=ISAVE2
   32 TABCL=TABI(L-1,J)
!        STATEMENTS 33 TO 29 DETERMINE SLOPES WITH RESPECT TO I
   33 TBISL(KK)=(VALI-TABI(I-1,J))/(TABI(I,J)-TABI(I-1,J))
      TBI2SL(KK)=((VALI-TABCL)**2-(TABI(I-1,J)-TABCL)**2)/((TABI(I,J)-  &
     &TABCL)**2-(TABI(I-1,J)-TABCL)**2)
      TBIJ1(KK)=TBISL(KK)*(TABIJ(I,J)-TABIJ(I-1,J))+TABIJ(I-1,J)
      TCIJ1(KK)=TBISL(KK)*(TACIJ(I,J)-TACIJ(I-1,J))+TACIJ(I-1,J)
      TBIJ2(KK)=TBI2SL(KK)*(TABIJ(I,J)-TABIJ(I-1,J))+TABIJ(I-1,J)
   29 TBIMX(KK)=TABI(ISAVE2,J)
      TBIMN(KK)=TABI(ISAVE1-1,J)
      IF (TBJSL.EQ.0.0) GO TO 26
      BVAL1=TBIJ1(KK)
      CVAL1=TCIJ1(KK)
      BVAL2=TBIJ2(KK)
      VALIMX=TBIMX(KK)
      VALIMN=TBIMN(KK)
      GO TO 25
   26 IF (KK.EQ.1) GO TO 23
      KK=KK-1
      J=J-1
      GO TO 300
!        STATEMENTS 23 TO 25 COMPUTE FINAL INTERPOLATED VALUES.
   23 J=J+1
      TBJSL=(VALJ-TABJ(J-1))/(TABJ(J)-TABJ(J-1))
      BVAL1=TBJSL*(TBIJ1(2)-TBIJ1(1))+TBIJ1(1)
      CVAL1=TBJSL*(TCIJ1(2)-TCIJ1(1))+TCIJ1(1)
      BVAL2=TBJSL*(TBIJ2(2)-TBIJ2(1))+TBIJ2(1)
      VALIMX=TBJSL*(TBIMX(2)-TBIMX(1))+TBIMX(1)
      VALIMN=TBJSL*(TBIMN(2)-TBIMN(1))+TBIMN(1)
      
   25 RETURN
END Subroutine Bilupv   ! ------------------------------------------------------


!+
PURE FUNCTION Viscosity(theta) RESULT(visc)
! ------------------------------------------------------------------------------
! PURPOSE - Compute viscosity using Sutherland's formula. Units are kg/(m s)
!   
  IMPLICIT NONE
  REAL,INTENT(IN) :: theta                ! temperature/sea-level temperature  

  REAL:: visc   ! Returns viscosity in kg/(meter-sec)
  
  REAL,PARAMETER:: BETAVISC = 1.458E-6 ! viscosity term, N sec/(sq.m sqrt(K)
  REAL,PARAMETER:: SUTH = 110.4              ! Sutherland's constant, kelvins
  REAL,PARAMETER:: TZERO = 288.15

  REAL:: temp                              ! temperature in kelvins
  INTRINSIC:: SQRT
!----------------------------------------------------------------------------
  temp=TZERO*theta
  visc=BETAVISC*SQRT(temp*temp*temp)/(temp+SUTH)
  RETURN
END Function Viscosity   ! -----------------------------------------------------


END Module TolNonCommonProcedures   ! ==========================================



!+
MODULE TolCommonVariables
! ------------------------------------------------------------------------------
! PURPOSE -
! NOTE - Variables followed by !* are in namelist NAM1
USE TolConstants,ONLY: WP
IMPLICIT NONE


  REAL(WP),DIMENSION(15,4):: ACDTO,ACDTOG
  REAL(WP):: ALFROLL   !* angle of attack during ground roll, degrees
  REAL(WP):: ALFROT    !* max angle of attack during rotation, degrees
  REAL(WP),DIMENSION(200):: ALPHA
  REAL(WP):: AMACH
  REAL(WP):: ANOE   !*
  REAL(WP),DIMENSION(5):: ANS
  REAL(WP):: ARALF1   !*
  REAL(WP):: ARALF2   !*
  REAL(WP):: ARFLP1   !*
  REAL(WP):: ARFLP2   !*
  REAL(WP):: ARVEL1   !*
  REAL(WP):: ARVEL2   !*
  REAL(WP),DIMENSION(15):: ASPARE,BSPARE,CSPARE   !*
  REAL(WP):: ATRDT = 0.0    !*
  REAL(WP):: ATRTOT = 0.0   !*
  REAL(WP),DIMENSION(20):: CA   !*
  REAL(WP):: CBALT    !*
  REAL(WP):: CBDIST   !*
  REAL(WP):: CBGRAD   !*
  REAL(WP):: CBLIMH = 0.0  !*
  REAL(WP),DIMENSION(200):: CDD
  REAL(WP):: CDGEAR   !*  not used ???
  REAL(WP),DIMENSION(15):: CDGRT
  REAL(WP),DIMENSION(5):: CDOUTT   !*
  REAL(WP),DIMENSION(20):: CE   !*
  REAL(WP):: CENTRI
  REAL(WP),DIMENSION(200):: CL
  REAL(WP),DIMENSION(15):: CLGRT   !*
  REAL(WP):: CLOFLP   !*
  REAL(WP),PARAMETER:: CONV1=6076.0, CONV2=3600.0, CONV3=CONV1/CONV2
  REAL(WP):: DCDS   !*
  REAL(WP):: DCLS   !*
  REAL(WP),DIMENSION(5):: DELCD
  REAL(WP):: DELLOD = 0.0  !* arbitrary increment to L/D in addition to tables
  REAL(WP):: DELTA   !*
  REAL(WP):: DESTOFL   !*
  REAL(WP),DIMENSION(200):: DIST
  REAL(WP):: DOUT1   !*
  REAL(WP):: DOUT2   !*
  REAL(WP),DIMENSION(200):: DRAG
  REAL(WP):: DRFAC
  REAL(WP),DIMENSION(5):: DRFACT   !*
  REAL(WP):: drone   ! ram drag of ONE sized engine
  REAL(WP),DIMENSION(5,5):: DROPT1
  REAL(WP):: DTBRA   !*
  REAL(WP):: DTEMP   !*
  REAL(WP):: DTFAIL   !*
  REAL(WP):: DTFAR = 2.0    !*
  REAL(WP):: DTFTD   !*
  REAL(WP):: DTGRUP   !*
  REAL(WP):: DTPOW   !*
  REAL(WP):: DTREC   !*
  REAL(WP):: DTSPO   !*
  REAL(WP):: DTTR   !*
  REAL(WP):: DVCLIMB   !*
  REAL(WP):: DWTAXI = 0.0 !*
  INTEGER:: ENGCODE   !*
  REAL(WP):: ENGSOUT   !*
  REAL(WP):: ESF = 0.0   !*
  REAL(WP),DIMENSION(12):: EVENT(12)
  REAL(WP):: FLAP
  REAL(WP),DIMENSION(25):: FLAPT   !*
  REAL(WP):: FPLIMH = 400.0  !*  US  
  REAL(WP),DIMENSION(25):: FPVAR   !*
  REAL(WP),DIMENSION(5,5):: FUEL1   !* 2-D array of fuel flow values
  REAL(WP):: GAMLIM = 50.0   !*
  REAL(WP),DIMENSION(200):: GAMMA
  REAL(WP):: GLIDE1   !*
  REAL(WP):: GLIDE2   !*
  REAL(WP):: GRAD1,GRAD2   !*
  REAL(WP):: GZERO   !*
  REAL(WP),DIMENSION(200):: H
  REAL(WP),DIMENSION(5,5):: HENGT   !* array of altitude values
  REAL(WP):: HNOGE = 0.0   !* if height > hnoge, there is no ground effect
  REAL(WP):: HOBS   !*
  REAL(WP):: HOBSL   !*
  REAL(WP):: HSTEP1,HSTEP2   !*
  REAL(WP),DIMENSION(5):: HTAB1   !*
  INTEGER:: IBAL = 0  !* =0, STANDARD TAKEOFF
                      !  =1, REFUSED T.O. DIST. FOR INPUT VFAILKT
                      !  =2, AUTO. SEARCH FOR CRITICAL VFAIL POINT
                      !  NOTE 11-30-77  IBAL=1 CALCS. MADE, BUT RESULTS NOT IN SUMMARY.
                      !  DO NOT USE WITH HSTEP1=5000.
  INTEGER:: IBUG   !*!
!      IBUG=0,  NO DEBUG, 1, AND 2 FOR INCREASED DEBUG LISTINGS.
!          FOR OTHER DEBUG OPTIONS SEE ISPARE(5,6,7,8).

  INTEGER:: ICODE  !!      ICODE - CODE FOR ANGLE OF ATTACK CHANGES
!        SET=1, DURING GROUND ROLL BEFORE REACHING VR
!        SET=2, AT VR AND DURING ROTATION TO ALFROT
!        SET=3, AT ALFROT (OR ALFLOF) AND DURING CLIMBOUT
!        SET=4, AT VCLIMB AND BEYOND
!        SET=9, DURING APPROACH CALCULATIONS
!        SET=10, DURING LANDING CALCULATIONS
!
  INTEGER:: ILAND = 0  !*
  !
!      ILAND=0, NO LANDING CALC.         NOTE%SEGMENT INPUTS CONTROL
!      ILAND=1, LANDING CALC. AFTER T.O.     EITHER APPROACH ONLY OR
!      ILAND=2, LANDING CALC. ONLY           FULL LANDING DIST. CALC
!

  INTEGER:: INAM = 0   !*
  INTEGER:: IPRINT = 1   !* !      IPRINT=0, MIN. PRINTOUT, 1,2, AND 3 INCREASE OUTPUT LISTINGS.
  INTEGER:: IRCDGR = 0  !* =0 to compute fraction gear drag
                        !  =1 to use full CDGEAR throughout gear retraction
  INTEGER:: IROT = 0   !* =0,  RUN WITH 1ST VROTKT VALUE ONLY.
                       !  =1,  RUN ALL INPUT VROTKT VALUES.
                       !  =2,  RUN 1ST VROTKT VALUE PLUS +5, +10, +15, +20 KTS.
  INTEGER:: ISEARCH = 0  !*  =0, STANDARD TAKEOFF LOGIC.
!      ISEARCH=1, AUTO. SEARCH FOR REQD. THROTTLE FOR DESTOFL.
!      ISEARCH=10, NO T.O. CALC., INTERP. AND LIST SELECTED ENGINE DATA

  INTEGER:: ISIN = 1   ! unit selector. =1 for US Customary units; =2 for SI units
                       ! input, but not in namelist. Why ???

  INTEGER:: ISOT = 1  !* =1,  ALL MAJOR OUTPUTS IN FPS UNITS; =2 for SI units

  INTEGER,DIMENSION(15):: ISPARE   !*
  INTEGER:: ISTP = 1  !*
  INTEGER:: ITER = 0   !*
  INTEGER:: JCODE ! CODE FOR THRUST VARIATIONS (not input, set in program)
    ! =1, DURING GROUND ROLL, ALL ENGINES OPERATING
    ! =2, AT VFAIL POINT, FOR ENGINE-OUT TAKEOFF
    ! =3, AT (VFAIL TIME + DTFAIL), ENGINE-OUT TAKEOFF
    ! =4, AT VFAIL POINT, FOR ENGINE-OUT REFUSED TAKEOFF
    ! =5, AT START OF DECEL. GROUND ROLL
    ! =6, FOR ENGINE-OUT CLIMB GRADIENT CALC. 
    ! =7, FOR CUTBACK LOGIC DURING CLIMBOUT
    ! =10, DURING REFUSED TAKEOFF IF CANNOT BALANCE FIELD

  INTEGER:: JTCODE = 0  !*
  INTEGER:: K
  INTEGER:: L
  INTEGER:: LCODE   !  CODE FOR LIFTOFF AND CLIMBOUT (not input, set in program)
    ! =1, DURING GROUND ROLL BEFORE REACHING VLOF
    ! =2, AT VLOF AND DURING CLIMBOUT TO HOBS
    ! =3, AT HOBS AND DURING CLIMBOUT
    ! =4, AT VCLIMB AND BEYOND

  INTEGER:: LF   !*  input, but never used. (???) load factor?
  REAL(WP),DIMENSION(200):: LIFT
  INTEGER:: LOC
  REAL(WP),DIMENSION(200):: LOD
  INTEGER:: M
  REAL(WP),DIMENSION(200):: MACH
  REAL(WP),DIMENSION(5):: MENGT   !* array of Mach values used for interpolation
  REAL(WP),DIMENSION(5):: MPENGT1   !*
  REAL(WP):: MUBRAKE   !* coefficient of froction with
  REAL(WP):: MUROLL = 0.0  !*
  REAL(WP),DIMENSION(200):: NACC
  INTEGER:: NCODE = 0 !* !      NCODE=0, NO NOISE CALCS.    OPTIONS REMOVED 5-82.
  INTEGER:: NEI   !* no. of rows in array HENGT of altitude values
  INTEGER:: NEJ   !* length of array of MENGT of Mach number values
  INTEGER:: NFD   !* number of columns used in arrays TCLTO,TCDTO,TALPHTO
  INTEGER:: NPI,NPJ   !*
  INTEGER:: NTOG   !* number of rows used in arrays TCLTOG,TCDTOG,TALPHTOG
  INTEGER:: NTOP   !* number of rows used in arrays TCLTO,TCDTO,TALPHTO
  REAL(WP),DIMENSION(10,5):: PARRAM1   !*
  REAL(WP):: PARREF
  REAL(WP),DIMENSION(10,5):: PARTH1   !*
  REAL(WP),DIMENSION(10,5):: PARWF1   !*
  REAL(WP),DIMENSION(8):: PBTAB = (/ 101325.0, 22632.029, 5474.872147, 868.0144315, &
                           110.9055258, 59.0004873, 18.20991199, 1.03769949E01 /)
                       !*
  REAL(WP):: PINT1=500.0, PINT2=500.0, PINT3=500.0   !* US
  REAL(WP),DIMENSION(25):: POWER   !*
  REAL(WP),DIMENSION(200):: Q
  REAL(WP),DIMENSION(5,5):: RAM1   !* 2-D array of ram drag values
  REAL(WP):: REFA1,REFB1   !*
  REAL(WP),PARAMETER:: REFRHO = 0.00237689  
  REAL(WP),DIMENSION(10,5):: RORTAB1
  REAL(WP):: ROTATE
  REAL(WP):: ROTRATE   !*
  REAL(WP):: RZERO   !*
  REAL(WP):: S = 0.0   !*
  
! CONVERSION FACTORS  
  REAL(WP):: S1=4.44822   ! pounds to Newtons
  REAL(WP):: S2=.09290304  ! square feet to square meters
  REAL(WP):: S3=.3048   ! feet to meters
  REAL(WP):: S4=.51444   ! knots to meters per second
  REAL(WP):: S5=1.0000   ! available
  REAL(WP):: S6=.04788026   ! pounds per square foot to kilopascals
  REAL(WP):: S7=.000126   ! pounds per hour to kilograms per second (fuel flow)
  REAL(WP):: S8=.4535924   ! pounds to kilograms (weight to mass, hm!)
  REAL(WP):: S9=.00064516   ! square inches to square meters
  REAL(WP):: S10=1.000   ! available
  
  REAL(WP),DIMENSION(5):: SFCFACT   !*
  REAL(WP),DIMENSION(10):: SPBFL
  REAL(WP),DIMENSION(10):: SPTFL
  REAL(WP),DIMENSION(10):: SPVR
  REAL(WP):: STOPDIS   !*
  REAL(WP),DIMENSION(200):: TACC
  REAL(WP),DIMENSION(15,4):: TCLTO,TCDTO,TALPHTO   !*
  REAL(WP),DIMENSION(15,4):: TCLTOG,TCDTOG,TALPTOG   !*
  REAL(WP):: TDIDLE   !*
  REAL(WP),DIMENSION(200):: TDR
  REAL(WP):: TFAC
  REAL(WP),DIMENSION(5):: TFACT   !*
  REAL(WP),DIMENSION(200):: TFF
  REAL(WP),DIMENSION(4):: TFSET(4)   !* table of flap settings used 
  REAL(WP),DIMENSION(5,5):: THRUST1   !* 2-D array of thrust values
  REAL(WP):: TIDLE   !* idle thrust ??
  REAL(WP),DIMENSION(200):: TIME
  REAL(WP),DIMENSION(25):: TLVAR   !* independent variable for POWER table
                                   ! tlvar can be velocity OR altitude
                                   ! used to calculate tot
  REAL(WP),DIMENSION(200):: TMD
  REAL(WP):: TNOTN
  REAL(WP):: TOFLP = 0.0  !* takeoff flap setting, degrees
  REAL(WP):: tone   ! thrust of ONE sized engine
  REAL(WP),DIMENSION(5,5):: TOPT1
  REAL(WP):: TOT = 1.0
  REAL(WP),DIMENSION(10,5):: TOTTAB1   ! non-dimensional PARTH1. Set in CNVT41
  REAL(WP):: TOWSLS = 0.0   !*  takeoff weight. never changed
  REAL(WP):: TRDT   !* thrust reverser deployment time, seconds
  REAL(WP):: TREV   !* ==0.0 for no reverse thrust
  REAL(WP):: TSINAL   !* ==0.0 makes SINALF=0,COSALF=1 regardless
  REAL(WP),DIMENSION(200):: TT
  REAL(WP),DIMENSION(200):: V
  REAL(WP):: VARGRA
  REAL(WP):: VFAIL
  REAL(WP):: VFAILKT   !* engine failure velocity, knots
  REAL(WP),DIMENSION(200):: VKT
  REAL(WP):: VROT
  REAL(WP),DIMENSION(10):: VROTKT   !*
  REAL(WP):: VSTEP1,VSTEP2,VSTEP3,VSTEP5 !*
  REAL(WP),DIMENSION(200):: W
  REAL(WP),DIMENSION(5,5):: WFOPT1
  REAL(WP),DIMENSION(5):: WGROSS
  REAL(WP),DIMENSION(8):: WHAT
  REAL(WP):: WOSSLS = 0.0  !*
  REAL(WP),DIMENSION(10,5):: WOWTAB1
  REAL(WP):: WTLAND   !*
  REAL(WP):: ZHEAD   !*
  REAL(WP):: ZRANGE   !*
  REAL(WP):: ZTIME   !*
  REAL(WP),DIMENSION(5):: ZW   !*



! PURPOSE - Replace the old common block CBAK
!!!      COMMON/CBAK/ CBPERT,CBTT,CBCL,CBGAM
  REAL(WP):: cbpert,cbtt,cbcl,cbgam

! PURPOSE - Replace the old common block CONS
!     COMMON/CONS/IGRD1,IGRD2
  INTEGER:: igrd1,igrd2

! PURPOSE - Replace the old common block GRAD
!      COMMON/GRAD/IGRDCAL,VLOF,VOBS,WOBS,WLOF,MISGRAD,VK1ST,VK2ND       &
!     &,FPLOF,FPOBS,BALFL,C1STGR,C2NDGR,VZERC

  INTEGER:: IGRDCAL
  REAL(WP):: VLOF,VOBS,WOBS,WLOF
  INTEGER:: MISGRAD
  REAL(WP):: VK1ST,VK2ND       
  REAL(WP):: FPLOF,FPOBS,BALFL,C1STGR,C2NDGR,VZERC

! PURPOSE - Replace the old common block APPR
!      COMMON/APPR/APVKT2,APM2,APH2,APALF2,APCL2,APLOD2,ROS2
  REAL(WP):: APVKT2,APM2,APH2,APALF2,APCL2,APLOD2,ROS2

! PURPOSE - Replace the old common block SE
!      COMMON/SE/ INENG,IFILE,IMIL
  INTEGER:: INENG,IFILE,IMIL

!-------------------------------------------------------------------------------

END Module TolCommonVariables   ! ==============================================



!+
MODULE TolProcedures
! ------------------------------------------------------------------------------
! PURPOSE - Collect all of the procedures that use COMMON storage
USE TolNonCommonProcedures
IMPLICIT NONE

  PUBLIC:: Aero41
  PUBLIC:: Approach
  PUBLIC:: ClimbGradients
  PUBLIC:: Cnvt41
  PUBLIC:: Consi
  PUBLIC:: Cutback
  PUBLIC:: Extra
  PUBLIC:: Qant4
  PUBLIC:: Refused

!-------------------------------------------------------------------------------

CONTAINS


!+
      SUBROUTINE Aero41(halt,alph,cld,cd)
! ------------------------------------------------------------------------------      
! PURPOSE - Compute aerodynamic coefficients.
!  11-23-76 new aero.format  cd now function of cl (15),
!            flap angle (4), and ground effect (2).
! NOTE - Called by Wef1041, Approach, ClimbGradients, Cutback, Refused. Calls ftlup2, bilupv
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: acdto,acdtog,aspare,conv3,dellod,flap,flapt,fpvar, &
  hnoge,  icode,ispare,jcode,k,l,loc,nfd,ntog,ntop, &
  refa1,refb1,s,talphto,talptog,tclto,tcltog,tfset,v
IMPLICIT NONE
  REAL,INTENT(IN):: halt
  REAL,INTENT(IN OUT):: ALPH
  REAL,INTENT(IN OUT):: CLD
  REAL,INTENT(IN OUT):: CD  ! on input, CD=1.0 means alph is input; 
  !  CLD and CD are computed. CD=2.0 means CLD is input; alph and CD computed

  REAL(WP):: a1,a2,a3,a4,a5
  REAL(WP):: alod
  REAL(WP):: alphge
  REAL(WP):: avkt
  REAL(WP):: cdge
  REAL(WP):: cldge
  REAL(WP):: clreqd
  REAL(WP):: hg
  REAL(WP):: hslope
  REAL(WP):: shnoge
  REAL(WP):: span
  REAL(WP):: svalph
  REAL(WP):: svalt
  REAL(WP):: svcd
  REAL(WP):: svcld
  REAL(WP):: svflap

!-------------------------------------------------------------------------------
      SPAN=REFB1*SQRT(REFA1/S)
      HG=ASPARE(7)
      IF (ASPARE(7).EQ.0.0) HG=.2*SPAN
      SVFLAP=FLAP   !!! svflap set, but never used
!     WRITE(6,50) SVFLAP
!  50 FORMAT(10X,"ENTER AERO41 WITH SVFLAP=",F6.2)
      SVALPH=ALPH
      SVCLD=CLD
      SVCD=CD
      SVALT=HALT
!        DETERMINE FLAP SETTING, UNLESS IN APPROACH OR ClimbGradients.
      IF (ICODE.EQ.9.OR.JCODE.EQ.6) GO TO 2
      IF (ISPARE(3).EQ.0) GO TO 2

      IF (K.EQ.1) AVKT=V(L)/CONV3
      IF (K.GE.2) AVKT=V(K)/CONV3
!     WRITE(6,51) L,K,AVKT,SVFLAP,HALT
!  51 FORMAT(4X,2I5,2X,"AVKT=",F6.2,2X,"FLAP=",F5.2,2X,"HALT=",F8.2)
      LOC=12
      IF (ISPARE(3).EQ.1) CALL FTLUP2(AVKT,FLAP,+1,25,FPVAR,FLAPT,LOC)
      IF (ISPARE(3).EQ.2) CALL FTLUP2(HALT,FLAP,+1,25,FPVAR,FLAPT,LOC)
      IF (LOC==1000) RETURN
!     IF (FLAP.NE.SVFLAP.AND.IPRINT.NE.0) WRITE(6,3) FLAP
!   3 FORMAT(3X,"** IN AERO41 FLAP=",F5.2,75X,"F3"/)
    2 CONTINUE
      SHNOGE=HNOGE
      HNOGE=HNOGE*SQRT(REFA1/S)   !  Modify hnoge for a/c span change.
      
      IF (SVCD.EQ.2.0) GO TO 12
      
!        ALPHA KNOWN, LOOK UP CL AND CD.
      IF (SVALT.GE.(HNOGE-HG)) GO TO 8
!      LOOK-UP CLREQD. FOR ALF. THEN LOOK-UP CD AND ALF FOR CLREQD.
!       FIRST LOOK-UP, FOR FULL ground effect
    4 CALL BILUPV(15,4,TALPTOG,TFSET,ACDTOG,TCLTOG,NTOG,NFD,SVALPH,FLAP, &
     & A1,CLREQD,A3,A4,A5)
      CALL BILUPV(15,4,TCLTOG,TFSET,ACDTOG,TALPTOG,NTOG,NFD,CLREQD,FLAP, &
     & A1,A2,CD,A4,A5)
      CLD=CLREQD
      IF (SVALT.LE.0.0001) GO TO 100
      CLDGE=CLD
      CDGE=CD
!       SECOND LOOK-UP, FOR FREE AIR (no ground effect)
    8 CALL BILUPV(15,4,TALPHTO,TFSET,ACDTO,TCLTO,NTOP,NFD,SVALPH,FLAP,  &
     & A1,CLREQD,A3,A4,A5)
      CALL BILUPV(15,4,TCLTO,TFSET,ACDTO,TALPHTO,NTOP,NFD,CLREQD,FLAP,  &
     & A1,A2,CD,A4,A5)
      CLD=CLREQD
      IF (SVALT.GE.(HNOGE-HG)) GO TO 100
      HSLOPE=SVALT/(HNOGE-HG)
      IF (ISPARE(9).EQ.1) HSLOPE=SQRT(HSLOPE)
      CLD=CLDGE+HSLOPE*(CLD-CLDGE)
      CD=CDGE+HSLOPE*(CD-CDGE)
      GO TO 100
      
      
   12 IF (SVALT.GE.(HNOGE-HG)) GO TO 18
!        CL KNOWN, LOOK UP ALPHA AND CD.
!       FIRST LOOK-UP, FOR FULL ground effect
   14 CALL BILUPV(15,4,TCLTOG,TFSET,ACDTOG,TALPTOG,NTOG,NFD,SVCLD,FLAP, &
     & A1,ALPH,CD,A4,A5)
      IF (SVALT.LE.0.0001) GO TO 100
      ALPHGE=ALPH
      CDGE=CD
!       SECOND LOOK-UP, FOR FREE AIR (NO ground effect)
   18 CALL BILUPV(15,4,TCLTO,TFSET,ACDTO,TALPHTO,NTOP,NFD,SVCLD,FLAP,   &
     & A1,ALPH,CD,A4,A5)
      IF (SVALT.GE.(HNOGE-HG)) GO TO 100
      HSLOPE=SVALT/(HNOGE-HG)
      IF (ISPARE(9).EQ.1) HSLOPE=SQRT(HSLOPE)
      ALPH=ALPHGE+HSLOPE*(ALPH-ALPHGE)
      CD=CDGE+HSLOPE*(CD-CDGE)
      
      
  100 HNOGE=SHNOGE   ! restore hnoge
      ALOD=CLD/CD
      IF (DELLOD.EQ.0.0) GO TO 200   ! dellod input to modify level of l/d clean.
      IF (DELLOD.LT.50.) ALOD=ALOD+DELLOD
      IF (DELLOD.GE.50.) ALOD=ALOD*DELLOD/100.
      CD=CLD/ALOD
      
  200 RETURN
END Subroutine Aero41   ! ------------------------------------------------------

!+
      SUBROUTINE Approach()
! ------------------------------------------------------------------------------
! PURPOSE - Only key points are calculated; for purpose of time history
!        data, intermediate points are synthesized. see aspare(3-6)

! NOTE - STANDARD APPROACH PROFILE

! The standard approach is a constant velocity descent along a 3 degree glide
! slope terminating at a 15.2 m (50 ft) obstacle at the end of the runway. 
! The performance is calculated at two points along this glide path. The first
! (outer) flight point is arbitrarily selected to be 11.1 km (6 nmi) from the
! obstacle. For the standard glide slope, this point is at an altitude of 598 m
! (1960 ft). The second (inner) flight point is above the noise monitoring
! station 1.84 km (1 nmi) from the obstacle. The altitude at this point is 112 m
! (368 ft). At each of these points, the equations of motion are balanced in an
! iterative manner to determine the required throttle setting for a steady
! glide. This may be done for either a given velocity or a given aircraft angle
! of attack. The aircraft flap setting may be different for each point. If it
! is desired to determine an optimum setting, the flap angle must be treated as
! a parameter in a series of cases. The safety regulations (ref. 6) limit the
! approach speed to not less than 1.3 times the aircraft stall speed. The
! maximum speed limit of 128.6 m/s (250 kt) in the terminal area also applies 
! during approach operations.


! Time history data along the approach profile between the two computed points 
! is synthesized using the average flight conditions (velocity and altitude) of
! the two points. Time history data between the inner flight point and the 
! obstacle are synthesized based on the results computed at the inner point by 
! maintaining a constant velocity during final approach. 

! ADVANCED APPROACH PROCEDURES

! The procedures considered incorporate increased glide slopes and decelerations
! from the outer to the inner point. The thrust required for the decelerating
! approach is determined by assuming a constant deceleration between the outer
! and the inner points. The resulting throttle level is about 10 percent lower
! than for the standard approach. The decelerations continue to an arbitrarily
! selected point 305~m (1000~ft) from the obstacle in an effort to maintain the
! low thrust level achieved at the 1.8~km (1~nmi) noise monitoring station.
! Between the inner station and the obstacle, the throttle setting returns to
! the higher level to permit the final approach at constant velocity. This
! increase in throttle setting is not as noticeable for the steeper guide slope
! since the throttle change is smaller because of the smaller deceleration.


! NOTE - Called by Wef1041; calls Aero41, Bilupv, Ftlup2, Qant4

USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: alfroll,alfrot,alpha,apalf2,ans,apcl2,aph2, &
  aplod2,apm2,apvkt2,aralf1,aralf2,arflp1,arflp2,arvel1,arvel2,aspare, &
  cbcl,cbgam,cbpert,cbtt, &
  cdgrt,cl,clgrt,conv3,delta,dout1,dout2,dtemp,dist,drag,engsout, &
  flap,gamma,glide1,glide2,gzero,h,hobsl,ibug,icode,isot,jtcode, &
  k,l,loc,lod,m,mach, &
  mpengt1,npi,npj,pbtab,q,refa1,refrho,rortab1,ros2,s,s1,s3,s4,s5, &
  tdr,tff,time,tnotn,tot,tottab1,tsinal,tt,v,vkt,w,wowtab1,wtland,zw
IMPLICIT NONE


  REAL(WP):: a3,a4,a5
  REAL(WP):: acbcl
  REAL(WP):: apalf,apalf1
  REAL(WP):: apalt
  REAL(WP):: apcb1,apcb2
  REAL(WP):: apcd
  REAL(WP):: apcl,apcl1
  REAL(WP):: apdels,apdelt,apdelv
  REAL(WP):: apdis
  REAL(WP):: apdrag
  REAL(WP):: apdrg1,apdrg2
  REAL(WP):: apert
  REAL(WP):: apflp1,apflp2
  REAL(WP):: apgam
  REAL(WP):: apgam1,apgam2
  REAL(WP):: aph
  REAL(WP):: aph1
  REAL(WP):: aplod, aplod1
  REAL(WP):: apm1
  REAL(WP):: apmach
  REAL(WP):: apq
  REAL(WP):: aprpt
  REAL(WP):: aprstp
  REAL(WP):: aptacc
  REAL(WP):: aptdr1,aptdr2
  REAL(WP):: aptff1,aptff2
  REAL(WP):: aptt1,aptt2
  REAL(WP):: apvel,apvel1
  REAL(WP):: apvkt,apvkt1
  REAL(WP):: atacc
  REAL(WP):: atdr
  REAL(WP):: atff
  REAL(WP):: att
  REAL(WP):: avros1,avros2
  REAL(WP):: avvel,avvel1,avvel2
  REAL(WP):: bpert
  REAL(WP):: btacc
  REAL(WP):: cbalf
  REAL(WP):: cbcd
!!!  REAL(WP):: cbcl
  REAL(WP):: cbdrag
!!!  REAL(WP):: cbgam
  REAL(WP):: cbh
  REAL(WP):: cbm
!!!  REAL(WP):: cbpert
  REAL(WP):: cbq
  REAL(WP):: cbtacc
  REAL(WP):: cbtdr
  REAL(WP):: cbtn
!!!  REAL(WP):: cbtt
  REAL(WP):: cbv
  REAL(WP):: cosalf
  REAL(WP):: cosg1
  REAL(WP):: cosgam
  REAL(WP):: delsec
  REAL(WP):: dlist
  REAL(WP):: dodr
  REAL(WP):: dot
  REAL(WP):: dsec
  REAL(WP):: dt1,dt2
  REAL(WP):: dtacc
  REAL(WP):: dtobs1,dtobs2
  REAL(WP):: dttd,dttd1,dttd2
  REAL(WP):: dvktdt
  REAL(WP):: fgreqd
  REAL(WP):: gam
  REAL(WP):: gearcd
  INTEGER:: iapr
  INTEGER:: iaprpt
  INTEGER:: icb
  INTEGER:: itd
  INTEGER:: ka1,ka2
  INTEGER:: ksave
  REAL(WP):: pathdis
  REAL(WP):: prslope
  REAL(WP):: ratio
  REAL(WP):: ros,ros1
  REAL(WP):: sans1
  REAL(WP):: scbtdr
  REAL(WP):: sinalf
  REAL(WP):: sing1,sing2
  REAL(WP):: singam
  REAL(WP):: spvkt1
  REAL(WP):: sqrho
  REAL(WP):: tanalf
  REAL(WP):: tdrterm
  REAL(WP):: tloss
  REAL(WP):: tnopr
  REAL(WP):: tnref
  REAL(WP):: val1,val2
  REAL(WP):: vkeas
  REAL(WP):: vkt1,vkt2
  REAL(WP):: wdowd
  REAL(WP):: wf
  REAL(WP):: wl
  REAL(WP):: zft
!-------------------------------------------------------------------------------
      ICODE=9
      DTACC=0.0
      ENGSOUT=0.0
      DELSEC=0.0
      WL=WTLAND
      IF (WTLAND.LT.1.0) WL=WTLAND*ZW(1)
      IF (WTLAND.EQ.1.0) WL=.50*ZW(1)
!        WILL HAVE INTERNAL LOOP FOR TWO APPR. POINTS.
      IAPR=2
      DLIST=DOUT2
      APH=DOUT2*TAN(GLIDE2/57.2958)+HOBSL
!      NOTE. GLIDANG INPUT AS POSITIVE ANGLE.
      APGAM=-1.0*GLIDE2
      FLAP=ARFLP2
      APALF=ARALF2
      IF (ARALF2.EQ.0.0) APALF=ALFROT-ALFROLL
      GO TO 100

   20 IAPR=1
      DLIST=DOUT1
      APH=APH2+(DOUT1-DOUT2)*TAN(GLIDE1/57.2958)
      APGAM=-1.0*GLIDE1
      FLAP=ARFLP1
      APALF=ARALF1
  100 CONTINUE
      IF (ISOT.EQ.1) WRITE(6,702) IAPR,DLIST,WL,APGAM,APH
      IF (ISOT.EQ.2) WRITE(6,702) IAPR,S3*DLIST,S1*WL,S5*APGAM,S3*APH
  702 FORMAT(/2X,"** STEADY STATE APPROACH CALCULATIONS AT POINT",I3,  &
     & 4X,"DIST. OUT=",F8.2/8X,"APPR. WT.=",F9.1,4X,"APPR. ANGLE=",F6.2, &
     & 4X, "HAPPR=",F7.2,50X,"F702"/)
      IF (IAPR.EQ.1.AND.ARVEL1.NE.0.0) GO TO 600
      IF (IAPR.EQ.2.AND.ARVEL2.NE.0.0) GO TO 600
      
      COSGAM=COS(APGAM/57.2958)
      SINGAM=SIN(APGAM/57.2958)
!        FORCE INPUT ALPHA, SOLVE FOR VELOCITY AND CBPERT.
      ZFT=APH
      APALT=APH
!        ALPHA KNOWN, LOOK UP CL AND CD. (set apcd=1.0)
      APCD=1.0
      apcl=-999.9   ! will be overwritten, but Aero41 needs a value
      CALL AERO41(APALT,APALF,APCL,APCD)
      SINALF=0.0
      COSALF=1.0
      
      IF (TSINAL.EQ.0.0) GO TO 704
      SINALF=SIN((APALF+DELTA)/57.2958)
      COSALF=COS((APALF+DELTA)/57.2958)
  704 CONTINUE
  
      LOC=9
      CALL FTLUP2(APCL   ,GEARCD,+1,15,CLGRT,CDGRT,LOC)
      IF (LOC.EQ.1000) GO TO 750
      APCD=APCD+GEARCD*REFA1/S
      APLOD=APCL/APCD
  
          IF (ISOT.EQ.1) WRITE(6,706) FLAP,APALF,APCL,APCD,APLOD
      IF (ISOT.EQ.2) WRITE(6,706) S5*FLAP,S5*APALF,APCL,APCD,APLOD
  706 FORMAT(8X,'APPRFLP=',F6.2,2X,'APALF=',F5.2,2X,'APCL=',F7.4,             &
     & 2X,'APCD=',F7.4,2X,'APLOD=',F5.2)
      TANALF=SINALF/COSALF
      CALL AT62SP(ZFT,ANS,DTEMP,PBTAB)
      IF (IAPR.EQ.1) SANS1=ANS(1)   !!! sans1 set, but never used
      ITD=1
      TDRTERM=0.0
      DODR=.8
      
  707 VAL1=COSGAM-SINGAM*TANALF-TDRTERM
      VAL2=APCL+APCD*TANALF
      APQ=(WL*VAL1)/(S*VAL2)
      APMACH=SQRT(APQ/(.7*ANS(2)))
      APVEL=APMACH*ANS(4)
  200 CONTINUE
  
      APVKT=APVEL/CONV3
      M=1
      H(M)=APH
      V(M)=APVEL
      JTCODE=0
      TOT=1.0
      CALL QANT4
      IF (LOC.EQ.1000) GO TO 750
      
      APDRAG=APCD*APQ*S
      IF (ISOT.EQ.1) WRITE(6,708) APVKT,APMACH,APDRAG,TT(1),TDR(1)
      IF (ISOT.EQ.2) WRITE(6,708) S4*APVKT,APMACH,S1*APDRAG,             &
     &S1*TT(1),S1*TDR(1)
  708 FORMAT(8X,"APVEL=",F6.2,2X,"APMACH=",F6.3,1X,"APDRAG=",F8.1,         &
     & 2X,"TTREF=",F8.1,2X,"TDRREF=",F8.1,32X,"F708")
      ATT=TT(1)
      ATDR=TDR(1)
      TNREF=ATT-ATDR
      CBTDR=ATDR*DODR
  709 SCBTDR=CBTDR
      FGREQD=(APDRAG+CBTDR+WL*SINGAM)/COSALF
      DOT=FGREQD/ATT
      CBM=APMACH
      CALL BILUPV(10,5,TOTTAB1,MPENGT1,RORTAB1,WOWTAB1,NPI,NPJ,DOT,CBM, &
     & DODR,WDOWD,A3,A4,A5)
      TFF(1)=TFF(1)*WDOWD
      CBTDR=ATDR*DODR
      ITD=ITD+1
      IF (ITD.GT.5) GO TO 713
      IF (ABS(1.-CBTDR/SCBTDR).GT.0.001) GO TO 709
      CBTN=FGREQD-CBTDR
      TNOPR=CBTN
      TNOTN=TNOPR/TNREF
      CBTACC=(FGREQD*COSALF-CBTDR-APDRAG-WL*SINGAM)/WL
      ROS=-1.0*APVEL*SINGAM
      IF (ISOT.EQ.1) WRITE(6,710) CBTDR,CBTN,CBTACC,DOT,TNOTN
      IF (ISOT.EQ.2) WRITE(6,710) S1*CBTDR,S1*CBTN,CBTACC,DOT,TNOTN
  710 FORMAT(8X,"CBTDR=",F8.0,2X,"CBTN=",F8.0,2X,"CBTACC=",F8.5,           &
     & 2X,"TGOTG=",F7.4,2X,"TNOTN=",F7.4,33X,"F710")
      IF (ABS(CBTACC).LE.0.0001) GO TO 715
      IF (ITD.GT.10) GO TO 713
      ITD=ITD+1
      TDRTERM=CBTDR*TANALF/WL
      GO TO 707
  600 CBH=APH
!        FORCE INPUT VELOCITY, SOLVE FOR ALPHA AND CBPERT.
      APALF=0.0
      IF (IAPR.EQ.1) GO TO 601
      VKT2=ARVEL2
      CBV=VKT2*CONV3
      GO TO 602
  601 VKT1=ARVEL1
      CBV=VKT1*CONV3
  602 M=1
      H(M)=CBH
      V(M)=CBV
      JTCODE=0
      TOT=1.0
      CALL QANT4
      IF (LOC.EQ.1000) GO TO 750
      
      IF (IAPR.EQ.1) SANS1=ANS(1)
      CBM=MACH(1)
      CBQ=Q(1)
      ATT=TT(1)
      ATDR=TDR(1)
      ATFF=TFF(1)
      TNREF=ATT-ATDR
      K=1
      W(K)=WTLAND
      ICB=1
      TLOSS=1.0
      APERT=0.0
      ATACC=0.0
      GAM=APGAM
      COSGAM=COS(GAM/57.2958)
      SINGAM=SIN(GAM/57.2958)
      CBPERT=1.0
!     IF (ICB.EQ.3) CBPERT=*****
      CBGAM=GAM   !!! cbgam set, but never used???
  608 CBTT=ATT*TLOSS*CBPERT
      SINALF=0.0
      COSALF=1.0
      CBCL=(W(K)*COSGAM)/(CBQ*S)
      GO TO 612
  610 CBCL=ACBCL
  612 CBCD=2.0
      cbalf=-999.0   ! will be overwritten, but Aero41 needs a value
!        CL KNOWN , LOOK UP ALPHA AND CD. (set cbcd=2.0)
      CALL AERO41(CBH,CBALF,CBCL,CBCD)
      LOC=10
      CALL FTLUP2(CBCL   ,GEARCD,+1,15,CLGRT,CDGRT,LOC)
      IF (LOC.EQ.1000) RETURN
      CBCD=CBCD+GEARCD*REFA1/S
      IF (TSINAL.EQ.0.0) GO TO 614
      SINALF=SIN((CBALF+DELTA)/57.2958)
      COSALF=COS((CBALF+DELTA)/57.2958)
      ACBCL=(W(K)*COSGAM-CBTT*SINALF)/(CBQ*S)
      IF (IBUG.NE.0) WRITE(6,613)CBPERT,CBTT,CBCL,ACBCL
  613 FORMAT(12X,4E15.8,20X,"F613")
      IF (ABS(1.-ACBCL/CBCL).GT..0001) GO TO 610
  614 CBDRAG=CBCD*CBQ*S
      DOT=CBPERT
      CALL BILUPV(10,5,TOTTAB1,MPENGT1,RORTAB1,WOWTAB1,NPI,NPJ,DOT,CBM, &
     & DODR,WDOWD,A3,A4,A5)
      CBTDR=ATDR*TLOSS*DODR
      CBTACC=(CBTT*COSALF-CBTDR-CBDRAG-W(K)*SINGAM)/W(K)
      IF (IBUG.NE.0) WRITE(6,615)CBPERT,CBTT,ACBCL,CBTACC
  615 FORMAT(12X,4E15.8,20X,"F615")
      IF (ABS(CBTACC-DTACC).LT.0.0001) GO TO 620
!        ASSUME TWO THROTTLE SETTINGS AND USE SLOPE TECH. TO
!     FIND REDUCTION FOR ZERO ACCEL.
      IF (APERT.EQ.0.0) GO TO 616
      BPERT=CBPERT
      BTACC=CBTACC
      PRSLOPE=(APERT-BPERT)/(ATACC-BTACC)
      CBPERT=BPERT+(DTACC-BTACC)*PRSLOPE
      APERT=BPERT
      ATACC=BTACC
      GO TO 608
  616 APERT=CBPERT
      ATACC=CBTACC
      CBPERT=.90
      GO TO 608
  620 WF=ATFF*WDOWD
      APCL=CBCL
      APCD=CBCD
      APLOD=APCL/APCD
      APMACH=CBM
      APDRAG=CBDRAG
      FGREQD=CBTT
      CBTN=FGREQD-CBTDR
      TNOPR=CBTN
      TNOTN=TNOPR/TNREF
      APVKT=CBV/CONV3
      DOT=CBPERT
      APMACH=CBM
      APALF=CBALF
      APGAM=GAM
      TFF(1)=WF
      ROS=-1.0*CBV*SINGAM
      IF (ISOT.EQ.1) WRITE(6,706) FLAP,APALF,APCL,APCD,APLOD
      IF (ISOT.EQ.2) WRITE(6,706) S5*FLAP,S5*APALF,APCL,APCD,APLOD
      IF (ISOT.EQ.1) WRITE(6,708) APVKT,APMACH,APDRAG,TT(1),TDR(1)
      IF (ISOT.EQ.2) WRITE(6,708) S4*APVKT,APMACH,S1*APDRAG,             &
     &S1*TT(1),S1*TDR(1)
      IF (ISOT.EQ.1) WRITE(6,710) CBTDR,CBTN,CBTACC,DOT,TNOTN
      IF (ISOT.EQ.2) WRITE(6,710) S1*CBTDR,S1*CBTN,CBTACC,DOT,TNOTN
!        1ST TIME,   IAPR=2
!        2ND TIME,   IAPR=1
!        3RD TIME,   IAPR=3
      IF (IAPR.EQ.1) GO TO 718
      IF (IAPR.EQ.2) GO TO 716
      IF (IAPR.EQ.3) GO TO 834
      GO TO 718
  713 IF (ISOT.EQ.1) WRITE(6,714)APVKT,DOT,SCBTDR,CBTDR
      IF (ISOT.EQ.2) WRITE(6,714) S4*APVKT,DOT,S1*SCBTDR,S1*CBTDR
  714 FORMAT(10X,"AT APVEL=",F6.2,2X,"DOT=",F8.4,2X,"SCBTDR=",F8.0,        &
     & 2X,"CBTDR=",F8.0,10X,"F714")
  715 IF (IAPR.EQ.1) GO TO 718
  716 APH2=APH
      ROS2=ROS
      DTOBS2=(APH2-HOBSL)/ROS2
      DTTD2=APH2/ROS2
      APDIS=DOUT2
      DTTD=DTTD2   !!! dttd set, but never used???
      APVKT2=APVKT


      APCB2=DOT
      APM2=APMACH
      APGAM2=APGAM
      APALF2=APALF
      APFLP2=FLAP
      APTFF2=TFF(1)
      APCL2=APCL
      APLOD2=APLOD
      APDRG2=APDRAG
      APTT2=FGREQD
      APTDR2=CBTDR
      IF (ISOT.EQ.1) WRITE(6,712) APVKT2,APH2,DOUT2,ROS2,DTTD2
      IF (ISOT.EQ.2) WRITE(6,712) S4*APVKT2,S3*APH2,S3*DOUT2,            &
     &S3*ROS2,DTTD2
  712 FORMAT(8X,"APVEL2=",F7.2,2X,"APH2=",F7.2,2X,"DOUT2=",F8.2,           &
     & 2X,"ROS2=",F8.4,2X,"DTTD2=",F8.4/)
      IF (DOUT1.NE.0.0.AND.DOUT1.GT.DOUT2) GO TO 20
      CBPERT=DOT
      K=1
      TIME(K)=-1.0*DTOBS2
      H(K)=APH2
      DIST(K)=DOUT2
      VKT(K)=APVKT2
      MACH(K)=APM2
      GAMMA(K)=APGAM2
      ALPHA(K)=APALF2
      W(K)=WTLAND
      FLAP=APFLP2
      GO TO 820
  718 APH1=APH
      ROS1=ROS
!        CALC. AVROS1 BASED ON AVVEL1 AND GLIDE1.
      AVVEL1=(APVKT+APVKT2)*CONV3/2.
      AVROS1=AVVEL1*SIN(GLIDE1/57.2958)
      DELSEC=(APH1-APH2)/AVROS1
      DTOBS1=DTOBS2+DELSEC
      DTTD1=DTTD2+DELSEC
      APDIS=DOUT1
      DTTD=DTTD1
      APVKT1=APVKT

      APCB1=DOT   !!! apcb1 set, but not used???
      APM1=APMACH
      APGAM1=APGAM   !!! apgam1 set, but not used???
      APALF1=APALF   !!! apalf1 set, but not used???
      APFLP1=FLAP
      APTFF1=TFF(1)   !!! apttf1 set, but not used???
      APCL1=APCL   !!! apcl1 set, but not used???
      APLOD1=APLOD   !!! aplod1 set, but not used???
      APDRG1=APDRAG   !!! apdrg1 set, but not used???
      APTT1=FGREQD   !!! aptt1 set, but not used???
      APTDR1=CBTDR   !!! apdr1 set, but not used???

      IF (ISOT.EQ.1) WRITE(6,711) APVKT1,APH1,DOUT1,ROS1,DTTD1
      IF (ISOT.EQ.2) WRITE(6,711) S4*APVKT1,S3*APH1,S3*DOUT1,S3*ROS1,DTTD1
  711 FORMAT(8X,"APVEL1=",F7.2,2X,"APH1=",F7.2,2X,"DOUT1=",F8.2,           &
     & 2X, "ROS1=",F8.4,2X,"DTTD1=",F8.4,33X,"F711"/)
      IF (ASPARE(3).LE.0.0) GO TO 717
      
      IF (ISOT.EQ.1) WRITE(6,701) DOUT1,DOUT2
      IF (ISOT.EQ.2) WRITE(6,701) S3*DOUT1,S3*DOUT2
  701 FORMAT(2X,"**SYNTHESIZE POINTS BETWEEN DOUT1=",F8.0,              &
     & 1X,"AND DOUT2=",F8.0,53X,"F701")
      IF (ISOT.EQ.1) WRITE(6,705) AVVEL1,AVROS1,DELSEC
      IF (ISOT.EQ.2) WRITE(6,705) S3*AVVEL1,S3*AVROS1,DELSEC
  705 FORMAT(7X,"AVVEL1=",F7.2,2X,"AVROS1=",F7.4,2X,"DELSEC=",F7.2/)
!        AT THIS POINT GENERATE INTERMEDIATE FLIGHT POINTS.
!        BETWEEN THE DOUT1 AND DOUT2 STATIONS.
!        ASPARE(3)=NO. OF STEPS
!        ASPARE(4)=STEP TIME INTERVAL, SEC.

  800 CONTINUE
      IF (DELSEC.EQ.0.0) GO TO 820
      DVKTDT=(APVKT1-APVKT2)/DELSEC
!        DECEL. IN KTS. PER SEC.
      APRSTP=ASPARE(3)
      APRPT=APRSTP+1.
      IAPRPT=IFIX(APRPT)
      DT2=ASPARE(4)
      DT1=DELSEC-(APRSTP-1.)*DT2
      SING1=SIN(GLIDE1/57.2958)
      COSG1=COS(GLIDE1/57.2958)
      IAPR=3
      APDELS=(DOUT1-DOUT2)/COSG1
      APDELV=(APVKT1-APVKT2)*CONV3
      APVEL1=APVKT1*CONV3   !!! apvel1 set, but never used???
      APDELT=APDELS/((APVKT1+APVKT2)*CONV3/2.)
      DELSEC=APDELT
      APTACC=APDELV/APDELT
!      APTACC IS DECEL IN FT. PER. SEC2
      DTACC=-1.*APTACC/GZERO
!      DTACC IS DECEL IN NEG. G UNITS
      DVKTDT=APTACC/CONV3
!        DVKTDT IS DECEL IN KTS. PER. SEC.
      DT1=DELSEC-(APRSTP-1.)*DT2
      SPVKT1=(APVKT1+APVKT2)/2.
      CBV=SPVKT1*CONV3
      CBH=(APH1+APH2)/2.
      FLAP=ARFLP1
      APGAM=-1.0*GLIDE1
      IF (ISOT.EQ.1) WRITE(6,810) DTACC,WL,APGAM,CBH,SPVKT1
      IF (ISOT.EQ.2) WRITE(6,810) DTACC,S1*WL,S5*APGAM,S3*CBH,S4*SPVKT1
  810 FORMAT(/2X,"*** APPROACH CALCS. FOR DECEL.=",F10.6," G UNITS"/    &
     & 6X,"APPR. WT.=",F9.1,4X,"APPR. ANGLE=",F6.2,                      &
     & 2X,"AVG. ALT.=",F7.2,2X,"AVG. VEL=",F6.2,34X,"F810"/)
      GO TO 602
      
      
!        BASED ON AVG. VEL. AND ALT. ITERATE FOR DECEL. CL AND TG.
  834 CONTINUE
!        SET UP 1ST PT. BASED ON DECEL. CALC.
      WRITE(6,836)
  836 FORMAT(//)
      K=1
      H(K)=APH1
      MACH(K)=APM1
      DIST(K)=DOUT1
      TIME(K)=DTTD1
      VKT(K)=APVKT1
      W(K)=WTLAND
      FLAP=APFLP1
      GAMMA(K)=GAM
      ALPHA(K)=CBALF
      TFF(K)=WF
      CL(K)=APCL
      LOD(K)=APLOD
      DRAG(K)=APDRAG
      TT(K)=CBTT
      TDR(K)=CBTDR
      CBPERT=DOT
      DSEC=DT1
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      
      IF (ISOT.EQ.1) WRITE(6,840) K,VKT(K),MACH(K),H(K),W(K),DIST(K),    &
     & TIME(K),VKEAS,FLAP,GAMMA(K),ALPHA(K),CBPERT
      IF (ISOT.EQ.2) WRITE(6,840) K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),   &
     & S3*DIST(K),TIME(K),S4*VKEAS,S5*FLAP,S5*GAMMA(K),S5*ALPHA(K),CBPERT
  840 FORMAT(2X,I3,2X,"V TAS=",F6.2,2X,"MACH=",F6.3,2X,"H=",F7.2,          &
     & 2X,"W=",F8.0,2X,"DIST=",F7.0,2X,"TIME=",F7.2/7X,"V EAS=",F6.2,2X, &
     "FLA&P=",F5.2,2X,"GAMMA=",F6.2,2X,"ALPHA=",F5.2,2X,"CBPERT=",F7.4,          &
     & 42X,"F840"/)
     
!       THIS LOCATION USED FOR SAVING TIME HISTORY DATA
!       SEE COMMENT IN MAIN BELOW STATEMENT NO. 418
      DO 845 K=2,IAPRPT
      L=K-1
      IF (K.GE.3) DSEC=DT2
      TIME(K)=TIME(L)-DSEC
      VKT(K)=VKT(L)-DVKTDT*DSEC
      V(L)=VKT(L)*CONV3
      PATHDIS=V(L)*DSEC-(APTACC/2.)*DSEC**2.
      DIST(K)=DIST(L)-PATHDIS*COSG1
      H(K)=H(L)-PATHDIS*SING1
      V(K)=VKT(K)*CONV3
      ZFT=H(K)
      CALL AT62SP(ZFT,ANS,DTEMP,PBTAB)
      MACH(K)=V(K)/ANS(4)
      W(K)=W(L)
      IF (K.EQ.IAPRPT) GO TO 830
      GAMMA(K)=APGAM
      FLAP=FLAP
      TFF(K)=TFF(1)
      CBPERT=CBPERT
      ALPHA(K)=APALF
      CL(K)=APCL
      LOD(K)=APLOD
      DRAG(K)=APDRAG
      TT(K)=CBTT
      TDR(K)=CBTDR
      GO TO 832
  830 GAMMA(K)=APGAM2
!        CONDITIONS AT POINT 2
      FLAP=APFLP2
      TFF(K)=APTFF2
      CBPERT=APCB2
      ALPHA(K)=APALF2
      CL(K)=APCL2
      LOD(K)=APLOD2
      DRAG(K)=APDRG2
      TT(K)=APTT2
      TDR(K)=APTDR2
  832 KSAVE=K
      WRITE(6,835)
  835 FORMAT('0')
  
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      
      IF (ISOT.EQ.1) WRITE(6,840) K,VKT(K),MACH(K),H(K),W(K),DIST(K),    &
     &TIME(K),VKEAS,FLAP,GAMMA(K),ALPHA(K),CBPERT
      IF (ISOT.EQ.2) WRITE(6,840) K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),   &
     &S3*DIST(K),TIME(K),S4*VKEAS,S5*FLAP,S5*GAMMA(K),S5*ALPHA(K),CBPERT
!       THIS LOCATION USED FOR SAVING TIME HISTORY DATA
!       SEE COMMENT IN MAIN BELOW STATEMENT NO. 418
  845 END DO
  
  
      GO TO 855
  855 K=KSAVE
!        CALC. AVROS2 BASED ON CONSTANT VKT AND GLIDE2.
      GAMMA(K)=APGAM2
      FLAP=APFLP2
      MACH(K)=APM2
      IF (ASPARE(6).LE.0.0) GO TO 717
      AVVEL2=APVKT2*CONV3
      SING2=SIN(GLIDE2/57.2958)
      AVROS2=AVVEL2*SING2
      DELSEC=(APH2-HOBSL)/AVROS2
      
!        NOW GEN. POINTS FROM DOUT2 TO HOBSL.
!        ASPARE(5)=STEP TIME INTERVAL, SEC.
!        ASPARE(6)=NO. OF STEPS
      IF (ISOT.EQ.1) WRITE(6,851) DOUT2
      IF (ISOT.EQ.2) WRITE(6,851) S3*DOUT2
  851 FORMAT(2X,"**SYNTHESIZE POINTS BETWEEN DOUT2=",F8.0,              &
     & 1X,"AND OBSTACLE",59X,"F851")
      IF (ISOT.EQ.1) WRITE(6,852) AVVEL2,AVROS2,DELSEC
      IF (ISOT.EQ.2) WRITE(6,852) S3*AVVEL2,S3*AVROS2,DELSEC
  852 FORMAT(7X,"AVVEL2=",F7.2,2X,"AVROS2=",F7.4,2X,"DELSEC=",F7.2/)
      DT2=ASPARE(5)
      DSEC=DT2
      KA1=KSAVE+1
!        ASPARE(6)= NO. OF POINTS INBOUND OF 1 NM. PT.
      KA2=KSAVE+IFIX(ASPARE(6))
      DO 870 K=KA1,KA2
      L=K-1
      RATIO=DSEC/DELSEC
!      VELOCITY CONSTANT INBOUND OF DOUT2 POINT.
      VKT(K)=APVKT2
      AVVEL=AVVEL2
      H(K)=H(L)-DSEC*SING2*AVVEL
      IF (H(K).LE.HOBSL) GO TO 717
      DIST(K)=DIST(L)-RATIO*DOUT2
      TIME(K)=TIME(L)-DSEC
      GAMMA(K)=APGAM2
      W(K)=W(L)
      FLAP=APFLP2
      CL(K)=APCL2
      LOD(K)=APLOD2
      DRAG(K)=APDRG2
      TT(K)=APTT2
      TDR(K)=APTDR2
      TFF(K)=APTFF2
      MACH(K)=APM2
      CBPERT=APCB2
      ALPHA(K)=APALF2
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      IF (ISOT.EQ.1) WRITE(6,840) K,VKT(K),MACH(K),H(K),W(K),DIST(K),    &
     &TIME(K),VKEAS,FLAP,GAMMA(K),ALPHA(K),CBPERT
      IF (ISOT.EQ.2) WRITE(6,840) K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),   &
     &S3*DIST(K),TIME(K),S4*VKEAS,S5*FLAP,S5*GAMMA(K),S5*ALPHA(K),CBPERT
!       THIS LOCATION USED FOR SAVING TIME HISTORY DATA
!       SEE COMMENT IN MAIN BELOW STATEMENT NO. 418
  870 END DO
      GO TO 717
  820 CBPERT=DOT
      K=1
      TIME(K)=-4000.
      H(K)=APH
      DIST(K)=APDIS
      VKT(K)=APVKT
      GAMMA(K)=APGAM
      ALPHA(K)=APALF
      W(K)=WTLAND
      FLAP=FLAP
      TFF(K)=TFF(1)
      CL(K)=1.0
      LOD(K)=1.0
      DRAG(K)=1.0
      TT(K)=1.0
      TDR(K)=1.0
      MACH(K)=APMACH
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      IF (ISOT.EQ.1) WRITE(6,840) K,VKT(K),MACH(K),H(K),W(K),DIST(K),    &
     &TIME(K),VKEAS,FLAP,GAMMA(K),ALPHA(K),CBPERT
      IF (ISOT.EQ.2) WRITE(6,840) K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),   &
     &S3*DIST(K),TIME(K),S4*VKEAS,S5*FLAP,S5*GAMMA(K),S5*ALPHA(K),CBPERT
  717 CONTINUE
  
  750 RETURN
END Subroutine Approach   ! -------------------------------------------------------


!+
      SUBROUTINE ClimbGradients()
! ------------------------------------------------------------------------------
! PURPOSE -  Compute 1st and 2nd segment climb gradients.
!  For each rotational velocity, after calculations of takeoff field length are
!  completed, the first and second segment gradients are computed. These are
!  minimum climb gradients, depending on the number of engines on the aircraft,
!  which must be available to satisfy the regulations of reference 6. The first
!  segment gradient is computed at $V_{LO}$, with one engine out, gear extended, 
!  and out of ground effect. The second segment gradient is computed at V2, with
!  one engine out, gear up, and out of ground effect.  The required gradients
!  (see Table I) are built into the program logic, and, if appropriate levels
!  are not met, the output will so indicate. 

! NOTE - Called by Wef1041; calls Aero41,Ftlup2,Qant4
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: alpha,anoe,aspare,balfl,cdd,cdgrt,cdoutt, &
  cl,clgrt,c1stgr,c2ndgr,conv3, &
  delta,drag,drone,engsout,esf,flap,fplof,fpobs,h,hnoge,hobs, &
  igrdcal,iprint,isot,jcode,jtcode, &
  k,loc,m,misgrad,q,refa1,s,s3,s4,s5,tdr,tone,tot,tsinal,tt,  &
  v,vk1st,vk2nd,vlof,vobs,vzerc, w,wlof,wobs
IMPLICIT NONE


  REAL(WP):: acon
  REAL(WP):: agd
  REAL(WP):: alph
  REAL(WP):: avz,bvz
  REAL(WP):: bgd
  REAL(WP):: cd
  REAL(WP):: cdengf
  REAL(WP):: cosalf
  REAL(WP):: fcl
  REAL(WP):: gcl
  REAL(WP):: gearcd
  REAL(WP):: gdslp
  REAL(WP):: grad
  REAL(WP):: gradmin
  REAL(WP):: grlod
  REAL(WP):: gsingam
  REAL(WP):: halt
  INTEGER:: igrad
  INTEGER:: ivz
  INTEGER:: noe
  REAL(WP):: sinalf


!-------------------------------------------------------------------------------
      JCODE=6
      IF (BALFL.EQ.0.0) WRITE(6,919)
  919 FORMAT(///2X,"***** 1ST AND 2ND SEGMENT ENGINE OUT CLIMB GRADIENTS",2X, &
     " AT VLOF AND V2, FOR ALL ENGINES OPERATING.",19X,"F919")

      IF (BALFL.NE.0.0) WRITE(6,917)
  917 FORMAT(///2X,"***** 1ST AND 2ND SEGMENT ENGINE OUT CLIMB GRADIENTS",2X, &
      " AT VLOF AND V2, FOR BALANCED FIELD LENGTH.",19X,"F917")

      WRITE(6,918)
  918 FORMAT(6X,"** GRADIENTS CALC. OUT OF GROUND EFFECT")
      ACON=1.0/CONV3
      IF (ISOT.EQ.1) WRITE(6,100) ACON*VLOF,ACON*VOBS
      IF (ISOT.EQ.2) WRITE(6,100) S3*VLOF,S3*VOBS
  100 FORMAT(10X, "(VLOF=",F7.2,2X,"VOBS=",F7.2,2X,")" )
      JTCODE=0
      TOT=1.0
      IF (ASPARE(11).EQ.0.0) GO TO 914
      JTCODE=1
      TOT=ASPARE(11)
      WRITE(6,913) TOT
  913 FORMAT(6X,                                                         &
       '** GRADIENTS CALC. WITH MODIFIED THROTTLE (TGOTG=', F7.4,').' )
      GO TO 912
  914 WRITE(6,915)
  915 FORMAT(6X,'** GRADIENTS CALC. WITH FULL THROTTLE (TGOTG=1.0).')
  912 WRITE(6,916)
  916 FORMAT(6X,"** GRADIENTS CALC. AT EXISTING FLAP SETTINGS.")
!        COMPUTE 1ST AND 2ND SEGMENT ENGINE OUT CLIMB GRADIENTS
      IGRAD=1
      MISGRAD=0
      C1STGR=0.0
      C2NDGR=0.0
      IGRDCAL=1
      IF (VLOF.LE.0.0) GO TO 70
      
      V(K)=VLOF
      W(K)=WLOF
      H(K)=0.0
      FLAP=FPLOF
      GO TO 922
      
  920 IF (VOBS.LE.0.0) GO TO 74
      V(K)=VOBS
  921 W(K)=WOBS
      H(K)=HOBS
      FLAP=FPOBS
  922 M=K
!        ASSUME STEADY LEVEL FLIGHT, SOLVE FOR CL AND GRAD.
!      NOTE THAT THROTTLE MAY BE INCLUDED IN GRADIENT CALCS.
!
!      NOTE THAT FLAP VARIATIONS ARE INCLUDED IN GRADIENT CALC.
!
      CALL QANT4()
!      *** QANT4 CALCS. TG AND DR FOR ONE ENGINE. (SIZED AND THROTTLED)
      IF (LOC.EQ.1000) GO TO 80
      TT(K)=(ANOE-1.)*TONE
      TDR(K)=(ANOE-1.)*DRONE
!     NEXT STATMENT FORCES NO GROUND EFFECT AERO. DATA
      HALT=HNOGE
      GCL=W(K)/(Q(K)*S)
      GO TO 926
  924 GCL=FCL
  926 CD=2.0
      alph=-999.0   ! will be overwritten, but Aero41 needs a value
!        CL KNOWN , LOOK UP ALPHA AND CD. (Set CD=2.0)
      CALL AERO41(HALT,ALPH,GCL,CD)
      ALPHA(K)=ALPH
      SINALF=0.0
      COSALF=1.0
      IF (TSINAL.EQ.0.0) GO TO 928
      SINALF=SIN((ALPHA(K)+DELTA)/57.2958)
      COSALF=COS((ALPHA(K)+DELTA)/57.2958)
  928 FCL=(W(K)-TT(K)*SINALF)/(Q(K)*S)
      IF (ABS(1.-FCL/GCL).GT.0.0001) GO TO 924
      
      CL(K)=GCL
      CDD(K)=CD
      CDENGF=0.0
!      SIMILAR LOGIC FOR ENGINE OUT T.O. NEAR .920
      IF (CDOUTT(3).EQ.0.0) GO TO 927
      CDENGF=CDOUTT(3)*CDOUTT(5)*ESF/S
      GO TO 929
  927 CDENGF=CDOUTT(1)*CDOUTT(2)*ESF/S
  929 CDD(K)=CDD(K)+CDENGF*ENGSOUT
      
      IF (IGRAD.GE.2) GO TO 930
      LOC=8
      CALL FTLUP2(  CL(K),GEARCD,+1,15,CLGRT,CDGRT,LOC)
      IF (LOC.EQ.1000) GO TO 80
      CDD(K)=CDD(K)+GEARCD*REFA1/S
  930 DRAG(K)=CDD(K)*Q(K)*S
      GSINGAM=(TT(K)*COSALF-TDR(K)-DRAG(K))/W(K)
      GRAD=TAN(ASIN(GSINGAM))
      GRLOD=GCL/CDD(K)
      IF (IPRINT.GE.3) WRITE(6,89)GCL,CDD(K),GRLOD
   89 FORMAT(10X,"GRADIENT AERO.,", 2X," CL=",F7.4,2X," CD=",F7.4,        &
      2X, " LOD=",F6.2,54X,"F89")
      NOE=INT(ANOE)
      IF (IGRAD.EQ.3) GO TO 110
      IF (IGRAD.EQ.2) GO TO 949
      GO TO (940,940,941,942),NOE    ! SELECT CASE
  940 GRADMIN=.000
      GO TO 945
  941 GRADMIN=.003
      GO TO 945
  942 GRADMIN=.005
  945 VK1ST=V(K)/CONV3
      C1STGR=GRAD
      IF (ISOT.EQ.1) WRITE(6,800) GRAD,VK1ST,FLAP,GRADMIN
      IF (ISOT.EQ.2) WRITE(6,800) GRAD,S4*VK1ST,S5*FLAP,GRADMIN
  800 FORMAT(3X,"***** FIRST SEG. ENGINE OUT CLIMB GRAD.=",F8.5,        &
     &2X, "AT VEL=",F6.1,2X,",FLAP=",F6.2,2X,                              &
      "(MIN. GRAD.=",F6.4," )",15X,"F800"/)
      IF (GRAD.GE.GRADMIN) GO TO 946
      MISGRAD=1
      WRITE(6,4022)
 4022 FORMAT(6X,"***** FIRST SEGMENT CLIMB GRADIENT NOT MET *****"//)
  946 IGRAD=2
      GO TO 920
  949 GO TO (950,950,951,952),NOE   ! SELECT CASE
  950 GRADMIN=.024
      GO TO 955
  951 GRADMIN=.027
      GO TO 955
  952 GRADMIN=.030
  955 VK2ND=V(K)/CONV3
      C2NDGR=GRAD
      IF (ISOT.EQ.1) WRITE(6,802) GRAD,VK2ND,FLAP,GRADMIN
      IF (ISOT.EQ.2) WRITE(6,802) GRAD,S4*VK2ND,S5*FLAP,GRADMIN
  802 FORMAT(3X,"***** SECOND SEG. ENGINE OUT CLIMB GRAD.=",F8.5,       &
     & 2X,"AT VEL=",F6.1,2X,",FLAP=",F6.2,2X,                              &
     & "(MIN. GRAD.=",F6.4," )",15X,"F802"/)
      IF (GRAD.GE.GRADMIN) GO TO 80
      MISGRAD=MISGRAD+2
      WRITE(6,4024)
 4024 FORMAT(6X,"***** SECOND SEGMENT CLIMB GRADIENT NOT MET *****"//)
      GO TO 80
   70 VK1ST=0.0
      VK2ND=0.0
      WRITE(6,72)
   72 FORMAT(4X,"*** VLOF NOT KNOWN.")
      GO TO 200
   74 VK2ND=0.0
      WRITE(6,76)
   76 FORMAT(4X,"*** VOBS NOT KNOWN.")
      GO TO 200
   80 IGRAD=3
!     IF (ANOE.LT.4) GO TO 200
!       CALC. VZERC AT HOBS.
      IVZ=1
      V(K)=VOBS/1.2
      GO TO 921
  110 AVZ=V(K)
      AGD=GRAD
!     WRITE(6,111) IVZ,AVZ,AGD
! 111 FORMAT(4X,"IVZ=",I4,2X,"AVZ=",F7.2,2X,"AGD=",F7.4)
      IVZ=IVZ+1
      IF (ABS(AGD).LT.0.0001) GO TO 116
      IF (IVZ.GT.10) GO TO 200
      IF (IVZ.GE.3) GO TO 112
      V(K)=AVZ-10.
      GO TO 114
  112 GDSLP=(AVZ-BVZ)/(AGD-BGD)
      V(K)=BVZ-BGD*GDSLP
  114 BVZ=AVZ
      BGD=AGD
      IF (ABS(V(K)-BVZ).GT.0.5) GO TO 922
  116 VZERC=AVZ/CONV3
      IF (ISOT.EQ.1) WRITE(6,118) VZERC,1.125*VZERC
      IF (ISOT.EQ.2) WRITE(6,118) S4*VZERC,1.125*S4*VZERC
  118 FORMAT(4X,"*** AT OBSTACLE, VZERC=",F7.2,10X,"1.125 VZERC=", F7.2)
  
  200 RETURN
END Subroutine ClimbGradients   ! ------------------------------------------------------



!+
      SUBROUTINE CNVT41()
! ------------------------------------------------------------------------------
! PURPOSE - Convert aero to polar format and engine data to non-dimen. format
! NOTE - Called by Wef1041; calls AT62sp
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: acdto,acdtog,ans,delcd,dropt1,dtemp, &
  fuel1,hengt, &
  isearch,nei,nej,npi,npj,parram1,parth1,parwf1,pbtab,ram1,refa1,rortab1,s, &
  tcdto,tcdtog,thrust1,topt1,tottab1,wfopt1,wowtab1
IMPLICIT NONE

  REAL(WP),DIMENSION(5):: lasTh1,lasWf1,lasRam1
!
!      MODIFIED CONVERT.  THIS VERSION DOES NOT CONTAIN OPTION FOR
!      PARABOLIC POLAR INPUTS.    ONLY PS=1 DATA ARE CONVERTED.
!      ALSO LIMITED TO ENGCODE=3 FORMAT.
!
!      NO LONGER CONVERTS NOISE RELATED ENGINE PARAM.
!
!       CDGEAR IS ASSUMED TO BE REMOVED FROM INPUT POLARS.
!        FOR MILITARY AIRCRAFT, DELCD IS FOR EXTERNAL STORES.

  INTEGER:: i,j,nnpi
  REAL(WP):: zft

!-------------------------------------------------------------------------------
      DO 2 I=1,5
      DO 2 J=1,5
      TOPT1(I,J)=0.0
      WFOPT1(I,J)=0.0
      DROPT1(I,J)=0.0
    2 CONTINUE
    
      IF (ISEARCH.GE.10) GO TO 111
      DO 102 J=1,4   !      MODIFY INPUT DRAG LEVELS BY INPUT DELCD.
      DO 102 I=1,15
      ACDTO(I,J)=TCDTO(I,J)+DELCD(J)*(REFA1/S)
      ACDTOG(I,J)=TCDTOG(I,J)+DELCD(J)*(REFA1/S)
  102 CONTINUE
  
  111 DO 42 I=1,NEI
      DO 42 J=1,NEJ
      ZFT=HENGT(I,J)
      CALL AT62SP(ZFT,ANS,DTEMP,PBTAB)
  503 TOPT1(I,J)=THRUST1(I,J)/ANS(2)
      WFOPT1(I,J)=FUEL1(I,J)/ANS(2)
      DROPT1(I,J)=RAM1(I,J)/ANS(2)
!      NOTE THAT AJ AND TEMP. ARE NOT NON-DIMEN.
   42 CONTINUE
   
!            AT THIS POINT HAVE T/PA , WF/PA , DRAM/PA
      IF (NPJ.EQ.0) RETURN
      DO 50 J=1,NPJ
      DO 51 I=1,NPI
      NNPI=NPI+1-I
      IF (PARTH1(NNPI,J).EQ.0.) CYCLE   !!! GO TO 51
      LASTH1(J)=PARTH1(NNPI,J)
      LASWF1(J)=PARWF1(NNPI,J)
      LASRAM1(J)=PARRAM1(NNPI,J)
      EXIT   !!! GO TO 50
   51 END DO
   50 END DO
   
      DO 56 I=1,NPI
      DO 56 J=1,NPJ
!        ** NOTE.  ONLY ROR IS PROTECTED AGAINST XX.X/0.0=INDEF. **
      TOTTAB1(I,J)=PARTH1(I,J)/LASTH1(J)
      WOWTAB1(I,J)=PARWF1(I,J)/LASWF1(J)
      IF (LASRAM1(J).NE.0.0) GO TO 54
      RORTAB1(I,J)=1.
      GO TO 56
   54 RORTAB1(I,J)=PARRAM1(I,J)/LASRAM1(J)
   56 CONTINUE
   
  RETURN
END Subroutine Cnvt41   ! ------------------------------------------------------

!+
      SUBROUTINE CONSI()
! ------------------------------------------------------------------------------
! PURPOSE - Convert all namelist dimensional data
!         if isin = 1 ,  from fps to s1 units
!         if isin = 2 ,  from s1 to fps units
!        all namelist items with units must be included as added.
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: arvel1,arvel2,aspare,ca,cbalt,cbdist,cblimh, &
  cdoutt,ce,dout1,dout2,dvclimb,dwtaxi,fplimh,fpvar,fuel1, &
  grad1,grad2,gzero,hnoge,hobs,hobsl,hstep1,hstep2, &
  htab1,hengt,igrd1,igrd2,isin,ispare, parram1,parth1,parwf1,&
  pint1,pint2,pint3,ram1,refa1,refb1,rzero, &
  s,s1,s2,s3,s4,s5,s6,s7,s8,s9,s10, &
  stopdis,tdidle,tidle, &
  thrust1, tlvar,vfailkt,vrotkt,vstep1,vstep2,vstep3,vstep5,wossls,wtland,zrange,zw
IMPLICIT NONE

  REAL(WP):: f
  INTEGER:: i
!-------------------------------------------------------------------------------

!       LBF TO NEWTON ITEMS  (S1)
      IF (ISIN.EQ.1) F=S1
      IF (ISIN.EQ.2) F=1.0/S1
      TIDLE=F*TIDLE
      TDIDLE=F*TDIDLE
      WTLAND=F*WTLAND
      DWTAXI=F*DWTAXI
!!!      DO 2 I=1,25
!!!      THRUST1(I)=F*THRUST1(I)
!!!      RAM1(I)=F*RAM1(I)
!!!      PARTH1(I)=F*PARTH1(I)
!!!      PARRAM1(I)=F*PARRAM1(I)
!!!      PARTH1(I+25)=F*PARTH1(I+25)
!!!      PARRAM1(I+25)=F*PARRAM1(I+25)
!!!    2 CONTINUE
      thrust1(:,:)=F*thrust1(:,:)
      ram1(:,:)=F*ram1(:,:)
      parth1(:,:)=F*parth1(:,:)
      parram1(:,:)=F*parram1(:,:)
      zw(:)=F*zw(:)

!!!      DO 4 I=1,5
!!!      ZW(I)=F*ZW(I)
!!!    4 CONTINUE
!
!          FT**2 TO M**2 ITEMS  (S2)
      IF (ISIN.EQ.1) F=S2
      IF (ISIN.EQ.2) F=1.0/S2
      REFA1=F*REFA1
      S=F*S
      CDOUTT(2)=F*CDOUTT(2)
      CDOUTT(4)=F*CDOUTT(4)
      CE(3)=F*CE(3)
      CE(8)=F*CE(8)
      CA(2)=F*CA(2)
      CA(4)=F*CA(4)
      CA(7)=F*CA(7)
!
!         FT TO M  ITEMS  (S3)
      IF (ISIN.EQ.1) F=S3
      IF (ISIN.EQ.2) F=1.0/S3
      ASPARE(7)=F*ASPARE(7)
      CE(4)=F*CE(4)
      CE(6)=F*CE(6)
      CE(9)=F*CE(9)
      CA(3)=F*CA(3)
      CA(5)=F*CA(5)
      CA(8)=F*CA(8)
      CA(10)=F*CA(10)
      CA(11)=F*CA(11)
      CA(13)=F*CA(13)
      CA(14)=F*CA(14)
!!!      DO 6 I=1,25
!!!      HENGT(I)=F*HENGT(I)
!!!      IF (ISPARE(2).EQ.2) TLVAR(I)=F*TLVAR(I)
!!!      IF (ISPARE(3).EQ.2) FPVAR(I)=F*FPVAR(I)
!!!    6 CONTINUE
      hengt(:,:)=F*hengt(:,:)
      IF (ispare(2)==2) tlvar(:)=F*tlvar(:)
      IF (ispare(3)==2) fpvar(:)=F*fpvar(:)

!!!      DO 8 I=1,5
!!!      HTAB1(I)=F*HTAB1(I)
!!!    8 CONTINUE
      htab1(:)=F*htab1(:)

      HNOGE=F*HNOGE
      REFB1=F*REFB1
      RZERO=F*RZERO
      GZERO=F*GZERO
      HOBS=F*HOBS
      HSTEP1=F*HSTEP1
      HSTEP2=F*HSTEP2
      STOPDIS=F*STOPDIS
      PINT1=F*PINT1
      PINT2=F*PINT2
      PINT3=F*PINT3
      CBALT=F*CBALT
      CBDIST=F*CBDIST
      CBLIMH=F*CBLIMH
      FPLIMH=F*FPLIMH
      HOBSL=F*HOBSL
      DOUT1=F*DOUT1
      DOUT2=F*DOUT2
      IF (ISIN.EQ.1) ZRANGE=ZRANGE*6076.*S3
      IF (ISIN.EQ.2.AND.ZRANGE.NE.0.0) ZRANGE=ZRANGE/(6076.*S3)
!
!          KT TO M/S  ITEMS  (S4)
      IF (ISIN.EQ.1) F=S4
      IF (ISIN.EQ.2) F=1.0/S4
      VSTEP1=F*VSTEP1
      VSTEP2=F*VSTEP2
      VSTEP3=F*VSTEP3
      VSTEP5=F*VSTEP5
      VFAILKT=F*VFAILKT
      DVCLIMB=F*DVCLIMB
      ARVEL1=F*ARVEL1
      ARVEL2=F*ARVEL2
      ASPARE(9)=F*ASPARE(9)
      DO 12 I=1,10
      VROTKT(I)=F*VROTKT(I)
   12 END DO
      DO 10 I=1,25
      IF (ISPARE(2).EQ.1) TLVAR(I)=F*TLVAR(I)
      IF (ISPARE(3).EQ.1) FPVAR(I)=F*FPVAR(I)
   10 END DO
!       PSF TO KPA  (S6)
!
      IF (ISIN.EQ.1) F=S6
      IF (ISIN.EQ.2) F=1.0/S6
      DO 14 I=1,25
   14 END DO
      WOSSLS=F*WOSSLS
!
!       LBM/H TO KG/S  (S7)
      IF (ISIN.EQ.1) F=S7
      IF (ISIN.EQ.2) F=1.0/S7
!!!      DO 16 I=1,25
!!!      FUEL1(I)=F*FUEL1(I)
!!!      PARWF1(I)=F*PARWF1(I)
!!!      PARWF1(I+25)=F*PARWF1(I+25)
!!!   16 CONTINUE
      fuel1(:,:)=F*fuel1(:,:)
      parwf1(:,:)=F*parwf1(:,:)
!
!         LBM TO KG  (S8)
      IF (ISIN.EQ.1) F=S8
      IF (ISIN.EQ.2) F=1.0/S8
      DO 18 I=1,25
   18 END DO
!       IN**2 TO M**2  (S9)
      IF (ISIN.EQ.1) F=S9
      IF (ISIN.EQ.2) F=1.0/S9
      DO 20 I=1,25
   20 END DO
!
!      DEG R  TO  DEG K    (1.8)
      IF (ISIN.EQ.1) F=1.0/1.8
      IF (ISIN.EQ.2) F=1.8
      CE(2)=F*CE(2)
      DO 22 I=1,25
   22 END DO
!
!       PER MIN TO PER SEC   (60)
      IF (ISIN.EQ.1) F=1.0/60.
      IF (ISIN.EQ.2) F=60.
      DO 24 I=1,25
   24 END DO
!
!      FT PER MIN  TO  M PER SEC    (S3/60.)
      IF (ISIN.EQ.1) F=S3/60.
      IF (ISIN.EQ.2) F=60./S3
      IF (IGRD1.EQ.1) GRAD1=F*GRAD1
      IF (IGRD2.EQ.1) GRAD2=F*GRAD2
!
  RETURN
END Subroutine ConSI   ! -------------------------------------------------------


!+
SUBROUTINE Cutback()
! ------------------------------------------------------------------------------
! PURPOSE - Compute data with power cutback.
!  The regulations of references 6 and 8 permit engine cutback for the purpose
!  of noise alleviation over the community. The engine throttle setting used 
!  during the takeoff must be maintained until the aircraft reaches an altitude
!  of 213 m (700 ft). (For aircraft with less than four engines, this altitude 
!  restriction is 305 m (1000 ft).)
!  Above this altitude, throttle reductions are limited by the required minimum
!  climb gradients. With all engines operating, a climb gradient of at least 
!  four percent must be maintained. In the event of an engine failure, the 
!  gradient must be equal to or greater than zero (level flight). The allowable
!  cutback is limited so that the engines will provide sufficient thrust to 
!  meet the most critical requirement.

!  Within the program, throttle cutback is initiated when the aircraft reaches 
!  either a desired altitude or a desired distance from brake release. If the 
!  desired distance is reached before the limiting altitude of 213 m (700 ft) 
!  is attained, climbout continues until that altitude is reached before cutback
!  is initiated. At cutback, the throttle setting, as limited by the gradients 
!  described above, is calculated, and climbout is continued with the reduced 
!  level of thrust.

!  The control logic within the program for continued climbout is to maintain
!  the cutback-throttle-setting climb-gradient. Due to the normal thrust decay 
!  with increasing altitude, the forward velocity will decrease slightly.


! NOTE - Called by Wef1041; calls Aero41 and Bilupv
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: anoe,cbcl,cbgam,cbpert,cbtt,cdoutt,delta,dist, &
  engsout,esf,gamma,h,grad1,grad2,ibug,igrd1,igrd2,isot,jcode,k,l,mach, &
  mpengt1,npi,npj,q,rortab1,s,s1,s3,s5,tdr,tff,tnotn,tottab1,tsinal,tt,v,w,wowtab1
IMPLICIT NONE

! LOCAL VARIABLES
  REAL(WP):: a3,a4,a5
  REAL(WP):: acbcl
  REAL(WP):: agam
  REAL(WP):: apert
  REAL(WP):: atacc
  REAL(WP):: atdr
  REAL(WP):: atff
  REAL(WP):: att
  REAL(WP):: bgam
  REAL(WP):: bpert
  REAL(WP):: btacc
  REAL(WP):: cbalf
  REAL(WP):: cbcd
  REAL(WP):: cbdrag
  REAL(WP):: cbh
  REAL(WP):: cbm
  REAL(WP):: cbq
  REAL(WP):: cbtacc
  REAL(WP):: cbtdr
  REAL(WP):: cbv
  REAL(WP):: cdeout
  REAL(WP):: cosalf
  REAL(WP):: cosgam
  REAL(WP):: denom
  REAL(WP):: dodr
  REAL(WP):: dot
  REAL(WP):: gam
  REAL(WP):: gans
  REAL(WP):: grad3
  REAL(WP):: gradt
  INTEGER:: icb
  INTEGER:: igrad
  REAL(WP):: prslope
  REAL(WP):: rcbpert
  REAL(WP):: sbsp10
  REAL(WP):: scbcl
  REAL(WP):: scbgam
  REAL(WP):: scbpert
  REAL(WP):: sinalf
  REAL(WP):: singam
  REAL(WP):: stnref
  REAL(WP):: tloss
  REAL(WP):: tnopr
  REAL(WP):: tnref
  REAL(WP):: tratio
  REAL(WP):: wdowd

!-------------------------------------------------------------------------------
  800 cbh=h(k)
      cbv=v(k)
      cbm=mach(k)
      cbq=q(k)
!        FOLLOWING ENG. DATA FOR T/T=1.0
      att=tt(k)
      atdr=tdr(k)
      stnref=att-atdr
      atff=tff(k)   ! never used
      icb=1
      tloss=1.0
      IF (ISOT.EQ.1) WRITE(6,801) CBM,CBH,W(K),DIST(L)
      IF (ISOT.EQ.2) WRITE(6,801) CBM,S3*CBH,S1*W(K),S3*DIST(L)
  801 FORMAT(3X,"***** POWER CUTBACK AT M=",F5.2,2X,",H=",F7.2,2X,       &
     & ",WT=",F8.0,2X,",DIST=",F7.0,43X,"F801"/)
      igrad=igrd1
      gradt=grad1

  802 apert=0.0
      atacc=0.0
!      IF  IGRAD=1,  GRADT IS R/C IN FPM.
      IF (IGRAD.EQ.1) GO TO 804
      IF (GRADT.EQ.0.0) GO TO 806
      gam=57.2958*ATAN(gradt)
      GO TO 805
  804 gam=57.2958*ASIN(gradt/(cbv*60.))
  805 cosgam=COS(gam/57.2958)
      singam=SIN(gam/57.2958)
      GO TO 807
  806 cosgam=1.0
      singam=0.0
      gam=0.0


  807 cbpert=1.0
      tnref=stnref*tloss
      IF (ICB.EQ.3) CBPERT=RCBPERT
      CBGAM=GAM
      IF (GAM.LE.GAMMA(L)) GO TO 808

      gam=gamma(l)
      gradt=TAN(gam/57.2958)
      cbgam=gam
      cbpert=1.0
      WRITE(6,803) GRADT
  803 FORMAT(3X,"**** INPUT GRADIENT OF",F7.3,2X,                        &
     & "CANNOT BE ATTAINED WITH POWER CUTBACK"/)
      GO TO 950


  808 cbtt=att*tloss*cbpert
      sinalf=0.0
      cosalf=1.0
      cbcl=(w(k)*cosgam)/(cbq*s)
      GO TO 812
  810 cbcl=acbcl
  812 cbcd=2.0
      cbalf=-999.0  ! will be overwritten, but Aero41 needs a value
!        CL KNOWN , LOOK UP ALPHA AND CD. (set cbcd=2.0)
      CALL Aero41(cbh,cbalf,cbcl,cbcd)
      IF (tsinal.eq.0.0) go to 814
      sinalf=SIN((cbalf+delta)/57.2958)
      cosalf=COS((cbalf+delta)/57.2958)
      acbcl=(w(k)*cosgam-cbtt*sinalf)/(cbq*s)
      IF (IBUG.NE.0) WRITE(6,813) cbpert,cbtt,cbcl,acbcl
  813 FORMAT(12X,"C813",4E15.8)
      IF (ABS(1.0-ACBCL/CBCL) > 0.0001) GO TO 810


  814 CDEOUT=0.0
      IF (icb.NE.2) GO TO 818
      IF (cdoutt(3).EQ.0.0) GO TO 817
      cdeout=cdoutt(3)*cdoutt(5)*esf/s
      GO TO 818
  817 IF (cdoutt(2).EQ.0.0) GO TO 818
      cdeout=cdoutt(1)*cdoutt(2)*esf/s


  818 cbdrag=(cbcd+cdeout*engsout)*cbq*s
      dot=cbpert
      CALL BILUPV(10,5,TOTTAB1,MPENGT1,RORTAB1,WOWTAB1,NPI,NPJ,DOT,CBM, &
     & DODR,WDOWD,A3,A4,A5) ! a3,a4,a5 never used

      cbtdr=atdr*tloss*dodr
      cbtacc=(cbtt*cosalf-cbtdr-cbdrag-w(k)*singam)/w(k)
      tnopr=cbtt-cbtdr
      tnotn=tnopr/tnref

      IF (IBUG.NE.0) WRITE(6,815) CBPERT,CBTT,ACBCL,CBTACC
  815 FORMAT(12X,"C815",4E15.8)
      IF (ABS(CBTACC).LT. 0.0001) GO TO 820
      IF (ICB.EQ.3) GO TO 930


!        assume two throttle settings and use slope tech. to
!     find reduction for zero accel.
      IF (APERT.EQ.0.0) GO TO 816
      BPERT=CBPERT
      BTACC=CBTACC
      PRSLOPE=(APERT-BPERT)/(ATACC-BTACC)
      CBPERT=BPERT-BTACC*PRSLOPE
      APERT=BPERT
      ATACC=BTACC
      GO TO 808
  816 APERT=CBPERT
      ATACC=CBTACC
      CBPERT=.90
      GO TO 808
  820 IF (ICB.EQ.3) GO TO 940
      IF (ICB.EQ.2) GO TO 840
      SCBPERT=CBPERT
      SCBGAM=CBGAM
      SCBCL=CBCL
      SBSP10=TNOTN
      WRITE(6,830) GRADT,CBGAM,CBCL,CBTT,CBPERT,TNOTN
  830 FORMAT(4X,"** ALL ENG GRAD=",F7.4,2X,"GAM=",F6.3,2X,"CL=",F7.4,  &
     & 2X,"THRUST=",F9.1,2X,"TGOTG=",F7.4,2X,"TNOTN=",F7.4,19X,"F830"/)

      ICB=2
      IGRAD=IGRD2
      GRADT=GRAD2
      TLOSS=(ANOE-1.)/ANOE
      GO TO 802

  840 WRITE(6,842) GRADT,CBGAM,CBCL,CBTT,CBPERT,TNOTN
  842 FORMAT(4X,"** ENG OUT GRAD=",F7.4,2X,"GAM=",F6.3,2X,"CL=",F7.4,2X,  &
     & "THRUST=",F9.1,2X, "TGOTG=",F7.4,2X, "TNOTN=",F7.4,19X, "F842"/)
      IF (JCODE.EQ.7) GO TO 900
      GO TO 950

  900 IF (SCBPERT.LT.CBPERT) GO TO 920
!        SELECT HIGHEST CBPERT
      CBPERT=SCBPERT
      CBGAM=SCBGAM
      CBCL=SCBCL
      TNOTN=SBSP10
      GO TO 950

  920 WRITE(6,922) CBPERT
  922 FORMAT(4X,"*** ENGINE OUT ESTABLISHES CUTBACK POWER=",F8.5,       &
     &64X,"F922"/)
      RCBPERT=CBPERT
      ICB=3
!      force rcbpert, bal. cl, iterate for actual gamma with all engines
      TRATIO=SCBPERT/RCBPERT
      DENOM=TRATIO-TLOSS
      GANS=((1.0-TLOSS)*SIN(SCBGAM/57.2958))/DENOM
      AGAM=57.2958*ASIN(GANS)
      GAM=AGAM
      TLOSS=1.0
      GO TO 805

  930 SINGAM=(CBTT*COSALF-CBTDR-CBDRAG)/W(K)
      BGAM=57.2958*ASIN(SINGAM)
      GAM=BGAM
      TLOSS=1.0
      GO TO 805

  940 GRAD3=TAN(GAM/57.2958)
      IF (ISOT.EQ.1) WRITE(6,942) GRAD3,GAM,CBCL,CBTT,RCBPERT,TNOTN
      IF (ISOT.EQ.2) WRITE(6,942) GRAD3,S5*GAM,CBCL,S1*CBTT,RCBPERT,TNOTN
  942 FORMAT(3X,"**** FINAL GRAD=",F7.4,2X,"GAM=",F6.3,2X,"CL=",F8.5,2X,  &
     & "THRUST=",F9.1,2X,"TGOTG=",F7.4,2X, "TNOTN=",F7.4,19X, "F942"/)
      CBPERT=RCBPERT
      CBGAM=GAM
      
  950 RETURN
END Subroutine Cutback   !------------------------------------------------------


!+
      SUBROUTINE EXTRA()
! ------------------------------------------------------------------------------
! PURPOSE - For temp use to interpolate engine data at selected 
!  mach/altitude/power combinations.
! NOTE - Called by Wef1041; calls Qant4
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: anoe,cdgrt,clgrt,engsout,esf,h,icode,isearch, &
  jcode,jtcode,k,loc,m,mach,power,tdr,tfact,tff,tnotn,tot,tt
IMPLICIT NONE

  INTEGER:: ic
  INTEGER:: ih
  INTEGER:: im
  INTEGER:: ip
  INTEGER:: is, ish,ism
  REAL(WP):: openg
  REAL(WP):: sfc   ! ???????
  REAL(WP):: sumtfac
  REAL(WP):: tffsls
  REAL(WP):: tgosls
  REAL(WP):: tnosls
  REAL(WP):: ttgsls
  REAL(WP):: ttn
  REAL(WP):: ttnsls

!-------------------------------------------------------------------------------

!      SET UP TO LOOP THRU MACH/ALT. ARRAYS FOR EACH INPUT POWER.
!      LOOPS LIMITED TO 15 VALUES.  MACH=CLGRT,  H=CDGRT.
!
      ICODE=1041
      M=1
      IC=0   !!! ic set, but never used
      K = 1
      JCODE=7
!        CALC. SLS DATA.
      OPENG=ANOE-ENGSOUT
      WRITE(6,60) OPENG
   60 FORMAT(4X,                                                       &
       "FOLLOWING ENGINE DATA INTERPOLATED IN 1041. DATA IS FOR ",     &
     & F5.2," ENGINES."/ 4X,                                           &
     & "DATA HAS BEEN SIZED AND MODIFIED BY INPUTS (ESF,TFACT,AREXT,ETC.)."//)
      JTCODE=0
      TOT=1.0
      MACH(1)=0.0
      H(1)=0.0
      CALL QANT4
      IF (LOC.EQ.1000) GO TO 500   ! just return
      
      TTGSLS=TT(1)
      TTNSLS=TT(1)-TDR(1)
      TFFSLS=TFF(1)
      WRITE(6,80) TTGSLS,TTNSLS,TFFSLS
   80 FORMAT(6X,"FULL POWER, SLS DATA.",2X,"TTGSLS=",F7.0,              &
     &2X,"TTNSLS=",F7.0,2X,"TFFSLS=",F7.0)
      IF (ENGSOUT.NE.0.0) WRITE(6,81) ENGSOUT
   81 FORMAT(85X,"**** ENGSOUT=",F7.4,1X,"****")
   
      SUMTFAC=0.0
      DO 82 IS=1,5
      SUMTFAC=SUMTFAC+TFACT(IS)
   82 END DO
      WRITE(6,85) ESF,SUMTFAC
   85 FORMAT(24X,"( ESF=",F7.4,2X,",SUM OF TFACT(1-5)=",F7.4," )"///)
   
      IF (ISEARCH.EQ.10) WRITE(6,90)
   90 FORMAT(3X,"MACH",3X,"ALT.",3X,"TGOTG",4X,"TTG",5X,"TDR",5X,"TFF", &
     &7X,"TTN",5X,"SFC",4X,"TNOTN",9X,"TG/SLS",2X,"TN/SLS"/)
     
  100 DO 400 IP=1,15   ! start power loop
      TOT=POWER(IP)
      IF (POWER(IP).EQ.0.0) GO TO 500   ! just return
      
      IF (IP.EQ.1) GO TO 110
      IF (POWER(IP).EQ.POWER(IP-1)) GO TO 500
  110 CONTINUE
  
      IF (TOT.NE.1.0) JTCODE=1
      IF (TOT.EQ.1.0) JTCODE=0
      IF (TOT.EQ.1.0) TNOTN=1.0
      
      DO 300 IM=1,15   ! start Mach loop
      ISM=IM
      MACH(1)=CLGRT(IM)
      IF (IM.EQ.1) GO TO 140
      IF (CLGRT(IM).EQ.CLGRT(IM-1)) GO TO 380 ! ignore remaining Machs
      IF (IM.GE.2.AND.CLGRT(IM).EQ.0.0) GO TO 380   ! ignore remaining Machs
  140 CONTINUE
  
      WRITE(6,142)
  142 FORMAT(/)

      DO 200 IH=1,15   ! start altitude loop
      ISH=IH
      H(1)=CDGRT(IH)
      
      IF (IH.EQ.1) GO TO 150
      IF (CDGRT(IH).EQ.CDGRT(IH-1)) GO TO 301   ! ignore remaining altitudes
      IF (IH.GE.2.AND.CDGRT(IH).EQ.0.0) GO TO 301   ! ignore remaining altitudes
  150 CONTINUE
  
      CALL QANT4()
      IF (LOC.EQ.1000) GO TO 500    ! loc returned by Ftlup2
      
      TTN=TT(1)-TDR(1)
      TGOSLS=TT(1)/TTGSLS
      TNOSLS=TTN/TTNSLS
      WRITE(6,160)MACH(1),H(1),TOT,TT(1),TDR(1),TFF(1),TTN,             &
     & SFC,TNOTN,TGOSLS,TNOSLS
       !!! warning on line above:   sfc never set
  160 FORMAT(3X,F5.3,2X,F5.0,2X,F6.4,2X,F7.0,2X,F6.0,2X,F7.0,           &
     &2X,F7.0,2X,F5.3,2X,F6.4,9X,F6.4,2X,F6.4/)
  200 END DO   !      ALT. LOOP COMPLETED.

  301 CONTINUE
  300 END DO   !      M/ALT. LOOP COMPLETED.
  
  380 IC=ISM+ISH

  400 END DO   ! power loop completed
  
  500 RETURN
END Subroutine Extra   ! -------------------------------------------------------


!+
      SUBROUTINE QANT4()
! ------------------------------------------------------------------------------
! PURPOSE - Look up v,q,low,tt,tdr,and tff for flight mach,h, and power.
!   As of 3/77 qant4 can also be used to throttle engine data.
! NOTE - called by Wef1041, Approach, ClimbGradients, Extra, Refused
!        Calls At62sp, Bilupv, Ftlup2
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: amach,anoe,ans,aspare,cspare,drfac,drfact,drone, &
  dropt1,dtemp,engsout,esf,h,hengt,ibal,icode,isearch,jcode,jtcode,loc, &
  m,mach,mengt, &
  mpengt1,nei,nej,npi,npj,pbtab,q,rortab1,sfcfact,tdr,tfac,tfact,tff,tnotn, &
  tone,topt1,tot,tottab1,tt,v,wfopt1,wowtab1
IMPLICIT NONE

  REAL(WP):: a3,a4,a5
  REAL(WP):: dodr
  REAL(WP):: drop
  REAL(WP):: eror
  REAL(WP):: eslp
  REAL(WP):: etot
  REAL(WP):: ewow
  REAL(WP):: hval
  REAL(WP):: sfcfac
  REAL(WP):: tmac
  REAL(WP):: tnopr
  REAL(WP):: tnref
  REAL(WP):: top
  REAL(WP):: wdowd
  REAL(WP):: wfop
  REAL(WP):: zft
!-------------------------------------------------------------------------------
!     WRITE(6,1) TOT
!   1 FORMAT(3X,"** IN QANT4, TOT=",F7.4)
      ZFT=H(M)
      CALL AT62SP(ZFT,ANS,DTEMP,PBTAB)
      IF (ISEARCH.LT.10) MACH(M)=V(M)/ANS(4)
      IF (ISEARCH.GE.10) V(M)=MACH(M)*ANS(4)
      Q(M)=.5*ANS(1)*V(M)**2
      IF (ICODE.EQ.12) RETURN
      IF (ICODE.EQ.32) RETURN

      HVAL=H(M)
      AMACH=MACH(M)
      CALL BILUPV(5,5,HENGT,MENGT,TOPT1,WFOPT1,NEI,NEJ,HVAL,AMACH,      &
     &TOP,WFOP,A3,A4,A5)
      CALL BILUPV(5,5,HENGT,MENGT,TOPT1,DROPT1,NEI,NEJ,HVAL,AMACH,      &
     &TOP,DROP,A3,A4,A5)
      DODR=1.0
      WDOWD=1.0
      IF (JTCODE.EQ.0) GO TO 28
      IF (ICODE.EQ.9) GO TO 28
      TMAC=AMACH
      IF (TOT.LE.1.0) GO TO 25


!... Following equations simulate engine data extended beyond maximum 
!.   input power setting (tot > 1.0). ** ratios are used at all flight conditions
      IF (ASPARE(13).GT.1.0) GO TO 20
      TOT=1.0
      GO TO 28
   20 ETOT=TOT
      ESLP=(ETOT-1.0)/(ASPARE(13)-1.0)
      EWOW=1.0+ESLP*(ASPARE(14)-1.0)
      EROR=1.0+ESLP*(ASPARE(15)-1.0)
      WDOWD=EWOW
      DODR=EROR
      GO TO 28

   25 IF (TOT.EQ.1.0) GO TO 28
      CALL BILUPV(10,5,TOTTAB1,MPENGT1,RORTAB1,WOWTAB1,NPI,NPJ,TOT,TMAC, &
     & DODR,WDOWD,A3,A4,A5)
   28 CSPARE(5)=DODR   !      cspare(5) saved for use in Refused.
      LOC=11
      CALL FTLUP2(AMACH,TFAC   ,+1,NEJ,MENGT,TFACT  ,LOC)
      CALL FTLUP2(AMACH,SFCFAC ,+1,NEJ,MENGT,SFCFACT,LOC)
      CALL FTLUP2(AMACH,DRFAC  ,+1,NEJ,MENGT,DRFACT ,LOC)
      IF (LOC.EQ.1000) RETURN

!     WRITE(6,100) ZFT,TOT,DODR,WDOWD
! 100 FORMAT(9X,"* AT H=",F7.2," TOT=",F7.4," DODR=",F7.4," WDOWD=",F7.4)
      IF (ICODE.EQ.9) GO TO 30
      IF (ICODE.EQ.10) GO TO 29
      IF (JCODE.EQ.6) GO TO 29
      IF (JCODE.EQ.7) GO TO 30
      IF (IBAL.NE.0.AND.JCODE.GE.4) GO TO 29

   30 TT(M)=TOP*ANS(2)*(ANOE-ENGSOUT)*ESF*TFAC
      TDR(M)=DROP*ANS(2)*(ANOE-ENGSOUT)*ESF*DRFAC
      TFF(M)=WFOP*ANS(2)*(ANOE-ENGSOUT)*ESF*TFAC*SFCFAC
      TNREF=TT(M)-TDR(M)
      IF (JTCODE.EQ.0) RETURN
      IF (ICODE.EQ.9) RETURN
      TT(M)=TT(M)*TOT
      TDR(M)=TDR(M)*DODR
      TFF(M)=TFF(M)*WDOWD
      TNOPR=TT(M)-TDR(M)
      TNOTN=TNOPR/TNREF
      RETURN

   29 IF (JCODE.EQ.5) RETURN    
!!!      TONE=TOP*ANS(2)*ESF*TFACT*TOT
  TONE=TOP*ANS(2)*ESF*TFACT(1)*TOT   ! only way to get here is (icode==10 AND jcode/=5) OR jcode==6
  DRONE=DROP*ANS(2)*ESF*DRFAC*DODR
      
  RETURN
END Subroutine Qant4   ! -------------------------------------------------------


!+
SUBROUTINE Refused()
! ------------------------------------------------------------------------------
! PURPOSE - Compute refused takeoff distance.
!  The engine-out refused-takeoff distance is calculated considering several
!  events. From the time of the engine failure, there is a time interval 
!  referred to as the "pilot recognition time" before any action is taken by the
!  aircraft crew. After this time interval, the thrust level of the operating 
!  engines is reduced to the ground-idle thrust condition. The thrust is reduced
!  linearly with time over a specified time interval. The remainder of the 
!  refused takeoff is calculated at this reduced thrust level. No thrust 
!  reversal or other external drag effects due to engine operation are 
!  considered. The application of wheel brakes and of spoilers to decrease lift
!  and increase drag is controlled by separate time intervals, each measured 
!  from the time of pilot recognition. The distance required for the aircraft to
!  come to a complete stop is called the "refused-takeoff distance".

! NOTE - Called by Wef1041; calls Aero41,Ftlup2,Qant4
USE TolConstants,ONLY: WP
USE TolCommonVariables,ONLY: alpha,amach,anoe,ans,cdd,cdgrt,cdoutt,clgrt, &
  conv3,cl,cspare,dcds,dcls,delta,dist,drag,drfac,drone,&
  dtbra,dtfail,dtfar,dtftd,dtpow,dtrec,dtspo,dttr,engsout,esf, &
  event,flap,gamma,gzero,h,ibug,icode,iprint,ispare,isot,jcode,jtcode,k,l,   &
  lift,loc,lod,m,mach,mubrake,muroll,nacc,power,q,refa1,refrho,rzero
USE TolCommonVariables,ONLY:   s,s1,s3,s4,s5,s6,s7, &
  tacc,tdidle,tdr,tfac,tidle,tff,time,tlvar,tmd,tnotn,toflp, &
  tone,tot,trdt,trev,tsinal,tt,v,vkt,vstep5,w
IMPLICIT NONE

  REAL(WP):: alph
  REAL(WP):: avff
  REAL(WP):: avgnacc
  REAL(WP):: avgtacc
  REAL(WP):: avh
  REAL(WP):: avvel
  REAL(WP):: cd
  REAL(WP):: cdengf
  REAL(WP):: cld
  REAL(WP):: cosalf
  REAL(WP):: cosgam
  REAL(WP):: ddist
  REAL(WP):: deltime
  REAL(WP):: delvel
  REAL(WP):: dsec
  REAL(WP):: dvel
  REAL(WP):: enoutf
  REAL(WP):: gearcd
  REAL(WP):: halt
  INTEGER:: ic
  INTEGER:: iprop
  INTEGER:: iq5
  INTEGER:: ir
  INTEGER:: iterv
  INTEGER:: itot
  INTEGER:: itr
  INTEGER:: je
  INTEGER:: jsjtcd
  INTEGER:: ksave
  INTEGER:: lsave
  INTEGER:: me
  REAL(WP):: mustep
  INTEGER:: ne
  REAL(WP):: reftm
  REAL(WP):: sdodr
  REAL(WP):: sinalf
  REAL(WP):: singam
  REAL(WP):: spoil
  REAL(WP):: sqrho
  REAL(WP):: stot
  REAL(WP):: tdfeng
  REAL(WP):: tdmod
  REAL(WP):: tdoeng
  REAL(WP):: temp
  REAL(WP):: testm
  REAL(WP):: tfeng
  REAL(WP):: tgotg
  REAL(WP):: tmod
  REAL(WP):: tmult
  REAL(WP):: tnum
  REAL(WP):: toeng
  REAL(WP):: trdec
  REAL(WP):: vkeas
  REAL(WP):: wac

!-------------------------------------------------------------------------------      
      iq5=0
      iprop=0
      IF (CDOUTT(3).NE.0.0 .AND. TREV.EQ.0.0) IPROP=1
      IF (CDOUTT(3).NE.0.0 .AND. TREV.NE.0.0) IPROP=2
      ITR=0
!        NEW FAR .  NO RETARDING OPERATION CAN BE INITIATED
!              UNTIL ACCEL. AFTER VEF.(DTFAR=2.0 SEC.)
!          AUTOMATIC PROPELLER FEATHERING IS ASSUMED.
!         COMPUTE EVENTS FROM FAILURE POINT
      event(1)=dtfail
      event(2)=dtrec
      event(3)=dtftd
      event(4)=dtrec+dtfar
      event(5)=event(4)+dtpow
      event(6)=event(4)+dtbra
      event(7)=event(4)+dtspo
      event(8)=event(4)+dttr
      event(9)=event(4)+dttr+trdt/2.0
      event(10)=event(4)+dttr+trdt
      event(11)=0.0
      event(12)=0.0

! the above events may not be in chronological order.and may have duplicates
  DO ne=1,11
    je=ne+1
    DO  me=je,12
      IF (event(ne) .LE. event(me)) CYCLE   !!! go to 902
      temp=event(ne)
      event(ne)=event(me)
      event(me)=temp
    END DO
  END DO

!        SELECT EVENTS IN CHRON. ORDER and distinct
      ic=0  ! ic is the number of distinct times
      DO 904 IR=1,12
      IF (EVENT(IR).LE.0.0) CYCLE   !!! GO TO 904
      IF (IR.EQ.0) GO TO 903
      IF (EVENT(IR).EQ.EVENT(IR-1)) CYCLE   !!! GO TO 904
  903 IC=IC+1
      EVENT(IC)=EVENT(IR)
  904 END DO
  
!        events (non-zero, non-recurring) now in order
      JSJTCD=JTCODE
      DO 990 L=1,IC   ! loop on number of events
      K=L+1
      ITERV=1
      TNOTN=0.0
      TRDEC=0.0  ! never used ???
      TONE=0.0
      ITOT=0
      REFTM=EVENT(L)

      IF (L.EQ.1) GO TO 905
      DELTIME=EVENT(L)-EVENT(L-1)
      GO TO 906
  905 DELTIME=EVENT(L)

  906 DELVEL=TACC(L)*DELTIME*GZERO
      IF (ISPARE(8).NE.0) GO TO 913
      IF (ISPARE(5).EQ.0.OR.IPRINT.EQ.0) GO TO 908
  913 IF (L.GT.1) GO TO 852
      IF (ICODE.NE.10) WRITE(6,911) TIME(1)
  911 FORMAT(4X, &
        "REFTM IS TIME INTERVAL IN SECONDS AFTER FAILURE TIME (=", F6.2, " ).")
      IF (ICODE.EQ.10) WRITE(6,850) TIME(1)
  850 FORMAT(4X,"REFTM IS TIME INTERVAL IN SECONDS FROM START OF ROLL (=", F6.2, " ).")

  852 WRITE(6,907) REFTM,DELTIME
  907 FORMAT(24X,"REFTM=",F8.4,4X,"DELTIME=",F8.4,58X,"F907")

  908 ALPHA(K)=ALPHA(L)
      LSAVE=L
      KSAVE=K
      H(K)=H(L)
      ALPH=ALPHA(K)
      GAMMA(K)=GAMMA(L)
      SINGAM=0.0
      COSGAM=1.0
      W(K)=W(L)
  909 V(K)=V(L)+DELVEL
      M=K
!       statements to 928 develop appropriate eng. thrust.
!    logic for turbojet and propeller separate to facilitate changes.
      TOT=1.0
      IF (ITR.EQ.1 .OR. IPROP.GE.1) GO TO 222
      IF (JCODE.EQ.5) GO TO 222
      IF (JTCODE.EQ.0) GO TO 222
      VKT(K)=V(K)/CONV3
      IF (ISPARE(2).EQ.0) GO TO 222

      LOC=21
      IF (ISPARE(2).EQ.1) CALL FTLUP2(VKT(K),TOT,+1,25,TLVAR,POWER,LOC)
      IF (ISPARE(2).EQ.2) CALL FTLUP2(H(K)  ,TOT,+1,25,TLVAR,POWER,LOC)
      IF (LOC.EQ.1000) GO TO 998   ! loc=1000 is Ftlup2 failure


  222 CALL QANT4()
      IF (LOC.EQ.1000) GO TO 998
      IF (JTCODE.EQ.0) GO TO 164
      IF (ITOT.EQ.1) GO TO 162
      STOT=TOT   ! tot at time of recogn. Used as base for remaining operations.
      SDODR=CSPARE(5)
      ITOT=1
  162 TONE=TONE*STOT   !    tone=thrust of one sized engine.
      DRONE=DRONE*SDODR
  164 CONTINUE
      IF (ISPARE(7).EQ.0.) GO TO 166
      IF (IQ5.EQ.0) WRITE(6,265) TOT
  265 FORMAT(4X,"QANT4 AT 265",2X,"TOT=",F7.4)
      IQ5=1
  166 CONTINUE
      TGOTG=TOT
      IF (LOC.EQ.1000) GO TO 998
      AMACH=MACH(K)

      IF (DTFAIL.NE.0.0) GO TO 910
      ENOUTF=1.0
      GO TO 912
  910 ENOUTF=REFTM/DTFAIL
      IF (ENOUTF.GE.1.0) ENOUTF=1.0

  912 TFENG=TONE*(1.0-ENOUTF)   !      tfeng=thrust of failing engine.
      TDFENG=DRONE*(1.0-ENOUTF)
      IF (IPROP.GE.1) GO TO 800

!      FOLLOWING STATS. TO 748, FOR TJ OPERATION.
      TESTM=DTREC+DTFAR
      IF (TREV.EQ.0.0) GO TO 740

!       statments to 718 consider turbojet with a thrust reverser
      IF (ITR.EQ.1) GO TO 715
      TMULT=1.0
      IF (REFTM.LE.(TESTM+DTTR)) GO TO 718
      IF (REFTM.LE.(TESTM+DTTR+TRDT/2.0)) GO TO 717
      IF (REFTM.LE.(TESTM+DTTR+TRDT)) GO TO 716
      ITR=1
  715 TMULT=-1.0*TREV
      GO TO 718
  716 TMULT=0.0
      TNUM=REFTM-(TESTM+DTTR+TRDT/2.0)
      IF (TNUM.LE.0.0) GO TO 718
      TMULT=-1.0*TREV*TNUM/(TRDT/2.0)
      IF (ABS(TMULT).GT.TREV) TMULT=-1.0*TREV
      GO TO 718
  717 TNUM=REFTM-(TESTM+DTTR)
      TMULT=1.0-TNUM/(TRDT/2.0)
  718 TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TONE*TMULT
      TDR(K)=TDFENG*ENGSOUT+(ANOE-ENGSOUT)*DRONE*TMULT
      GO TO 928

  740 TMULT=1.0
!      statments to 748 for a turbojet without a thrust reverser.
      IF (REFTM.LE.TESTM) GO TO 748
      IF (REFTM.LE.(TESTM+DTPOW)) GO TO 746
      TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TIDLE*ESF*TFAC
      TDR(K)=TDFENG*ENGSOUT+(ANOE-ENGSOUT)*TDIDLE*ESF*DRFAC
      GO TO 928
  746 TMOD=TONE-(TIDLE*ESF*TFAC)
      TDMOD=DRONE-(TDIDLE*ESF*DRFAC)
      TOENG=TONE-TMOD*((REFTM-TESTM)/DTPOW)
      TDOENG=DRONE-TDMOD*((REFTM-TESTM)/DTPOW)
      TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TOENG
      TDR(K)=TDFENG*ENGSOUT+(ANOE-ENGSOUT)*TDOENG
      GO TO 928

  748 TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TONE*TMULT
      TDR(K)=TDFENG*ENGSOUT+(ANOE-ENGSOUT)*DRONE*TMULT
      GO TO 928

  800 TESTM=DTREC+DTFAR
!     following statments for propeller operation.
!        turboprop engine data assumed to be in same
!        format as turbojet data.
      TDR(K)=0.0
      IF (IPROP.EQ.1) GO TO 840   ! iprop=1 means no thrust reversing
      IF (ITR.EQ.1) GO TO 815
      TMULT=1.0
      IF (REFTM.LE.(TESTM+DTTR)) GO TO 818
      IF (REFTM.LE.(TESTM+DTTR+TRDT/2.0)) GO TO 817
      IF (REFTM.LE.(TESTM+DTTR+TRDT)) GO TO 816
      ITR=1
  815 TMULT=-1.0*TREV
      GO TO 818
  816 TMULT=0.0
      TNUM=REFTM-(TESTM+DTTR+TRDT/2.0)
      IF (TNUM.LE.0.0) GO TO 818
      TMULT=-1.0*TREV*TNUM/(TRDT/2.0)
      IF (ABS(TMULT).GT.TREV) TMULT=-1.0*TREV
      GO TO 818
  817 TNUM=REFTM-(TESTM+DTTR)
      TMULT=1.0-TNUM/(TRDT/2.0)
      GO TO 818
  818 TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TONE*TMULT
      GO TO 928

  840 TMULT=1.0
!      following stats. for prop. with no tr.  assume time
!       to reduce prop. thrust to idle is at least 5 sec.
      IF (DTPOW.LT.5.0) DTPOW=5.0
      IF (REFTM.LE.TESTM) GO TO 848
      IF (REFTM.LE.(TESTM+DTPOW)) GO TO 846
      TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TIDLE*ESF*TFAC
      GO TO 928
  846 TMOD=TONE-(TIDLE*ESF*TFAC)
      TOENG=TONE-TMOD*((REFTM-TESTM)/DTPOW)
      TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TOENG
      GO TO 928
  848 TT(K)=TFENG*ENGSOUT+(ANOE-ENGSOUT)*TONE*TMULT
      GO TO 928


! there are many jumps to statement 928
  928 TFF(K)=0.0
      IF (ISPARE(8).EQ.1)  &
        WRITE(6,20) k,tt(k),tfeng,tone,tidle,reftm,engsout,tmult
   20 FORMAT(12X,"K=",I2,2X,"TT(K)=",F9.0,2X,"TFENG=",F9.0,                &
     &2X," TONE=",F9.0,2X,"TIDLE=",F9.0/14X, "REFTM=",F6.2,                 &
     &7X, "ENGSOUT=",F7.3,4X,"TMULT=",F7.3,52X,"F20")

!     statements to 944 develop appropriate cl and cd.
!        as of 4-26-78 flap variations possible. at present flaps
!        will be set=toflp input during refused t.o. calc.
      HALT=H(K)
      FLAP=TOFLP
      IF (ISPARE(3).NE.0) WRITE(6,200) FLAP
  200 FORMAT(2X,"*** DURING REFUSED T.O. CALCS., FLAPS SET =",F6.2/)


!        alpha known, look up cl and cd. (set cd=1.0)
      cld=-999.0   ! will be overwritten, but Aero41 needs a value
      CD=1.0
      CALL AERO41(HALT,ALPH,CLD,CD)
      CL(K)=CLD
      SPOIL=1.0
      IF (REFTM.LE.(DTREC+DTFAR+DTSPO)) SPOIL=0.0

  934 CL(K)=CL(K)+DCLS*SPOIL
      CD=CD+DCDS*SPOIL

      SINALF=0.0
      COSALF=1.
      IF (TSINAL.EQ.0.0) GO TO 938
      SINALF=SIN((ALPHA(K)+DELTA)/57.2958)
      COSALF=COS((ALPHA(K)+DELTA)/57.2958)
  938 CONTINUE

      LOC=7
      CALL FTLUP2(  CL(K),GEARCD,+1,15,CLGRT,CDGRT,LOC)
      IF (LOC.EQ.1000) GO TO 998

      CD=CD+GEARCD*REFA1/S
      CDENGF=0.0
!       SIMILAR LOGIC FOR ENGINE OUT T.O. NEAR S1041.920
      IF (CDOUTT(3).EQ.0.0) GO TO 144

      IF (REFTM.GT.DTFTD) GO TO 142
      CDENGF=CDOUTT(3)*CDOUTT(4)*ESF/S
      GO TO 145
  142 CDENGF=CDOUTT(3)*CDOUTT(5)*ESF/S
      GO TO 145

  144 IF (CDOUTT(2).EQ.0.0) GO TO 145
      CDENGF=CDOUTT(1)*CDOUTT(2)*ESF/S

  145 CD=CD+CDENGF*ENGSOUT
      CDD(K)=CD
      LOD(K)=CL(K)/CDD(K)
      LIFT(K)=CL(K)*Q(K)*S
      DRAG(K)=CDD(K)*Q(K)*S
      WAC=W(K)-LIFT(K)-TT(K)*SINALF
      IF (WAC.LT.0.0) WAC=0.0
      MUSTEP=MUBRAKE
      IF (REFTM.LE.(DTREC+DTFAR+DTBRA)) MUSTEP=MUROLL 


  944 CONTINUE
!       calc. point accelerations, average with those of
!        previous point.  calc. dvel and iterate.
      TMD(K)=TT(K)*COSALF-TDR(K)-DRAG(K)-MUSTEP*WAC-W(K)*SINGAM
      TACC(K)=TMD(K)/W(K)
      NACC(K)=(TT(K)*SINALF+LIFT(K)-W(K)*COSGAM)/W(K)
      AVFF=(TFF(L)+TFF(K))/2.   ! never used ???
      AVGTACC=(TACC(L)+TACC(K))/2.
      AVGNACC=(NACC(L)+NACC(K))/2.   ! never used ???
      AVVEL=(V(K)+V(L))/2.
      AVH=(H(K)+H(L))/2.
      IF (JCODE.EQ.5) GO TO 950

      DVEL=DELTIME*AVGTACC*GZERO
      IF (IBUG.EQ.0) GO TO 946

      WRITE(6,5009) ITERV,REFTM,DELVEL,DVEL
 5009 FORMAT(13X,I4,3E15.8)
  946 IF (ABS(DELVEL-DVEL).LT.0.05) GO TO 960
      ITERV=ITERV+1
      IF (ITERV.GT.5) GO TO 992
      DELVEL=DVEL
      GO TO 909

  950 DELTIME=DELVEL/(AVGTACC*GZERO)
      GO TO 960
      
!        iterations completed, update variables and print
  960 DSEC=DELTIME
      TIME(K)=TIME(L)+DSEC
      DDIST=AVVEL*DSEC*COSGAM*RZERO/(RZERO+AVH)
      VKT(K)=V(K)/CONV3
      IF (REFTM.EQ.DTREC) CSPARE(2)=VKT(K)
!         CSPARE(2) SAVED FOR USE IN MAIN
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      DIST(K)=DIST(L)+DDIST
      IF (ISPARE(5).EQ.0) GO TO 985
      IF (IPRINT.NE.1) GO TO 976
      
      IF (ISOT.EQ.1)WRITE(6,930) K,VKT(K),MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,930)K,S4*VKT(K),MACH(K),S3*H(K),            &
         S1*W(K),S3*DIST(K),TIME(K)
  930 FORMAT(2X,I3,2X,"V TAS=",F6.2,5X,"M=",F6.3,4X,"H=",F7.2,5X,"W=",F8.0, &
     & 2X,"DIST=",F7.0,2X,"TIME=",F7.2,29X,"F930"/)
      GO TO 980
      
  976 IF (IPRINT.LT.2) GO TO 980

      IF (ISOT.EQ.1) WRITE(6,949)K,VKT(K),MACH(K),H(K),W(K),DIST(K),      &
     &TIME(K),VKEAS,Q(K),ALPHA(K),CL(K),CDD(K),DRAG(K),LOD(K),          &
     &TT(K),TDR(K),TFF(K),TACC(K),NACC(K),GAMMA(K)

      IF (ISOT.EQ.2)WRITE(6,949)K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),     &
     &S3*DIST(K),TIME(K),S4*VKEAS,S6*Q(K),S5*ALPHA(K),CL(K),CDD(K),     &
     &S1*DRAG(K),LOD(K),S1*TT(K),S1*TDR(K),S7*TFF(K),TACC(K),           &
     &NACC(K),S5*GAMMA(K)

  949 FORMAT(2X,I3,2X,"V TAS=",F6.2,5X,"M=",F6.3,4X,"H=",F7.2,5X,"W=",F8.0, &
     &2X,"DIST=",F7.0,2X,"TIME=",F7.2/7X,"V EAS=",F6.2,                    &
     &2X,"Q=",F7.2,1X,"ALPHA=",F5.2,3X,"CL=",F8.5,5X,"CD=",F8.5,            &
     &3X,"DRAG=",F8.0,2X,"L/D=",F5.2/17X,"TT=",F8.0,1X,"TDR=",F8.0,1X,      &
     &"TFF=",F8.0,1X,"TACC=",F7.4,1X,"NACC=",F7.4,1X,"GAMMA=",F6.3,         &
     &28X,"F949")

      IF (IPRINT.GE.2) WRITE(6,240) TONE,TOT,ENOUTF,TMULT,TT(K)
  240 FORMAT(11X," TONE=",F7.0,2X,"TOT=",F7.4,2X,"ENOUTF=",F6.3,           &
     &2X,"TMULT=",F6.3,2X,"TT(K)=",F7.0,45X,"F240")

  980 IF (JSJTCD.EQ.0.AND.ISPARE(3).EQ.0) GO TO 320
      IF (JSJTCD.NE.0.AND.ISPARE(3).NE.0) GO TO 318
      IF (JSJTCD.NE.0) WRITE(6,314) TGOTG,TNOTN
  314 FORMAT(14X,"** TGOTG=",F7.4,5X,"TNOTN=",F7.4,69X,"F314")
      IF (ISPARE(3).NE.0) WRITE(6,316) FLAP
  316 FORMAT(14X,"**",37X,"FLAP=",F5.2,54X,"F316")
      GO TO 320
      
  318 WRITE(6,319) TGOTG,TNOTN,FLAP
  319 FORMAT(14X,"** TGOTG=",F7.4,5X,"TNOTN=",F7.4,5X,"FLAP=",F5.2,54X,"F319")
  320 WRITE(6,322)
  322 FORMAT(/)
  985 IF (JCODE.EQ.5) GO TO 994
  990 END DO
!    ***  990 IS END OF MAIN LOOP  ***



  994 JCODE=5
      L=LSAVE
      K=KSAVE
      IF (VKT(K).LE.0.02) GO TO 998
      IF (L.GT.IC) GO TO 995
      DELVEL=CONV3*(VSTEP5*REAL(INT(VKT(K)/VSTEP5))-VKT(K))
      GO TO 996

  995 DELVEL=-VSTEP5*CONV3
      IF ((V(K)+DELVEL).GT.0.0) GO TO 996
      DELVEL=CONV3*(.01-VSTEP5)
      
  996 L=K
      K=L+1
!!!      GO TO 908
      GO TO 998   ! code added by RLC. Just guessing that this was
                  ! meant instead of the previous line.
  992 JCODE=10
  998 JTCODE=JSJTCD
      IF (VKT(K).GT.0.02) JCODE=10
      
  RETURN
END Subroutine Refused   ! -----------------------------------------------------



END Module TolProcedures   ! ===================================================







!+
PROGRAM WEF1041
! ------------------------------------------------------------------------------
USE TolConstants,ONLY: WP, IN,OUT,DBG
USE TolCommonVariables
USE TolProcedures
IMPLICIT NONE

  REAL(WP),DIMENSION(50):: c1,c2,c3,c4,c5,c6,c7,c8,c9,c10
  REAL(WP),DIMENSION(50):: c11,c12,c13,c14,c15,c16,c17,c18,c19,c20
  REAL(WP),DIMENSION(60):: et,eh,ex,ev,eg,ea,ew,ef,ecl,eld,edg,etn,esp

  REAL(WP):: a3,a4,a5
  REAL(WP):: acl
  REAL(WP):: acon
  REAL(WP):: adelh
  REAL(WP):: adg
  REAL(WP):: adh
  REAL(WP):: agamma
  REAL(WP):: ak
  REAL(WP):: alflof
  REAL(WP):: alfmin
  REAL(WP):: alph
  REAL(WP):: altk
  REAL(WP):: altl
  REAL(WP):: altlim
  REAL(WP):: anacc
  REAL(WP):: apprdt
  REAL(WP):: apprdx
  REAL(WP):: apprrs
  REAL(WP):: ar
  REAL(WP):: asingam
  REAL(WP):: atgrup
  REAL(WP):: atimek
  REAL(WP):: atrtime
  REAL(WP):: avff
  REAL(WP):: avggam
  REAL(WP):: avghdot
  REAL(WP):: avgnacc
  REAL(WP):: avgtacc
  REAL(WP):: avh
  REAL(WP):: avvel
  REAL(WP):: bcl
  REAL(WP):: bdg
  REAL(WP):: calgam
  REAL(WP):: cbm
  REAL(WP):: cd
  REAL(WP):: cdengf
  REAL(WP):: cdmin
  REAL(WP):: cld
  REAL(WP):: clmin, clming
  REAL(WP):: clstart
  REAL(WP):: cosalf
  REAL(WP):: cosgam
  REAL(WP):: ctow
  REAL(WP):: cwos
  REAL(WP):: dcblimh
  REAL(WP):: ddist
  REAL(WP):: delalf
  REAL(WP):: delh
  REAL(WP):: delh26
  REAL(WP):: deltime
  REAL(WP):: delvel
  REAL(WP):: dfl
  REAL(WP):: dfu
  REAL(WP):: dh
  REAL(WP):: disk
  REAL(WP):: disl
  REAL(WP):: disland
  REAL(WP):: disslp
  REAL(WP):: distlof
  REAL(WP):: distobs
  REAL(WP):: distplt
  REAL(WP):: dobs
  REAL(WP):: dobx115
  REAL(WP):: dodr
  REAL(WP):: dot
  REAL(WP):: dsec
  REAL(WP):: dtass
  REAL(WP):: dtest
  REAL(WP):: dv
  REAL(WP):: dvel
  REAL(WP):: edelh
  REAL(WP):: failtim
  REAL(WP):: farfl
  REAL(WP):: fat
  REAL(WP):: fcl
  REAL(WP):: ffat
  REAL(WP):: ficase
  REAL(WP):: flypt
  REAL(WP):: fthin
  REAL(WP):: gamdot
  REAL(WP):: gamin
  REAL(WP):: gearcd
  REAL(WP):: glimslp
  REAL(WP):: grdist
  REAL(WP):: grtime
  REAL(WP):: gslope
  REAL(WP):: halt
  REAL(WP):: hcbdis
  REAL(WP):: hg
  REAL(WP):: hsvfail
  INTEGER:: ialpha
  INTEGER:: iatr
  INTEGER:: ib
  INTEGER:: icbct
  INTEGER:: icbdist
  INTEGER:: icut
  INTEGER:: idelh
  INTEGER:: ifail
  INTEGER:: ifat
  INTEGER:: ifixh
  INTEGER:: iflap
  INTEGER:: igamlim
  INTEGER:: igamma
  INTEGER:: igeaero
  INTEGER:: igrw
  INTEGER:: il
  INTEGER:: inegt
  INTEGER:: inocut
  INTEGER:: ipa,ipb
  INTEGER:: ipfail
  INTEGER:: iq1,iq2,iq3,iq4
  INTEGER:: ir
  INTEGER:: is
  INTEGER:: isibal
  INTEGER:: isipa
  INTEGER:: isiprnt
  INTEGER:: isirot
  INTEGER:: isispr2
  INTEGER:: isisrch
  INTEGER:: isjtcod
  INTEGER:: isk
  INTEGER:: itape
  INTEGER:: itvfail
  INTEGER:: iv
  INTEGER:: j
  INTEGER:: jr
  INTEGER:: jrjtcd
  INTEGER:: js
  INTEGER:: jsjtcd
  INTEGER:: kprint
  INTEGER:: lr
  INTEGER:: mm
  INTEGER:: n
  INTEGER:: nptmax
  REAL(WP):: powtest
  REAL(WP):: ratr
  REAL(WP):: rdtass
  REAL(WP):: reftod
  REAL(WP):: s35h
  REAL(WP):: salflof
  REAL(WP):: salph
  REAL(WP):: salt
  REAL(WP):: savdtas
  REAL(WP):: scbalt
  REAL(WP):: scbdist
  REAL(WP):: schtot
  REAL(WP):: sdist
  REAL(WP):: sdtfail
  REAL(WP):: sel
  REAL(WP):: sficase
  REAL(WP):: sfplof
  REAL(WP):: sfpobs
  REAL(WP):: sgamlim
  REAL(WP):: sinalf
  REAL(WP):: singam
  REAL(WP):: slope1,slope2,slope3
  REAL(WP):: slp35
  REAL(WP):: slpcb
  REAL(WP):: slstot
  REAL(WP):: smach
  REAL(WP):: snacc
  REAL(WP):: span
  REAL(WP),DIMENSION(25):: spower,stlvar
  REAL(WP):: sqrho
  REAL(WP):: srot
  REAL(WP),DIMENSION(5):: srvrot, srd115, srpowr, stvrot, std115
  REAL(WP),DIMENSION(10):: svrv

  REAL(WP):: stacc
  REAL(WP):: stime
  REAL(WP):: svel
  REAL(WP):: svk
  REAL(WP):: svkclmb
  REAL(WP):: svlof
  REAL(WP):: svobs
  REAL(WP):: svrot
  REAL(WP):: swlof
  REAL(WP):: swobs
  REAL(WP):: swt
  REAL(WP):: test
  REAL(WP):: tgotg
  REAL(WP):: theta
  REAL(WP):: timelof
  REAL(WP):: tlhobs
  REAL(WP):: top
  REAL(WP):: tpow
  REAL(WP):: tslope
  REAL(WP):: tt100
  REAL(WP):: ttnet
  REAL(WP):: unbal1,unbal2
  REAL(WP)::v1,v2
  REAL(WP):: vclimb
  REAL(WP):: vef
  REAL(WP):: vel
  REAL(WP):: vi
  REAL(WP):: vk1,vk2
  REAL(WP):: vkclimb
  REAL(WP):: vkeas
  REAL(WP):: vklof
  REAL(WP):: vkobs
  REAL(WP):: vland
  REAL(WP):: vlofest
  REAL(WP):: vobest
  REAL(WP):: vslope
  REAL(WP):: wac
  REAL(WP):: wdowd
  REAL(WP):: wfop
  REAL(WP):: wsing








!!!      REAL LOADF
      REAL MUSTEP
      INTEGER:: errCode
      CHARACTER(LEN=80):: mainTitle
      COMMON /TITLE_BLOCK1/ mainTitle
      CHARACTER(LEN=15):: today

  CHARACTER(LEN=80):: fileName

  INTEGER:: IUNITS !!!,IGRD1,IGRD2  ! in namelist, not in blank common

  NAMELIST/NAM1/ ALFROLL, ALFROT, ANOE, ARALF1, ARALF2, ARFLP1, ARFLP2,         &
   ARVEL1, ARVEL2, ASPARE, ATRDT, ATRTOT, BSPARE, CA, CBALT, CBDIST, CBGRAD,    &
   CBLIMH, CDGEAR, CDGRT, CDOUTT, CE, CLGRT, CLOFLP, DCDS, DCLS, DELCD, DELLOD, &
   DELTA, DESTOFL, DOUT1, DOUT2, DRFACT, DTBRA, DTEMP, DTFAIL, DTFAR, DTFTD,    &
   DTGRUP, DTPOW, DTREC, DTSPO, DTTR, DVCLIMB, DWTAXI, ENGCODE, ENGSOUT, ESF,   &
   FLAPT, FPLIMH, FPVAR, FUEL1, GAMLIM, GLIDE1, GLIDE2, GRAD1, GRAD2, GZERO,    &
   HENGT, HNOGE, HOBS, HOBSL, HSTEP1, HSTEP2, HTAB1, IBAL, IBUG, IGRD1, IGRD2,  &
   ILAND, INAM, IPRINT, IRCDGR, IROT, ISEARCH, ISOT, ISPARE, ISTP, ITER,        &
   IUNITS, JTCODE, LF, MENGT, MPENGT1, MUBRAKE, MUROLL, NCODE, NEI, NEJ, NFD,   &
   NPI, NPJ, NTOG, NTOP, PARRAM1, PARTH1, PARWF1, PBTAB, PINT1, PINT2, PINT3,   &
   POWER, RAM1, REFA1, REFB1, ROTRATE, RZERO, S, SFCFACT, STOPDIS, TALPHTO,     &
   TALPTOG, TCDTO, TCDTOG, TCLTO, TCLTOG, TDIDLE, TFACT, TFSET, THRUST1, TIDLE, &
   TLVAR, TOFLP, TOWSLS, TRDT, TREV, TSINAL, VFAILKT, VROTKT, VSTEP1, VSTEP2,   &
   VSTEP3, VSTEP5, WOSSLS, WTLAND, ZHEAD, ZRANGE, ZTIME, ZW



!      NAMELIST/NAM1/NEI,NEJ,MENGT,HENGT,THRUST1,FUEL1,RAM1,             &
!     &NPI,NPJ,MPENGT1,HTAB1,PARTH1,PARWF1,PARRAM1,IUNITS,               &
!     &NTOP,NFD,TCLTO,TCDTO,TALPHTO,NTOG,TCLTOG,TCDTOG,TALPTOG,          &
!     &TFSET,HNOGE,TOFLP,CLOFLP,DELCD,CDOUTT,CDGEAR,IRCDGR,CDGRT,CLGRT,  &
!     &DELLOD,FLAPT,FPVAR,                                               &
!     &REFA1,REFB1,S,ANOE,ESF,ZW,WOSSLS,TOWSLS,DELTA,POWER,ENGSOUT,      &
!     &ENGCODE,TFACT,SFCFACT,DRFACT,TLVAR,DWTAXI,DTEMP,PBTAB,            &
!     &RZERO,GZERO,TSINAL,ZRANGE,ZTIME,ZHEAD,LF,JTCODE,ISOT,ISTP,        &
!     &ITER,IPRINT,INAM,IBUG,NCODE,VSTEP1,MUROLL,ALFROLL,IROT,           &
!     &VROTKT,ALFROT,ROTRATE,VSTEP2,HOBS,VSTEP3,DTGRUP,VFAILKT,DVCLIMB,  &
!     &VSTEP5,HSTEP1,HSTEP2,GRAD1,IGRD1,GRAD2,IGRD2,STOPDIS,PINT1,PINT2, &
!     &PINT3,ISEARCH,DESTOFL,IBAL,DTFAIL,DTREC,DTFAR,DTPOW,DTBRA,        &
!     &DTSPO,DTTR,TREV,TRDT,DTFTD,ATRTOT,ATRDT,DCLS,DCDS,TIDLE,TDIDLE,   &
!     &MUBRAKE,CBALT,CBDIST,CBGRAD,CBLIMH,FPLIMH,GAMLIM,CE,CA            &
!     &,ILAND,WTLAND,HOBSL,GLIDE1,ARALF1,ARFLP1,ARVEL1,DOUT1,            &
!     &GLIDE2,ARALF2,ARFLP2,ARVEL2,DOUT2,ISPARE,ASPARE,BSPARE
      NAMELIST/NAM52/NEI,NEJ,MENGT,HENGT,THRUST1,TOPT1,WFOPT1,DROPT1,   &
     &NPI,NPJ,MPENGT1,HTAB1,TOTTAB1,WOWTAB1,RORTAB1
!!!      NAMELIST/NAMCE/CE,CA
!!!      NAMELIST/NAMPH/ET,EH,EX,EV,EG,EA,EW,EF,ECL,ELD,                   &
!!!     &EDG,ETN,ESP

  today=GetDateTimeStr()
  OPEN(UNIT=DBG,FILE='toly.dbg',STATUS='REPLACE',ACTION='WRITE')
  WRITE(DBG,*) 'Running PDAS version of WEF1041 (NASA Langley). '//today
  
  WRITE(*,*) 'WEF1041 - takeoff and landing'
  DO
    WRITE(*,*) 'Enter input file name'
    READ(*,'(A)') fileName
    IF (fileName==' ') STOP
    OPEN(UNIT=5,FILE=filename,STATUS='OLD',IOSTAT=errCode,ACTION='READ')
    IF (errCode==0) EXIT
    WRITE(*,*) 'Unable to open this file. Try again.'
  END DO

  OPEN(UNIT=6,FILE='tol.out',STATUS='REPLACE',ACTION='WRITE')

      READ(5,2) INENG,IFILE,ISIN,IMIL
!     ************************
    2 FORMAT(4I5)
    
      IF (INENG.NE.1.AND.INENG.NE.2) INENG=1
!!!      IF (IFILE.NE.1.AND.IFILE.NE.2.AND.IFILE.NE.3) IFILE=2
      IF (ISIN.NE.1.AND.ISIN.NE.2) ISIN=1
      IF (IMIL.NE.1.AND.IMIL.NE.2) IMIL=0
!     IMIL=1, FOR MILITARY AIRCRAFT. Bypasses climb gradient calculation.
      READ(5,'(A)') mainTitle
!     ************************


      FICASE=0.0
      ITAPE=1

      REWIND 8
      IF (IFILE.NE.3) GO TO 131
      INENG=1
      ISIN=1
  131 CONTINUE
!        SELECTED DEFAULT VALUES.
      PBTAB(1)=.101325E06
      PBTAB(2)=.22632029E05
      PBTAB(3)=.5474872147E04
      PBTAB(4)=.8680144315E03
      PBTAB(5)=.1109055258E03
      PBTAB(6)=.590004873E02
      PBTAB(7)=.1820991199E02
      PBTAB(8)=.103769949E01
      IGEAERO=0
      TCDTOG(1,1)=0.0
      TOT=1.0
      IALPHA=1
      IGRD1=0
      IGRD2=0
      TLHOBS=.01
      ASPARE(1)=1.0
      ASPARE(2)=0.0
      ASPARE(7)=0.0
      ASPARE(10)=0.0
      ASPARE(11)=0.0
      CSPARE(1)=0.0
      CSPARE(2)=0.0
      ALTLIM=10000.
      NPTMAX=200
      IRCDGR=0
      NCODE=0
      JTCODE=0
      ISOT=1
      ISTP=1
      IROT=0
      IBAL=0
      ISEARCH=0
      DELLOD=0.0
      POWTEST=0.0
      ILAND=0
      INAM=0
      IPRINT=1
      ITER=0
      CBLIMH=0.0
      DWTAXI=0.0
      GAMLIM=50.0
      DTFAR=2.0
      ATRTOT=0.0
      ATRDT=0.0
      S=0.0
      ESF=0.0
      WOSSLS=0.0
      TOWSLS=0.0
      DCBLIMH=700.
!      REFRHO=.00237689
!      CONV1=6076.
!      CONV2=3600.
!      CONV3=CONV1/CONV2
!      S1=4.44822
!      S2=.09290304
!      S3=.3048
!      S4=.51444
!      S5=1.0000
!      S6=.04788026
!      S7=.000126
!      S8=.4535924
!      S9=.00064516
!      S10=1.000
!        ZERO SUMMARY ARRAYS.
      IPA=0
      DO 52 JS=1,20
      CE(JS)=0.0
      CA(JS)=0.0
   52 END DO
      DO 10 IS=1,50
      C1(IS)=0.0
      C2(IS)=0.0
      C3(IS)=0.0
      C4(IS)=0.0
      C5(IS)=0.0
      C6(IS)=0.0
      C7(IS)=0.0
      C8(IS)=0.0
      C9(IS)=0.0
      C10(IS)=0.0
      C11(IS)=0.0
      C12(IS)=0.0
      C13(IS)=0.0
      C14(IS)=0.0
      C15(IS)=0.0
      C16(IS)=0.0
      C17(IS)=0.0
      C18(IS)=0.0
      C19(IS)=0.0
      C20(IS)=0.0
   10 END DO


      IF (ISIN.EQ.2) GO TO 151
!       DEFAULT NAMELIST ITEMS IN FPS UNITS.
      PINT1=500.
      PINT2=500.
      PINT3=500.
      FPLIMH=400.
      GO TO 152
  151 PINT1=152.4
!       DEFAULT NAMELIST ITEMS IN SI UNITS.
      PINT2=152.4
      PINT3=152.4
      FPLIMH=121.9
  152 CONTINUE


!        READ NAMELIST DATA FROM INPUT BASE FILE (TAPE 9).
      IF (IFILE.EQ.2.OR.IFILE.EQ.3) READ(9,NAM1)
      REWIND 9
      IF (ISIN.EQ.2) GO TO 1000
!     ENGINE DATA READ OPTION IS NOT AVAILABLE IN  SI  UNITS.
!!!      GO TO(1000,3)INENG
!      SUB ENCV41 READS ENG. DATA IN FIXED FORMAT (FROM CARDS OR FILE )
!      *****   STILL TO BE TESTED   *****
!!! Note by RLC - call to ENCV41 commented out. Maybe next year...
!      AND CONVERTS TO NAMELIST NAMES AND FORMAT.

!!!    3 CALL ENCV41(NEI,NEJ,MENGT,THRUST1,FUEL1,RAM1,                     &
!!!     &NPI,NPJ,MPENGT1,HTAB1,PARTH1,PARWF1,PARRAM1)



! Read and process one case. Keep returning here after completion of each case.

 1000 CONTINUE

  READ(5,NAM1,IOSTAT=errCode)
  IF (errCode < 0) GO TO 2000   ! no more input. Go to final summary.

  ICODE=1041
  IF (ibal .NE. 0) hstep1=5000.
      

  IF (IFILE==3) THEN   ! ifile=3 is option to make a si file from a fps file.
    CALL CONSI()
    WRITE(8,NAM1)
    STOP
  END IF
      
  IF (ASPARE(10).NE.0.0) TLHOBS=ABS(ASPARE(10))

! At this point all input data for this case has been read in. 
! Units must be all US Customary or all SI.
      IF (POWER(1).GE.POWTEST) GO TO 134
      WRITE(6,133) FICASE+1.0,POWER(1),POWTEST
  133 FORMAT(2X,"**** AT START OF NEW CASE =",F7.3,2X, &
    "INPUT POWER(1) (=", F8.5,1X, ") IS LESS THAN LAST CASE POWTEST (=", &
       F8.5,1X,")."/)
      POWTEST=0.0
      NCODE=0
      ISEARCH=0
      GO TO 810

  134 IF (ISEARCH.GE.10) GO TO 117
      IF (IGEAERO.EQ.1) GO TO 117
      IF (TCDTOG(1,1).NE.0.0) GO TO 117
!        IF GROUND EFFECT DATA NOT INPUT, IS CALC. HERE.
      SPAN=REFB1
      AR=SPAN**2./REFA1
      HG=ASPARE(7)
      IF (ASPARE(7).EQ.0.0) HG=.2*SPAN
      NTOG=NTOP
      IGEAERO=1
      
      DO 116 J=1,NFD   !        FIND I FOR MIN. TCDTO AT EACH TFSET.
      JS=J
      DO 114 L=2,NTOP
      IF (TCDTO(L,JS).LT.TCDTO(L-1,JS)) CYCLE   !!! GO TO 114
      IS=L-1
      GO TO 115
  114 END DO
      IS=NTOP
  115 CLMIN=TCLTO(IS,JS)
      ALFMIN=TALPHTO(IS,JS)
      CDMIN=TCDTO(IS,JS)
      CALL GREFCT(NTOP,JS,AR,SPAN,ALFMIN,CLMIN,CDMIN,HG,TALPHTO,TCLTO,  &
     & TCDTO,TALPTOG,TCLTOG,TCDTOG,CLMING)
  116 END DO
  
  
!        All ground effect aero. data have now been generated.
  117 CONTINUE
      IF (INAM.NE.0.AND.ISIN.EQ.2) WRITE(6,NAM1)   ! changed to debug by RLC
      IF (ISIN.EQ.2) CALL CONSI()  ! changes SI units to US
! NOTE - Beyond this point all input data are US Customary units.

      IALPHA=ISPARE(1)
      IF (FICASE.NE.0.0) GO TO 120
      WRITE(8,NAM1)
!        NOTE THAT A TOA00X FILE IS MADE HERE FOR FIRST CASE INPUTS.
!        THIS FILE CAN BE SAVED WITH SUITABLE CONTROL CARDS.
  120 CONTINUE
      JSJTCD=JTCODE
      FICASE=REAL(INT(ficase))
      ficase=ficase+1.0

      IF (S.NE.0.0) GO TO 130
      S=ZW(1)/100.
      IF (WOSSLS.NE.0.0) S=ZW(1)/WOSSLS
  130 CONTINUE

!     convert aero to polar format and engine data to non-dimen. format
!        after 11-76 polars are function of cl(15),flap(4),and g.e.(2).

  125 CALL CNVT41()   ! only call to Cnvt41 in whole program

! At this point, we have converted all aero. and engine data to standard 
! format for use in the program. Velocity inputs and outputs are in knots. 
! Time in seconds.

      IF (ITER.NE.52) GO TO 105
      WRITE(6,NAM52)
      WRITE(6,127)
  127 FORMAT(//6X,"***  ABOVE LISTING OF INTERNAL ENGINE PARAMETERS IN FPS UNITS.  ***"/)

  105 IF (CBLIMH.NE.0.0) GO TO 98
      CBLIMH=DCBLIMH
      IF (ANOE.LE.3.0) CBLIMH=1000.

   98 IR=0
      IF (ISEARCH.GE.10) GO TO 108
      ISISRCH=ISEARCH
      IF (ISEARCH.EQ.0) GO TO 210
      ISK=1
      TPOW=1.0
      ISIROT=IROT
      ISIBAL=IBAL
      ISIPA=IPA+1
      ISJTCOD=JTCODE
      ISIPRNT=IPRINT
      IPRINT=0
      ISISPR2=ISPARE(2)
      ISPARE(2)=1
      SFICASE=FICASE
      IBAL=0
      IROT=2
      NCODE=0
      JTCODE=1

      DO 208 IL=1,25
      SPOWER(IL)=POWER(IL)
      POWER(IL)=TPOW
      STLVAR(IL)=TLVAR(IL)
      TLVAR(IL)=0.0
  208 END DO

      TLVAR(2)=10000.
  210 SCBALT=CBALT
      SCBDIST=CBDIST
      SGAMLIM=GAMLIM
      ICBDIST=0

      DO 107 JR=1,10
      SVRV(JR)=VROTKT(JR)
  107 END DO

      IF (IROT.NE.2) GO TO 108
      SROT=VROTKT(1)
      SVROT=FLOAT(IFIX(SROT))
      VROTKT(1)=SROT
!     SVROT=SVROT-5.0
!     VROTKT(1)=SVROT

      DO 99 IV=2,5
      VI=FLOAT(IV-1)
      VROTKT(IV)=SVROT+(VI*5.)
      VROTKT(IV+1)=0.0
   99 END DO
  108 CONTINUE


!        COMPUTE INPUT SLS THRUST , (100 PERCENT ENGINE)
      CALL AT76SP(0.0,ANS,DTEMP,PBTAB)  ! changed from AT62SP  6Aug09
!      CALL BILUPV(15,15,HENGT,MENGT,TOPT1,WFOPT1,NEI,NEJ,0.0,0.0,       &
!     &TOP,WFOP,A3,A4,A5)

      CALL BILUPV(5,5,HENGT,MENGT,TOPT1,WFOPT1,NEI,NEJ,0.0,0.0,       &
     &TOP,WFOP,A3,A4,A5)
      LOC=1
      CALL FTLUP2(0.0,TFAC,+1,NEJ,MENGT,TFACT,LOC)
      IF (LOC.EQ.1000) GO TO 146
      TT100=TOP*ANS(2)*ANOE*TFAC

      IF (ESF.NE.0.0) GO TO 135
      ESF=1.0
!     ASPARE(1)=TOW 10 / TOW 8
      IF (TOWSLS.NE.0.0) ESF=ZW(1)*TOWSLS*ASPARE(1)/TT100
  135 IF (INAM.NE.0.AND.ISIN.EQ.1) WRITE(6,NAM1)

      WRITE(6,5) TRIM(mainTitle),today
    5 FORMAT('1',//2X,A/8X,'WEF1041',8X,A/)
      IF (ISEARCH.GE.10) GO TO 121
      IF (ISOT.EQ.1) WRITE(6,101) FICASE,   ZW(1),   S,ESF,   ZRANGE,ZTIME
      IF (ISOT.EQ.2) WRITE(6,101) FICASE,S1*ZW(1),S2*S,ESF,S3*ZRANGE,ZTIME
  101 FORMAT(2X, "**** CASE=",F5.2,5X,"ZW=",F9.1,2X,"S=",F9.2,2X,"ESF=",F8.5, &
     & 2X,"ZRANGE=",F7.2,2X,"ZTIME=",F7.2/)

  121 IF (ISOT.EQ.1) WRITE(6,123) TT100,DTEMP
      IF (ISOT.EQ.2) WRITE(6,123) S1*TT100,DTEMP
  123 FORMAT(52X,"TT100=",F10.1,5X,"DTEMP=",F7.1,31X,"F123"/)
  
      IF (ISEARCH.LT.10) GO TO 124
      CALL EXTRA()
      GO TO 1500
      
  124 CONTINUE
      IF (DELLOD.EQ.0.0) GO TO 106
      WRITE(6,104) DELLOD
  104 FORMAT(2X, "****  AERO. L/D MODIFIED FOR THIS CASE, WITH DELLOD=",  &
     F7.2,2X,"****",50X,"F104"//)

  106 IF (NCODE.EQ.0) GO TO 150
      DO 148 IB=1,60
      ET(IB)=0.0
      EH(IB)=0.0
      EX(IB)=0.0
      EV(IB)=0.0
      EG(IB)=0.0
      EA(IB)=0.0
      EW(IB)=0.0
      EF(IB)=0.0
      ECL(IB)=0.0
      ELD(IB)=0.0
      EDG(IB)=0.0
      ETN(IB)=0.0
      ESP(IB)=0.0
  148 END DO

  150 IF (ILAND.EQ.2) GO TO 700

 1005 IR=IR+1
      IF (ISPARE(6).NE.0.AND.IR.EQ.ISPARE(6)) IBUG=2
      IF (ISPARE(6).NE.0.AND.IR.NE.ISPARE(6)) IBUG=0
      IF (IR.GE.2.AND.IROT.EQ.0) GO TO 700
      IF (VROTKT(IR).EQ.0.0) GO TO 700
      IF (IR.EQ.10) GO TO 700

 1006 SVLOF=0.0
      SWLOF=0.0
      SVOBS=0.0
      SWOBS=0.0
      VLOF=0.0
      VOBS=0.0
      DOBX115=0.0
      ICBDIST=0
      CBALT=SCBALT
      CBDIST=SCBDIST
      IF (CBALT.LT.CBLIMH) CBALT=CBLIMH
!        *** NOTE MIXED MODES BELOW.
      IF (ISEARCH.EQ.0) JTCODE=JSJTCD
      FAT=FICASE+0.10001
      FFAT=FAT*10.0
      IFAT=INT(FFAT)
      FTHIN=REAL(IFAT)
      FICASE=FTHIN/10.0
      VROT=VROTKT(IR)*CONV3

      IF (IR.NE.1) WRITE(6,9)
    9 FORMAT('1')
      IF (ISOT.EQ.1) WRITE(6,4) FICASE,VROTKT(IR)
      IF (ISOT.EQ.2) WRITE(6,4) FICASE,S4*VROTKT(IR)
    4 FORMAT(//3X,"*** CASE=",F5.2,2X,"VROT=",F6.2,87X,"F4"//)
    
      IF (IPRINT.NE.0) WRITE(6,60)
   60 FORMAT(12X,"****  UNITS FOR FLIGHT POINT OUTPUT DATA  ****",//    &
     &5X, &
     "SYSTEM   LENGTH  FORCE  TIME  ANGLE  WT. PRESSURE  VELOCITY  FUEL FLOW",/  &
     & 7X,"SI       M       N     SEC   DEG   N     KPA       M/S       KG/H",/    &
     & 7X,"FPS      FT     LBF    SEC   DEG   LBF   PSF       KT        LBM/H",//)
     
      IPA=IPA+1
      IF (ISEARCH .NE. 0) FICASE=SFICASE
      IF (ISEARCH.NE.0) IPA=ISIPA
      IF (ISEARCH.EQ.2) IPRINT=ISIPRNT
      IF (ISEARCH.EQ.2) ISEARCH=0
      IF (IPA.LT.49) GO TO 102
      WRITE(6,8)
    8 FORMAT(///6X,"***** RUN TERMINATED, CASE LIMIT *****"//)
      GO TO 2000


  102 c1(ipa)=ficase
      c2(ipa)=zw(1)/1000.
      cwos=zw(1)/s
      c3(ipa)=cwos
      ctow=tt100*esf/zw(1)
      IF (aspare(1) .NE. 0.0) ctow=ctow/aspare(1)
      C4(IPA)=CTOW
      IF (ISISRCH.NE.0.OR.JSJTCD.NE.0) C4(IPA)=-1.0*CTOW
      c5(ipa)=vrotkt(ir)
      c6(ipa)=alfrot
      c17(ipa)=toflp
      ifail=0
      ipfail=0
      itvfail=0
      hsvfail=0.0
      IF (ibal.EQ.1) GO TO 103
      vfailkt=500.
  103 vfail=vfailkt*conv3

!     BEGIN GROUND ROLL SEGMENT,   ICODE=1

      icode =1
      lcode=1
      jcode=1
      cbpert=1.0
      tnotn=0.0
      l=1
      k=1
      n=nptmax
      vlofest=0.0
      balfl=0.0
      reftod=0.0
      svkclmb=0.0
      vk1st=0.0
      misgrad=0
      vzerc=0.0
      igamlim=0
      gamlim=sgamlim
      inocut=0
      igrdcal=0
      icbct=0
      iflap=0
      igrw=0
      iq1=0
      iq2=0
      iq3=0
      iq4=0
      iatr=0
      kprint=1
      distplt=0.0
      dtest=0.0
      engsout=0.0
      delalf=alfrot-alfroll
      savdtas=delalf/(rotrate*10.)
      vkt(1)=.004
      v(1)=vkt(1)*conv3
      h(1)=0.0
      flap=toflp
      mustep=muroll
      delh=0.0
      gamma(1)=0.0
      gamdot=0.0
      singam=0.0
      avghdot=0.0
      cosgam=1.0
!
!         QANT2 COMPUTES M(1),Q(1),LOW(1),TT(1)TDR(1),AND TFF(1)
!         QANT4 (AS OF 3-77) CAN BE USED TO THROTTLE ENGINE DATA.
!
      M=L
      TOT=1.0
      IF (JTCODE.EQ.0) GO TO 111
      VKT(1)=V(1)/CONV3
      IF (ISPARE(2).EQ.0) GO TO 111
      LOC=2
      IF (ISPARE(2).EQ.1) CALL FTLUP2(VKT(1),TOT,+1,25,TLVAR,POWER,LOC)
      loc=3
      IF (ISPARE(2).EQ.2) CALL FTLUP2(H(1)  ,TOT,+1,25,TLVAR,POWER,loc)
      IF (LOC.EQ.1000) GO TO 146

      WRITE(6,112) TOT
  112 FORMAT(2X,"****  MODIFIED THROTTLE SCHEDULE, INITIAL TOT=",F8.5,  &
     & 61X,"F112"/)

  111 CALL QANT4()
      IF (LOC.EQ.1000) GO TO 146
      IF (ISPARE(7).EQ.0.OR.IQ1.EQ.1) GO TO 162
      
      WRITE(6,161) TOT
  161 FORMAT(4X,"QANT4 AT 161",2X,"TOT=",F6.4)
      IQ1=1
      IQ2=0
      IQ3=0
      IQ4=0

  162 CONTINUE
      IF (ISPARE(3).NE.0) WRITE(6,126) FLAPT(1)
  126 FORMAT(2X,"****  MODIFIED FLAP SCHEDULE, INITIAL FLAP=",F6.2,     &
     & 66X,"F126"//)
      SLSTOT=TOT
      TGOTG=TOT
      W(L)=ZW(1)-DWTAXI
!
!        COMPUTE PARAMETERS FOR INITIAL TAKE-OFF POINT
!
      TIME(L)=ZTIME
      DIST(L)=ZRANGE*CONV1
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(1)*SQRHO
      AMACH=MACH(1)
      ALPHA(L)=ALFROLL
      ALPH=ALPHA(L)
      HALT=H(L)
!        ALPHA KNOWN, LOOK UP CL AND CD. (Set cd=1.0)
      CD=1.0
      cld=-999.0   ! will be overwritten, but Aero41 needs a value
      CALL AERO41(HALT,ALPH,CLD,CD) ! computes cld and cd

      C17(IPA)=FLAP
      IF (ISPARE(3).NE.0) C17(IPA)=-1.0*C17(IPA)
      CL(L)=CLD

      SINALF=0.
      COSALF=1.
      IF (TSINAL.EQ.0.0) GO TO 61
      SINALF=SIN((ALPHA(L)+DELTA)/57.2958)
      COSALF=COS((ALPHA(L)+DELTA)/57.2958)
   61 CONTINUE

      LOC=3
      CALL FTLUP2(  CL(L),GEARCD,+1,15,CLGRT,CDGRT,LOC)
      IF (LOC.EQ.1000) GO TO 146  ! don't think loc can ever be 1000
      CD=CD+GEARCD*REFA1/S
      CDD(L)=CD
      LOD(L)=CL(L)/CDD(L)
      LIFT(L)=CL(L)*Q(L)*S
      DRAG(L)=CDD(L)*Q(L)*S
      WAC=W(L)-LIFT(L)-TT(L)*SINALF
      IF (WAC.LT.0.0) WAC=0.0
      TMD(L)=TT(L)*COSALF-TDR(L)-DRAG(L)-MUSTEP*WAC-W(L)*SINGAM
      TACC(L)=TMD(L)/W(L)
      NACC(L)=(TT(L)*SINALF+LIFT(L)-W(L)*COSGAM)/W(L)

      IF (TMD(L).GT.0.) GO TO 7
      WRITE(6,6) L
    6 FORMAT(2X,"***NEGATIVE THRUST-MINUS-DRAG FOR POINT NO. ",I2,69X,"F6")

    7 MM=N-1
      IF (IPFAIL.EQ.1.AND.ISPARE(5).EQ.0) GO TO 19
      
      IF (IPRINT.NE.1) GO TO 31
      IF (ISOT.EQ.1)WRITE(6,30)L,VKT(1),MACH(1),H(1),W(1),DIST(1),TIME(1)&
     &,ICODE,LCODE,JCODE
      IF (ISOT.EQ.2) WRITE(6,30) L,S4*VKT(1),MACH(1),S3*H(1),            &
     &S1*W(1),S3*DIST(1),TIME,ICODE,LCODE,JCODE
   30 FORMAT(2X,I3,2X,"V TAS=",F6.2,5X,"M=",F6.3,4X,"H=",F7.2,5X,"W=",F8.0, &
     & 2X,"DIST=",F7.0,2X,"TIME=",F7.2,15X,"I,L,J=",3I2,2X,"F30"/)
      GO TO 19
      
   31 IF (IPRINT.LT.2) GO TO 19
      IF (ISOT.EQ.1) WRITE(6,49)L,VKT(1),MACH(1),H(1),W(1),DIST(1),      &
     &TIME(1),ICODE,LCODE,JCODE,VKEAS,Q(1),                             &
     &ALPHA(1),CL(1),CDD(1),DRAG(1),LOD(1),                             &
     &TT(1),TDR(1),TFF(1),TACC(1),NACC(1),GAMMA(1)
      IF (ISOT.EQ.2) WRITE(6,49)L,S4*VKT(1),MACH(1),S3*H(1),S1*W(1),     &
     &S3*DIST(1),TIME(1),ICODE,LCODE,JCODE,S4*VKEAS,                    &
     &S6*Q(1),S5*ALPHA(1),CL(1),CDD(1),                                 &
     &S1*DRAG(1),LOD(1),S1*TT(1),S1*TDR(1),S7*TFF(1),TACC(1),           &
     &NACC(1),S5*GAMMA(1)
   49 FORMAT(2X,I3,2X,"V TAS=",F6.2,5X,"M=",F6.3,4X,"H=",F7.2,5X,"W=",F8.0, &
     & 2X,"DIST=",F7.0,2X,"TIME=",F7.2,15X,"I,L,J=",3I2/7X,"V EAS=",F6.2,    &
     & 2X,"Q=",F7.2,1X,"ALPHA=",F5.2,3X,"CL=",F7.5,5X,"CD=",F7.5,            &
     & 3X,"DRAG=",F8.0,2X,"L/D=",F5.2/17X,"TT=",F8.0,1X,"TDR=",F8.0,1X,      &
     & "TFF=",F8.0,1X,"TACC=",F7.4,1X,"NACC=",F7.4,1X,"GAMMA=",F6.3,         &
     & 23X,"F49")
      IF (JSJTCD.EQ.0.AND.ISPARE(3).EQ.0) GO TO 320
      IF (JSJTCD.NE.0.AND.ISPARE(3).NE.0) GO TO 318
      IF (JSJTCD.NE.0) WRITE(6,314) TGOTG,TNOTN
  314 FORMAT(14X,"** TGOTG=",F7.4,5X,"TNOTN=",F7.4,69X,"F314")
      IF (ISPARE(3).NE.0) WRITE(6,316) FLAP
  316 FORMAT(14X,"**",37X,"FLAP=",F5.2,54X,"F316")
      GO TO 320

  318 WRITE(6,319) TGOTG,TNOTN,FLAP
  319 FORMAT(14X,"** TGOTG=",F7.4,5X,"TNOTN=",F7.4,5X,"FLAP=",F5.2,        &
     &54X,"F319")
  320 WRITE(6,322)
  322 FORMAT(/)
  
  
  
   19 DELVEL=VSTEP1*CONV3   !        INITIAL POINT COMPLETED
!       THIS LOCATION USED FOR SAVING TIME HISTORY DATA
!       SEE COMMENT IN MAIN BELOW STATEMENT NO. 418
      GO TO 21

!        RE-INITIALIZE PT. COUNT IF POINTS EXCEED 99
   25 WRITE(6,69)
   69 FORMAT(2X,"***** POINTS EXCEED 100. *****"/)
      VKT(1)=VKT(K)
      MACH(1)=MACH(K)
      H(1)=H(K)
      W(1)=W(K)
      Q(1)=Q(K)
      ALPHA(1)=ALPHA(K)
      DIST(1)=DIST(K)
      TIME(1)=TIME(K)
      TFF(1)=TFF(K)
      TACC(1)=TACC(K)
      NACC(1)=NACC(K)
      GAMMA(1)=GAMMA(K)
!
!  Begin the main logic loop in program.
!  This loop has about 1040 statements.  Way down....
!
21 CONTINUE
  DO 20 L=1,MM
      INEGT=0
      IDELH=0
      ICUT=0
      IF (IGAMLIM.GT.0) IGAMLIM=1
      ACL=0.0
      ADH=0.0
      GSLOPE=0.0
      TNOTN=0.0
      IF (L.EQ.MM) GO TO 25
      K=L+1
!        IN FIRST PASS THRU EQUATIONS, USE PREVIOUS GAMMA
      GAMMA(K)=GAMMA(L)
      IF (ISPARE(3).GT.0) GO TO 44
      IF (H(L).LT.FPLIMH) GO TO 44
      IF (IFLAP.EQ.1.OR.LCODE.LT.3) GO TO 44
      IFLAP=1
      FLAP=CLOFLP
      IF (ISOT.EQ.1) WRITE(6,43) H(L),FLAP
      IF (ISOT.EQ.2) WRITE(6,43) S3*H(L),S5*FLAP
   43 FORMAT(3X,"*** ABOVE H=",F7.2,1X,",FLAP ANGLE=",F4.1,77X,"F43"/)
   44 V(K)=V(L)+DELVEL
      IF (ICODE.EQ.4.AND.LCODE.EQ.4) GO TO 26
      GO TO (660,630,665),JCODE
  630 IF (ICODE.EQ.1) GO TO 650
!        JCODE=2, ICODE=2 OR 3, TAKE NORMAL STEPS UNTIL REACH DTFAIL.
      IF ((FAILTIM+DTFAIL-TIME(L)-DTASS).GT.0.001) GO TO 26
      DTASS=(FAILTIM+DTFAIL)-TIME(L)
      DELVEL=DTASS*TACC(L)*GZERO
      V(K)=V(L)+DELVEL
      GO TO 26
  650 DTASS=DTFAIL
!     JCODE=2, ICODE=1,  FORCE DTASS, UNLESS MEET VROT
      DELVEL=DTASS*TACC(L)*GZERO
  651 V(K)=V(L)+DELVEL
      IF (V(K).LE.VROT) GO TO 26
      V(K)=VROT
      DELVEL=V(K)-V(L)
      DTASS=DELVEL/(TACC(L)*GZERO)
      GO TO 26

  660 IF (V(K).LE.VFAIL) GO TO 665
      V(K)=VFAIL
  665 IF (V(K).LE.VROT) GO TO 26
      IF (ICODE.EQ.1) V(K)=VROT

   26 H(K)=H(L)+DELH
      IDELH=IDELH+1
      IF (IDELH.GT.20) GO TO 143

      DELH26=DELH
      IF (IBUG.GT.0.AND.LCODE.GT.1)WRITE(6,202) L,DELH
  202 FORMAT(4X,I3,2X,"AT STA. 26, ASSUMED DELH=", F12.5)
      IF (ICODE.EQ.4) GO TO 29
      IF (LCODE.LT.3.OR.H(L).NE.HOBS) GO TO 27
      IF (IBUG.GT.0) WRITE(6,128) H(L),H(K)
  128 FORMAT(3X,"*** MODIFY ALTITUDE STEPS FROM H(L)=",F7.2,            &
     & " TO H(K)=",F7.2,55X,"F128")
   29 H(K)=H(K)+.01
      IFIXH=INT(H(K)/DELH)
      H(K)=DELH*REAL(IFIXH)
      EDELH=H(K)-H(L)
      IF (IBUG.GT.0.AND.EDELH.NE.DELH26) WRITE(6,203) EDELH
  203 FORMAT(9X,"AT STA. 29, ASSUMED DELH=",F10.3)
   27 W(K)=W(L)
      IF (INOCUT.EQ.1) GO TO 504
      IF (JCODE.EQ.7) GO TO 72
      IF (ICBDIST.EQ.5) GOTO 504
      IF (ICBDIST.EQ.1) GOTO 500
!         TESTS FOR C/B ALTITUDE.
      IF (CBALT.LT.100.) GO TO 504
      IF (H(K).LE.CBALT) GOTO 504
      IF (H(L).EQ.CBALT) GOTO 500
      H(K)=CBALT
      DELH=CBALT-H(L)
      GOTO 504

  500 JCODE=7
      IF (ICBDIST.NE.5) GOTO 505
      WRITE(6,520) H(K)
  520 FORMAT(2X, &
    "**** ERROR, CALLING SUB. CUTBACK FOR 2ND TIME AT H(K)=",F8.2,10X,"F520"/)
      GOTO 89

  505 M=K
      H(K)=H(L)   !        AT LAST ALTITUDE COMPUTE CBPERT FOR CUTBACK GRADIENT.
      JTCODE=0
      TOT=1.0
      CALL QANT4()
      IF (LOC.EQ.1000) GO TO 146
      IF (ISPARE(7).EQ.0.OR.IQ2.EQ.1) GO TO 164
      WRITE(6,163) TOT
  163 FORMAT(4X,"QANT4 AT 163",2X,"TOT=",F6.4)
      IQ2=1
      IQ1=0
      IQ3=0
      IQ4=0
  164 CONTINUE
      AMACH=MACH(K)
      CALL CUTBACK()
      WRITE(6,503) CBPERT,TNOTN
  503 FORMAT(3X,"**** RETURN FROM CUTBACK WITH CBPERT=",F8.5,           &
     & 4X,"TNOTN=",F8.5,51X,"F503"/)
     
      IF (IPA.GT.49) GO TO 506
      C13(IPA)=H(L)
      C14(IPA)=DIST(L)
      C15(IPA)=CBPERT
      C19(IPA)=V(L)/CONV3
  506 ICBDIST=5
  
      IF (CBPERT.NE.1.0) GO TO 501
      INOCUT=1
      JCODE=1
      GO TO 504
      
  501 TACC(K)=0.0
      TACC(L)=0.0
      NACC(L)=0.0
      GAMMA(L)=CBGAM
      H(K)=H(L)+DELH
      IF (ICODE.EQ.4.AND.LCODE.EQ.4) GO TO 502
      ICODE=4
      LCODE=4
      VKT(K)=V(K)/CONV3
      SVKCLMB=VKT(K)
      WRITE(6,47) H(K),VKT(K)
   47 FORMAT(3X,"*** ABOVE H=",F6.0,2X," CLIMB AT VKT=",F6.1,74X,"F47"/)
  502 CLSTART=CBCL
      GO TO 72


  504 GO TO (22,23,24,72),ICODE
   22 ALPHA(K)=ALFROLL
      GO TO 28
   23 ALPHA(K)=ALPHA(L)+DTASS*ROTRATE
      IF (LCODE.LT.2.OR.ISPARE(4).EQ.0) GO TO 35
      IF (ALPHA(K).LT.(ALFLOF-0.005))GO TO 28
      ALPHA(K)=ALFLOF
      GO TO 45
   35 IF (ALPHA(K).LT.(ALFROT-0.005)) GO TO 28
      ALPHA(K)=ALFROT
   45 DTASS=(ALPHA(K)-ALPHA(L))/ROTRATE
      IF (LCODE.EQ.2) GO TO 28
      DVEL=DTASS*AVGTACC*GZERO
      V(K)=V(L)+DVEL
      GO TO 28
   24 ALPHA(K)=ALFROT
      IF (ISPARE(4).NE.0) ALPHA(K)=ALFLOF
      IF (LCODE.LT.3) GO TO 28
      THETA=ALPHA(L)+GAMMA(L)
   70 IF (IALPHA.EQ.0) GO TO 28
      IF (IALPHA.EQ.1) ALPHA(K)=THETA-GAMMA(K)
!     IF (IALPHA.EQ.2) ALPHA(K)=  *******
   28 ALPH=ALPHA(K)
      IF (V(K).GE.V(L)) GO TO 482
      WRITE(6,481) K,V(K),V(L)
  481 FORMAT(10X,"AT K=",I3,2X,"V(K)=",F8.3,2X,"IS LESS THAN V(L)=",F8.3)
      GO TO 913
  482 IF (H(K).GE.H(L)) GO TO 484
      WRITE(6,483) K,H(K),H(L)
  483 FORMAT(10X,"AT K=",I3,2X,"H(K)=",F9.2,2X,"IS LESS THAN H(L)=",F9.2)
      GO TO 913
  484 CONTINUE
      IF (JCODE.NE.2) GO TO 670
      DELTIME=(TIME(L)+DTASS)-FAILTIM
      ENGSOUT=DELTIME/DTFAIL
      IF (ENGSOUT.GT.1.0) ENGSOUT=1.0
  670 M=K
      TOT=1.0
      IF (JTCODE.EQ.0) GO TO 222
      VKT(K)=V(K)/CONV3
      IF (ISPARE(2).EQ.0) GO TO 222
      LOC=4
      IF (ISPARE(2).EQ.1) CALL FTLUP2(VKT(K),TOT,+1,25,TLVAR,POWER,LOC)
      loc=6
      IF (ISPARE(2).EQ.2) CALL FTLUP2(H(K)  ,TOT,+1,25,TLVAR,POWER,loc)
      IF (LOC.EQ.1000) GO TO 146
  222 IF (IBAL.EQ.0.OR.JCODE.LT.2) GO TO 237
      IF (ATRTOT.LE.0.0) GO TO 237
      SCHTOT=TOT
!      SCHTOT IS SCHEDULED TOT FOR THIS POINT.
      ATIMEK=TIME(L)+DTASS
      IF (ATIMEK.LT.ATRTIME) GO TO 237
      IF (ATRDT.LE.0.0) GO TO 233
      RATR=(ATIMEK-ATRTIME)/ATRDT
      IF (ATRTOT.GT.SCHTOT) IATR=1
      IF (RATR.GE.1.0) RATR=1.0
      GO TO 234
  233 RATR=1.0
  234 TOT=SCHTOT+(ATRTOT-SCHTOT)*RATR
      IF (ATIMEK.GT.(ATRTIME+ATRDT+.05)) GO TO 236
      IF (ISPARE(8).NE.0) WRITE(6,235) SCHTOT,ATIMEK,RATR,TOT
  235 FORMAT(10X,"SCHTOT=",F6.4,2X,"ATIMEK=",F6.2,2X,                     &
     & "RATR=",F6.4,2X,"TOT=",F6.4,44X,"F235")
  236 IF (TOT.LT.SCHTOT) TOT=SCHTOT
!      ONE CALL TO QANT4 ASSUMES FAILING ENG. IS OPERATING AT ATR ALSO.
  237 CALL QANT4()
      IF (LOC.EQ.1000) GO TO 146
      IF (ISPARE(7).EQ.0.OR.IQ3.EQ.1) GO TO 166

      WRITE(6,165) TOT
  165 FORMAT(4X,"QANT4 AT 165",2X,"TOT=",F6.4)
      IQ3=1
      IQ1=0
      IQ2=0
      IQ4=0
  166 CONTINUE
      AMACH=MACH(K)
      HALT=H(K)
!        ALPHA KNOWN, LOOK UP CL AND CD. (Set cd=1.0)
      CD=1.0
      cld=-999.0   ! will be overwritten, but Aero41 needs a value
      CALL AERO41(HALT,ALPH,CLD,CD)
      CL(K)=CLD
      GO TO 76
   72 FCL=CLSTART
   73 CL(K)=FCL
      M=K
      IF (CBPERT.EQ.1.0) GO TO 109
!        SET UP 100 PERCENT REF. FOR C/B THRUST.
      JTCODE=0
      TOT=1.0
      GO TO 113
  109 TOT=1.0
      IF (JTCODE.EQ.0) GO TO 113
      VKT(K)=V(K)/CONV3
      IF (ISPARE(2).EQ.0) GO TO 113
      LOC=5
      IF (ISPARE(2).EQ.1) CALL FTLUP2(VKT(K),TOT,+1,25,TLVAR,POWER,LOC)
      loc=8
      IF (ISPARE(2).EQ.2) CALL FTLUP2(H(K)  ,TOT,+1,25,TLVAR,POWER,loc)
      IF (LOC.EQ.1000) GO TO 146
  113 CALL QANT4
      IF (LOC.EQ.1000) GO TO 146
      IF (ISPARE(7).EQ.0.OR.IQ4.EQ.1) GO TO 168
      WRITE(6,167) TOT
  167 FORMAT(4X,"QANT4 AT 167",2X,"TOT=",F6.4)
      IQ4=1
      IQ1=0
      IQ2=0
      IQ3=0
  168 CONTINUE
      AMACH=MACH(K)
      IF (JCODE.NE.7) GO TO 510
      CBM=MACH(K)
      GAMMA(K)=CBGAM
      IF (ICUT.EQ.1) GO TO 510
      TT(K)=TT(K)*CBPERT
      ICUT=1
      DOT=CBPERT
      CALL BILUPV(10,5,TOTTAB1,MPENGT1,RORTAB1,WOWTAB1,NPI,NPJ,DOT,CBM, &
     &DODR,WDOWD,A3,A4,AK)
      TDR(K)=TDR(K)*DODR
      TFF(K)=TFF(K)*WDOWD
  510 CLD=CL(K)
      HALT=H(K)
!        CL KNOWN, LOOK UP ALPHA AND CD. (Set cd=2.0)
      CD=2.0
      alph=-999.0   ! will be overwritten, but Aero41 needs a value
      CALL AERO41(HALT,ALPH,CLD,CD)   ! computes alph and cd
      ALPHA(K)=ALPH


   76 SINALF=0.0
      IF (LCODE.LT.3.OR.IPRINT.LT.3.OR.IBUG.EQ.0) GO TO 77
      IF (IBUG.GE.2) WRITE(6,450) CL(K),CD,TT(K)
  450 FORMAT(15X,"M450",2X,"CL=",E15.8," CD=",E15.8," TT=",E15.8)
   77 COSALF=1.

      IF (TSINAL.EQ.0.0) GO TO 14
      SINALF=SIN((ALPHA(K)+DELTA)/57.2958)
      COSALF=COS((ALPHA(K)+DELTA)/57.2958)
   14 CONTINUE

      GEARCD=0.0
      IF (IGRW.EQ.1) GO TO 16

! note by RLC - assumes clgrt, cdgrt fill out the length 15 arrays.
! probably OK because always search from botton and never extrapolate.
! Could well be a bug waiting to appear
      LOC=6
      CALL FTLUP2(CL(K), GEARCD, +1, 15, CLGRT, CDGRT, LOC)
      IF (LOC.EQ.1000) GO TO 146
      
      IF (LCODE.EQ.1) GO TO 16
      ATGRUP=TIMELOF+DTGRUP
!       NOTE THAT GEAR TIME INTERVAL IS FOR PREVIOUS
!       STEP, SINCE TIME(K) IS NOT YET CALCULATED.
      IF (TIME(L).GE.ATGRUP) GO TO 231
      IF (IRCDGR.EQ.0) GEARCD=GEARCD*(1.-(TIME(L)-TIMELOF)/DTGRUP)
      IF (IRCDGR.EQ.1)GEARCD=GEARCD
      GO TO 16

  231 GEARCD=0.0
      WRITE(6,232) ATGRUP
  232 FORMAT(2X, "*** GEAR FULLY RETRACTED AT TIME=", F6.2, 25X, "F232"/)
      IGRW=1

   16 CD=CD+GEARCD*REFA1/S
      CDENGF=0.0
      IF (JCODE.LT.2) GO TO 145
      IF (CDOUTT(3).EQ.0.0) GO TO 142
!        SIMILAR LOGIC FOR REFUSED T.O. NEAR S1041.2320
!         FOR T.O. ASSUME WINDMILLING EFFECT IS NEGLIGIBLE
      CDENGF=CDOUTT(3)*CDOUTT(5)*ESF/S
      GO TO 145

  142 IF (CDOUTT(2).EQ.0.0) GO TO 145
      CDENGF=CDOUTT(1)*CDOUTT(2)*ESF/S


  145 CD=CD+CDENGF*ENGSOUT
      CDD(K)=CD
      LOD(K)=CL(K)/CDD(K)
      LIFT(K)=CL(K)*Q(K)*S
      DRAG(K)=CDD(K)*Q(K)*S
      WAC=0.0

      IF (JCODE.EQ.7) GO TO 15
      IF (H(K).EQ.0.0) WAC=W(K)-LIFT(K)-TT(K)*SINALF
      IF (WAC.LT.0.0) WAC=0.0
      IF (LCODE.NE.4) GO TO 15
      SINGAM=(TT(K)*COSALF-TDR(K)-DRAG(K))/W(K)
      GAMMA(K)=57.2958*ASIN(SINGAM)
      COSGAM=COS(GAMMA(K)/57.2958)
      TMD(K)=0.0
      TACC(K)=0.0
      GO TO 36


   15 SINGAM=SIN(GAMMA(K)/57.2958)
      COSGAM=COS(GAMMA(K)/57.2958)
      IF (JCODE.NE.7) GO TO 42
      ANACC=(TT(K)*SINALF+LIFT(K)-W(K)*COSGAM)/W(K)
      IF (ABS(ANACC).LT.0.0005) GO TO 42
      CL(K)=(W(K)*COSGAM-TT(K)*SINALF)/(Q(K)*S)
!      FORCE CL FOR NACC=0.0
      GO TO 510

   42 NACC(K)=0.0
      TMD(K)=TT(K)*COSALF-TDR(K)-DRAG(K)-MUSTEP*WAC-W(K)*SINGAM
      TACC(K)=TMD(K)/W(K)
      IF (JCODE.NE.7) GO TO 36

      IF (IBUG.GE.2) WRITE(6,452) GAMMA(K),TACC(K),NACC(K)
  452 FORMAT(15X,"M452",2X,"GAMMA(K)=",E15.8,2X,"TACC(K)=",E15.8,           &
     & 2X,"NACC(K)=",E15.8)
      IF (ABS(TACC(K)).GT.0.0005) GO TO 36
      TMD(K)=0.0
      TACC(K)=0.0

   36 IF (IGAMLIM.EQ.0)NACC(K)=(TT(K)*SINALF+LIFT(K)-W(K)*COSGAM)/W(K)
      IF (IGAMLIM.NE.0) NACC(K)=0.0
      AVFF=(TFF(L)+TFF(K))/2.
      AVGTACC=(TACC(L)+TACC(K))/2.
      IF (IGAMLIM.EQ.0) AVGNACC=(NACC(L)+NACC(K))/2.
      IF (IGAMLIM.NE.0) AVGNACC=0.0
      AVVEL=(V(K)+V(L))/2.
      AVH=(H(K)+H(L))/2.
      DELVEL=V(K)-V(L)

      IF (AVGTACC.NE.0.0) GO TO 37
      DSEC=0.0
      GO TO 38
   37 DSEC=DELVEL/(AVGTACC*GZERO)

   38 VKT(K)=V(K)/CONV3
      IF (IBUG.LT.2) GO TO 18
      IF (JCODE.EQ.7) GO TO 18
      WRITE(6,454)ALPHA(K),DVEL,DTASS,ENGSOUT,VLOFEST,AVGHDOT
      WRITE(6,455)AVGNACC,GAMDOT,DSEC,VKT(K),GAMMA(K),NACC(K)
      WRITE(6,456) AVGTACC,H(K),DELVEL,V(K),DELH,TACC(K)
  454 FORMAT(15X,"F454",6E15.8)
  455 FORMAT(16X,"F455",6E15.8)
  456 FORMAT(17X,"F456",6E15.8)


   18 IF (TMD(K).GE.0.) GO TO 600
      INEGT=INEGT+1
      IF (INEGT.LT.5) GO TO 600
   41 WRITE(6,11) K
   11 FORMAT(2X,"***NEGATIVE THRUST-MINUS-DRAG FOR POINT NO. ",I2,68X,"F11")
      TTNET=TT(K)*COSALF-TDR(K)
      WSING=W(K)*SINGAM
      IF (IBUG.GE.2.OR.INEGT.GT.2) WRITE(6,13) INEGT,VKT(K),H(K),        &
     &ALPHA(K),CL(K),CDD(K),DRAG(K),TTNET,TT(K),TDR(K),WSING
   13 FORMAT(10X,"INEGT=",I2,2X,"VKT=",F6.2,2X,"H=",F7.2,2X,"ALPHA=",F5.2,  &
     & 2X,"CL=",F7.5,2X,"CD=",F7.5,2X,"DRAG=",F8.0,2X,"TTNET=",F8.0/20X,     &
     & "TT=",F8.0,2X,"TDR=",F8.0,2X,"WSING=",F8.0,55X,"F13")
     
      IF (INEGT.GT.5) GO TO 82
!      TRIAL MODS. FOR LCODE=2, AND NEGATIVE TACC(K).
      IF (LCODE.NE.2) GO TO 600
      DELVEL=TACC(L)*DELVEL/(TACC(L)-TACC(K))
      IF (DELVEL.LE.0.0) GO TO 82
      V(K)=V(L)+DELVEL
      DELH=DELH/10.0
      GO TO 26


  600 IF (JCODE.EQ.1) GO TO 655
      IF (JCODE.EQ.7) GO TO 90
      IF (JCODE.EQ.3) GO TO 696
      IF (ICODE.EQ.1) GO TO 690
!     JCODE=2, ICODE=2, FORCE DTASS TO MEET (FAILTIM+DTFAIL)
      DVEL=DTASS*AVGTACC*GZERO
      VEL=V(L)+DVEL
      IF (ABS(VEL-V(K)).LT.0.01) GO TO 685
      V(K)=VEL
      GO TO 26

  685 IF (ABS(FAILTIM+DTFAIL-TIME(L)-DTASS).LT.0.001) GO TO 694
      GO TO 655
  690 IF (V(K).EQ.VROT) GO TO 692
      IF (ABS(DSEC-DTFAIL).LT.0.001) GO TO 694
!        JCODE=2, ICODE=1,    FORCE DELVEL TO MATCH DTFAIL
      DELVEL=DTFAIL*AVGTACC*GZERO
      GO TO 651
  692 IF (ABS(DSEC-DTASS).LT.0.001) GO TO 695
!     JCODE=2, ICODE=1, FORCE DTASS=DSEC TO MEET VROT
      DTASS=DSEC
      GO TO 26

  694 JCODE=3
      ENGSOUT=1.0
      GO TO 655

  695 IF ((TIME(L)+DSEC).LE.(FAILTIM+DTFAIL)) GO TO 655
      DTASS=(FAILTIM+DTFAIL)-TIME(L)
      DELVEL=DTASS*AVGTACC*GZERO
      GO TO 651

  696 IF (ICODE.NE.2) GO TO 655
!     JCODE=3, ICODE=2, FORCE DTASS TO MATCH ALPHA
      DVEL=DTASS*AVGTACC*GZERO
      VEL=V(L)+DVEL
      IF (ABS(VEL-V(K)).LT.0.01) GO TO 655
      V(K)=VEL
      GO TO 26

  655 IF (LCODE.GE.2) GO TO 12
!      DETERMINE VLOF (NACC(K)=0.0)
      IF (NACC(K).LT.-0.001) GO TO 12

      IF (IPRINT.LT.3) GO TO 32
      VKT(K)=V(K)/CONV3
      WRITE(6,34) VKT(K),ALPHA(K),NACC(K)
   34 FORMAT(3X,"*** AT VKT=",F6.1,2X,"ALPHA=",F5.2,2X,"NACC=",F8.5/)

   32 IF (NACC(K).LE.0.001) GO TO 33
      SLOPE1=(V(K)-V(L))/(NACC(K)-NACC(L))
      VLOFEST=V(L)-NACC(L)*SLOPE1
      IF (ABS(VLOFEST-V(K)).LT.0.1) GO TO 33
      V(K)=VLOFEST
      DVEL=V(K)-V(L)
      DTASS=DVEL/(AVGTACC*GZERO)
      IF (ICODE.NE.2) GO TO 26
!         MODIFY ALPHA TO MATCH DVEL
      ALPHA(K)=ALPHA(L)+DTASS*ROTRATE
      GO TO 28

   33 LCODE=2
      IF (ISPARE(4).NE.0.AND.ICODE.EQ.2) ICODE=3
      NACC(K)=0.0
   12 IF (NACC(K).EQ.0.0.AND.LCODE.EQ.2.AND.H(K).EQ.0.0) GO TO 17
      IF (LCODE.EQ.3.AND.ICODE.EQ.3) GO TO 90
      IF (LCODE.EQ.4) GO TO 96
      IF (LCODE.EQ.2) GO TO 51
      IF (JCODE.GE.2) GO TO 17
      IF (ICODE.NE.2) GO TO 17
!        DURING ROTATION ITERATE FOR VELOCITY STEP FOR EACH ALPHA
      DVEL=DTASS*AVGTACC*GZERO
      VEL=V(K-1)+DVEL
      IF (ABS(VEL-V(K)).LT.0.01) GO TO 17
      V(K)=VEL
      GO TO 26

   17 DV=V(K)-V(L)
      DH=H(K)-H(L)
      DSEC=DV/(AVGTACC*GZERO)
      GO TO 55

   51 IF (IGAMLIM.EQ.0) GAMDOT=57.2958*AVGNACC*GZERO/AVVEL
      IF (IGAMLIM.NE.0) GAMDOT=0.0
!      LCODE=2, LIFTOFF TO HOBS, ITERATE FOR DSEC,DELVEL,AND DELH
!      GAMMA AND GAMDOT ARE IN DEGREES
      DSEC=DELVEL/(AVGTACC*GZERO)

      IF (ICODE.NE.2) GO TO 57
      DSEC=DTASS   !      force dtass to meet alpha
      DELVEL=DSEC*AVGTACC*GZERO
      V(K)=V(L)+DELVEL

   57 IF (IGAMLIM.EQ.0.OR.GAMLIM.GE.49.)GAMMA(K)=GAMMA(L)+DSEC*GAMDOT
      IF (IGAMLIM.GE.1.AND.GAMLIM.LT.49.) GAMMA(K)=GAMLIM
      IF (GAMMA(K).LE.GAMLIM) GO TO 243
      IF (H(K).GT.((1.0+TLHOBS)*HOBS)) GO TO 243
!!!      IF (GAMLIM .LT. 49.) IGAMLIM=IGAMLIM=1
      IF (GAMLIM .LT. 49.) IGAMLIM=1
      IF (IGAMLIM.LT.5) GO TO 242
      WRITE(6,241) K,H(K)
  241 FORMAT(6X,"*** AT PT.",I3,2X,"H(K)=",F10.2,2X,                    &
     & "ITER TO LIMIT GAMMA(K)",20X,"F241")
      GO TO 81


  242 GLIMSLP=(GAMLIM-GAMMA(L))/(GAMMA(K)-GAMMA(L))
      DELVEL=(V(K)-V(L))*GLIMSLP
      DELH=(H(K)-H(L))*GLIMSLP
      GAMIN=GAMMA(K)
      V(K)=V(L)+DELVEL
      GAMMA(K)=GAMLIM
      IF (GAMLIM.LT.49) WRITE(6,664) GAMLIM
  664 FORMAT(3X,"** GAMMA RESTRICTED TO ",F7.3,                        &
     & " TO IMPROVE TACC.",67X,"F664"/)
      IF (IPRINT.GE.3)WRITE(6,666) K,GAMIN,GLIMSLP,DELVEL,DELH
  666 FORMAT(6X,"** K=",I3,2X,"GAMIN=",F8.2,2X,"GLIMSLP=",F8.5,            &
     & 2X,"DELVEL=",F8.3,2X,"DELH=",F8.3)
      GO TO 26
      
  243 AVGGAM=(GAMMA(K)+GAMMA(L))/2.
      ASINGAM=SIN(AVGGAM/57.2958)
      AVGHDOT=AVVEL*ASINGAM
      ADELH=DSEC*AVGHDOT
      
      IF (IBUG.GT.0.AND.LCODE.EQ.2) WRITE(6,204) ADELH
  204 FORMAT(9X,"AT STA. 57+4L. CALCULATED DELH=",F12.5)
      IF (ABS(1.-(DELH/ADELH)).LT.(TLHOBS/10.)) GO TO 53
      DELH=ADELH

      IF (H(K).GE.((1.0+TLHOBS)*HOBS)) GO TO 64
!        EXPERIMENTAL MOD.
      H(K)=H(L)+ADELH
      IF ((H(L)+ADELH).GE.((1.0+TLHOBS)*HOBS)) GO TO 64
      GO TO 26

   53 IF (H(K).LE.((1.0-TLHOBS)*HOBS)) GO TO 55
      IF (H(K).LT.((1.0+TLHOBS)*HOBS)) GO TO 54

   64 SLOPE2=(V(K)-V(L))/(H(K)-H(L))
      VOBEST=V(L)+(HOBS-H(L))*SLOPE2
      V(K)=VOBEST
      DELVEL=V(K)-V(L)
      DELH=HOBS-H(L)
      IF (IBUG.GT.0.AND.LCODE.EQ.2) WRITE(6,209) DELH
  209 FORMAT(9X,"AT STA. 64+6L. DELH TO MEET HOBS=",F10.3, "      F209")
      GO TO 26

   54 LCODE=3
      GO TO 78

!        FOR CLIMBOUT FORCE DELH, ITERATE FOR DELVEL AND GAMMA
   90 GAMDOT=57.2958*AVGNACC*GZERO/AVVEL
      IF (TMD(K).GE.0.0.OR.JCODE.EQ.7) GO TO 224
      IF (AVGNACC.LT.0.0.AND.AVGTACC.LT.0.0) GO TO 81
  224 CONTINUE
      AVGGAM=(GAMMA(K)+GAMMA(L))/2.
      ASINGAM=SIN(AVGGAM/57.2958)
      AVGHDOT=AVVEL*ASINGAM
      DELH=H(K)-H(L)
      VKT(K)=V(K)/CONV3
      DSEC=DELH/AVGHDOT
      IF (JCODE.EQ.7) GO TO 91
      AGAMMA=GAMMA(L)+DSEC*GAMDOT
      IF (IBUG.LT.2) GO TO 200
      DELVEL=V(K)-V(L)
      WRITE(6,457) AVGNACC,GAMDOT,DSEC,VKT(K),GAMMA(K),AVGHDOT
      WRITE(6,458) AVGTACC,H(K),DELVEL,V(K),DELH,AGAMMA
  457 FORMAT(15X,"F457",6E15.8)
  458 FORMAT(15X,"F458",6E15.8)
  200 DVEL=DSEC*AVGTACC*GZERO
      VEL=V(L)+DVEL
      IF (ABS(1.-(GAMMA(K)/AGAMMA)).LT.0.0005)GO TO 92

      IGAMMA=IGAMMA+1
      IF (IGAMMA.GT.25) GO TO 81
      GAMMA(K)=AGAMMA
      DELVEL=DVEL
      V(K)=VEL
      W(K)=W(L)-AVFF*DSEC/3600.
      IF (ICODE.EQ.3.AND.LCODE.EQ.3) GO TO 70
      GO TO 28
   91 DVEL=DSEC*AVGTACC*GZERO
      VEL=V(L)+DVEL
      
      IF (IBUG.GE.2) WRITE(6,477) H(K),DVEL,VEL
  477 FORMAT(15X,"F477",2X,"H(K)=",F7.0,2X,"DVEL=",E15.8,2X,"VEL=",E15.8)
   92 SVK=V(K)
      V(K)=VEL
      IF (ABS(1.-(SVK/VEL)).LT.0.0005) GO TO 55
      IGAMMA=1
      DELVEL=DVEL
      V(K)=VEL
      IF (JCODE.EQ.7) GO TO 71
      IF (ICODE.EQ.3.AND.LCODE.EQ.3) GO TO 70
      GO TO 28
   71 FCL=CL(K)
      ICUT=0
      GO TO 73
      
!        TMD NEGATIVE     LCODE=3  ICODE=3
!        BACKSPACE AND CLIMB TO PT. WHERE TACC=0.0
   94 LCODE=4
      KPRINT=1
      WRITE(6,95)
   95 FORMAT(3X,"*** HOLD CL, ACCEL TO PT. WHERE TACC=0",65X,"F95"/)
      TSLOPE=TACC(L)/(TACC(L)-TACC(K))
      DELH=(H(K)-H(L))*TSLOPE
      DELVEL=(V(K)-V(L))*TSLOPE
      V(K)=V(L)+DELVEL
      H(K)=H(L)+DELH
      IGAMMA=1
      GO TO 28
      
   96 GAMDOT=57.2958*AVGNACC*GZERO/AVVEL
      AVGGAM=(GAMMA(K)+GAMMA(L))/2.
      ASINGAM=SIN(AVGGAM/57.2958)
      VKT(K)=V(K)/CONV3
      DELH=H(K)-H(L)
      AVGHDOT=AVVEL*ASINGAM
      IF (ICODE.EQ.4) GO TO 250
      DSEC=(GAMMA(K)-GAMMA(L))/GAMDOT
      ADELH=DSEC*AVGHDOT
      IF (IBUG.LT.2) GO TO 201
      DELVEL=V(K)-V(L)
      WRITE(6,459) AVGNACC,GAMDOT,DSEC,VKT(K),GAMMA(K),AVGHDOT
      WRITE(6,460) AVGTACC,H(K),DELVEL,V(K),DELH,ADELH
  459 FORMAT(15X,"F459",6E15.8)
  460 FORMAT(15X,"F460",6E15.8)
  201 IF (ABS(1.-(DELH/ADELH)).LT.0.0005)GO TO 55
      IGAMMA=IGAMMA+1
      IF (IGAMMA.GT.25) GO TO 81
      DELH=ADELH
      H(K)=H(L)+ADELH
      W(K)=W(L)-AVFF*DSEC/3600.
      GO TO 28

  250 DSEC=(H(K)-H(L))/AVGHDOT
      AGAMMA   =GAMMA(L)+DSEC*GAMDOT
      IF (IBUG.LT.2) GO TO 251
      DELVEL=V(K)-V(L)
      WRITE(6,461) AVGNACC,GAMDOT,DSEC,VKT(K),GAMMA(K),AVGHDOT
      WRITE(6,462) AVGTACC,H(K),DELVEL,V(K),DELH,AGAMMA
  461 FORMAT(15X,"F461",6E15.8)
  462 FORMAT(15X,"F462",6E15.8)

      IF (IBUG.GE.3) WRITE(6,470) IGAMMA,GSLOPE,ACL
  470 FORMAT(3X,"*** IGAMMA=",I5,2X,"GSLOPE=",E15.8,2X,"ACL=",E15.8)
  251 IF (GSLOPE.EQ.0.0) GO TO 252
      IF (ABS(1.-(GAMMA(K)/AGAMMA)).LT.0.0005)GO TO 55
      IGAMMA=IGAMMA+1
  252 IF (IGAMMA.GT.25) GO TO 81
!        ASSUME TWO CLS AND USE SLOPE TECH. TO FIND CL FOR
!     GAMMA BALANCE
      IF (ACL.EQ.0.0) GO TO 302
      BCL=CL(K)
      BDG=GAMMA(K)-AGAMMA
      GSLOPE=(BCL-ACL)/(BDG-ADG)
      FCL=ACL-ADG*GSLOPE
      ACL=BCL
      ADG=BDG
      GO TO 73
  302 ACL=CL(K)
      ADG=GAMMA(K)-AGAMMA
      FCL=ACL-.005
      GO TO 73
   55 IF (JCODE.EQ.7) GO TO 67
      IF (LCODE.NE.3) GO TO 78
      IF (ICBDIST.NE.0) GO TO 78
      IF (V(K).LT.(VCLIMB-.2)) GO TO 78
      IF (V(K).LT.(VCLIMB+.2)) GO TO 75
      DELVEL=VCLIMB-V(L)
      SLOPE3=(H(K)-H(L))/(V(K)-V(L))
      DELH=DELVEL*SLOPE3
      H(K)=H(L)+DELH
      V(K)=V(L)+DELVEL
      IF (ISOT.EQ.1) WRITE(6,46) VKCLIMB,H(K)
      IF (ISOT.EQ.2) WRITE(6,46) S4*VKCLIMB,S3*H(K)
   46 FORMAT(3X,"*** MODIFY ALTITUDE TO MEET V CLIMB=",F6.2,            &
     & 4X,"H(K)=",F12.2,51X,"F46"/)
      GO TO 24
      
   75 LCODE=4
   78 IF (TMD(K).GE.0.0) GO TO 67
   
      WRITE(6,66) K
   66 FORMAT(2X,"***NEGATIVE THRUST-MINUS-DRAG FOR POINT NO. ", I2,68X,"F66")
      WSING=W(K)*SINGAM
      WRITE(6,13) INEGT,VKT(K),H(K),ALPHA(K),CL(K),CDD(K),DRAG(K),      &
     &TTNET,TT(K),TDR(K),WSING
      IF (IGAMLIM.NE.0) GO TO 82
      IF (LCODE.EQ.3.AND.ICODE.EQ.3) GO TO 94
      GO TO 82
      
!        ITERATIONS COMPLETED, UPDATE VARIABLES AND PRINT
!        ************************************************
   67 TIME(K)=TIME(L)+DSEC
      W(K)=W(L)-AVFF*DSEC/3600.
      DDIST=AVVEL*DSEC*COSGAM*RZERO/(RZERO+AVH)
      VKT(K)=V(K)/CONV3
      SQRHO=SQRT(ANS(1)/REFRHO)
      VKEAS=VKT(K)*SQRHO
      DIST(K)=DIST(L)+DDIST
      IF (IGAMLIM.EQ.0) GAMMA(K)=GAMMA(L)+DSEC*GAMDOT
      IF (IGAMLIM.NE.0) GAMMA(K)=GAMLIM
      IF (GAMMA(K).LE.GAMLIM) GO TO 245
      IF (H(K).GT.((1.0+TLHOBS)*HOBS)) GO TO 245
      CALGAM=GAMMA(K)
      GAMMA(K)=GAMLIM
!      TEMP. WRITE

      IF (H(K).GT.10.0) WRITE(6,246) K,H(K),DSEC,GAMDOT,CALGAM,GAMMA(K)
  246 FORMAT(10X,"AT K=",I3,2X,"H(K)=",F10.2,2X,"DSEC=",F8.4,2X,           &
     & "GAMDOT=",F8.4,2X,"CALGAM=",F7.4,2X,"GAMMA(K)=",F7.4,10X,"F246"/)

  245 DISL=DIST(L)
      DISK=DIST(K)
      ALTL=H(L)
      ALTK=H(K)
!     WRITE(6,804)DIS1,ALT1,DIS2,ALT2
! 804 FORMAT(6X,4F12.2/)
      IF (ICBDIST.EQ.2.AND.H(K).EQ.CBALT) GO TO 118
      IF (JCODE.EQ.7) GO TO 300
      IF (ICBDIST.EQ.5) GO TO 300
      IF (ICBDIST.EQ.3) GO TO 118
!        TESTS FOR C/B DISTANCE
      IF (DISL.GT.CBDIST.OR.DISK.LT.CBDIST) GO TO 300
      IF (H(K).LE.CBLIMH) GOTO 119
      IF (ABS(DIST(K)-CBDIST).GT.50.0) GO TO 110

  118 ICBDIST=1
!        SOLUTION FOUND FOR C/B POSITION.
      CBALT=10000.
      KPRINT=1
      GO TO 300

  119 CBDIST=50000.
      ICBDIST=2
      CBALT=CBLIMH
      KPRINT=1
      IF (H(K).EQ.CBLIMH) GO TO 118
      GO TO 300

  110 SLPCB=(ALTK-ALTL)/(DISK-DISL)
      HCBDIS=ALTL+(CBDIST-DISL)*SLPCB
      IF (HCBDIS.GE.CBLIMH) H(K)=HCBDIS
      IF (HCBDIS.LT.CBLIMH) H(K)=CBLIMH
      ICBDIST=3
      W(K)=W(L)
      IF (ISOT.EQ.1) WRITE(6,419) VKT(K),DISK,H(K)
      IF (ISOT.EQ.2) WRITE(6,419) S4*VKT(K),S3*DISK,S3*H(K)
  419 FORMAT(3X,"*** AT VEL=",F6.2,2X,",DIST=",F12.2,2X,                 &
     & "EXCEEDS CBDIST. TRY ALT=",F12.2,39X,"F419"/)

      IF (ICBCT.GT.5) GO TO 89
      ICBCT=ICBCT+1
      IF (LCODE.EQ.4.AND.ICODE.EQ.3) LCODE=3
      IGAMMA=1
      ACL=0
      GSLOPE=0.0
      GO TO 504

  300 IF (NACC(K).EQ.0.0.AND.LCODE.EQ.2.AND.H(K).EQ.0.)TIMELOF=TIME(K)
      IF (DIST(K).LE.(STOPDIS+50.)) GO TO 303
      DELH=(H(K)-H(L))*(STOPDIS-DIST(L))/(DIST(K)-DIST(L))
      IGAMMA=1
      ACL=0.0
      GSLOPE=0.0
      H(K)=H(L)+DELH
      GO TO 27

  303 KPRINT=0
!        KPRINT IS CONTROL FOR PRINTOUT
      IF (V(K).NE.VFAIL) GO TO 400
      KPRINT=1
      JCODE=2
      FAILTIM=TIME(K)
  400 IF (V(K).NE.VROT) GO TO 402
      KPRINT=1
      IF (ISOT.EQ.1) WRITE(6,401) VKT(K),DIST(K)
      IF (ISOT.EQ.2) WRITE(6,401) S4*VKT(K),S3*DIST(K)
  401 FORMAT(3X,"*** V ROT=",F6.1,2X,"DIST=",F7.0,84X,"F401"/)
      IF (IPRINT.NE.0) GO TO 402

      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K),                        &
     &S1*W(K),S3*DIST(K),TIME(K)
   39 FORMAT(27X,"M=",F5.2,5X,"H=",F6.2,5X,"W=",F8.0,2X,"DIST=",F7.0,           &
     & 2X,"TIME=",F7.2,28X,"F39"/)

  402 IF (NACC(K).NE.0.0.OR.LCODE.NE.2.OR.H(K).NE.0.0) GO TO 404
      VLOF=V(K)
      VKLOF=VKT(K)
      WLOF=W(K)
      ALFLOF=ALPHA(K)
      FPLOF=FLAP
      DISTLOF=DIST(K)
      IF (ENGSOUT.NE.0.0.AND.IBAL.EQ.2) GO TO 430
      IF (IPA.GT.49) GO TO 430
      C7(IPA)=VKT(K)
      C8(IPA)=ALFLOF
  430 KPRINT=1
      IF (ISOT.EQ.1)WRITE(6,403) VKT(K),ALPHA(K),FLAP,DIST(K)
      IF (ISOT.EQ.2)WRITE(6,403) S4*VKT(K),S5*ALPHA(K),S5*FLAP,S3*DIST(K)
  403 FORMAT(3X,"*** V LOF=",F6.1,2X,"AT ALPHA=",F5.2,2X,"TOFLP=",F5.2,   &
     & 2X,"DIST=",F7.0,55X,"F403"/)
     
      IF (IPRINT.NE.0) GO TO 404
      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K),S1*W(K),S3*DIST(K),TIME(K)
  404 IF (ALPHA(K).NE.ALFROT.OR.ICODE.NE.2) GO TO 406
  
      KPRINT=1
      IF (ISOT.EQ.1) WRITE(6,405) VKT(K),DIST(K)
      IF (ISOT.EQ.2) WRITE(6,405) S4*VKT(K),S3*DIST(K)
  405 FORMAT(3X,"*** V ALROT=",F6.1,2X,"DIST=",F7.0,82X,"F405"/)
      IF (IPRINT.NE.0) GO TO 406
      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K),S1*W(K),S3*DIST(K),TIME(K)
  406 IF (ABS(1.-(H(K)/HOBS)).GE.TLHOBS) GO TO 408
      KPRINT=1
      VKOBS=VKT(K)
      DISTOBS=DIST(K)
      DOBS=DIST(K)
      IF (ENGSOUT.NE.0.0.OR.IPA.GT.49) GO TO 340
      C13(IPA)=DOBS
      IF (IGAMLIM.NE.0) C13(IPA)=-1.0*DOBS

  340 VOBS=V(K)
      FPOBS=FLAP
      WOBS=W(K)
!     IF (ISPARE(10).EQ.1) GO TO 326
!         NO GAMMA LIMIT ABOVE HOBS
!        SAVE AND REINSTATE FOR NEXT T.O. CALC.
      GAMLIM=50.
      IGAMLIM=0

  326 THETA=ALPHA(K)+GAMMA(K)
      IF (ENGSOUT.NE.0.0) GO TO 412
!       SAVE THESE ALL-ENGINE DATA FOR USE IN ClimbGradients.
      SVLOF=VLOF
      SALFLOF=ALFLOF
      SFPLOF=FPLOF
      SWLOF=WLOF
      SVOBS=VOBS
      SWOBS=WOBS
      SFPOBS=FPOBS
      DOBX115=DIST(K)*1.15
      IF (IPA.GT.49) GO TO 327
      C9(IPA)=DOBX115
      C16(IPA)=VKOBS
      IF (SLSTOT.EQ.0.0) GO TO 327
      C11(IPA)=-100.*SLSTOT
      C12(IPA)=100.*TOT

  327 IF (ISEARCH.NE.0) WRITE(6,427) IR,ISK
  427 FORMAT(2X,"  IR=",I5,"  ISK=",I5/)
      IF (ISOT.EQ.1) WRITE(6,407) VKOBS,DIST(K),DOBX115
      IF (ISOT.EQ.2) WRITE(6,407) S4*VKOBS,S3*DIST(K),S3*DOBX115
  407 FORMAT(3X,"*** V OBS=",F6.1,5X,"DOBS=",F7.0,                       &
     &5X,"T.O. FIELD LENGTH=",F7.0,51X,"F407"/)
      IF (ISEARCH .EQ. 0) GOTO 240
!    NOTE% THE SEARCH FOR MIN. REDUCED POWER TO MEET
!      DESTOFL, ALSO LOCATES THE MIN. VR IF THE INITIAL
!      VROT IS INPUT LOW ENOUGH.
      IF (ISK.EQ.1) WRITE(6,423)
  423 FORMAT(4X,"SEARCH LOGIC INCLUDES VARIATIONS ABOVE INPUT VROT"/   &
     & 4X,"IF INPUT LOW, LOGIC WILL LOCATE MIN. VR ALSO.")
      STVROT(IR)=VROTKT(IR)
      STD115(IR)=DOBX115
      IF (IR .EQ.1) GOTO 1005
      IF (STD115(IR) .LE. STD115(IR-1)) GOTO 1005
      SRD115(ISK)=STD115(IR-1)
      SRVROT(ISK)=STVROT(IR-1)
      SRPOWR(ISK)=POWER(1)
      POWTEST=SRPOWR(ISK)
      FICASE=FICASE+.01
      DFU=DESTOFL+0.1
      DFL=DESTOFL-100.
      IF (SRD115(ISK) .GT. DFL .AND. SRD115(ISK) .LE. DFU) GOTO 230
      IR=0
      IF (ISK .GT. 1) GOTO 218
      TPOW=.90
      IF (ASPARE(2).EQ.0.0) GO TO 219
      IF (ASPARE(2).LT.TOWSLS) TPOW=ASPARE(2)/TOWSLS
      GO TO 219
      
  218 IF (ISK.EQ.10) GO TO 225
      DISSLP=(DESTOFL-SRD115(ISK-1))/(SRD115(ISK)-SRD115(ISK-1))
      TPOW=SRPOWR(ISK-1)+(SRPOWR(ISK)-SRPOWR(ISK-1))*DISSLP
      IF (ISK.GT.2.AND.TPOW.GT.1.0) GO TO 227
      IF (ISK.GT.2) GO TO 219
      IF (SRD115(ISK).GE.DESTOFL) GO TO 219
      TPOW=(SRPOWR(ISK)+TPOW)/2.0
  219 DO 220 IL=1,25
      POWER(IL)=TPOW
  220 END DO
  
  
      WRITE(6,425) IR,ISK,SRD115(ISK),SRVROT(ISK),SRPOWR(ISK)
  425 FORMAT(2X,"IR=",I5,"  ISK=",I5,"  SRD115=",F9.1,"  SRVROT=",F7.2,     &
     & "  SRPOWR=",F8.5,48X,"F425"/)
      ISK=ISK+1
      GOTO 1005
  225 WRITE(6,226)
  226 FORMAT(3X,"**** CASE TERMINATED, THROTTLE SEARCH ISK EXCEEDED",64X,"F226"/)
      GO TO 229
  227 WRITE(6,228)
  228 FORMAT(3X,"**** CASE TERMINATED, THROTTLE REQD. EXCEEDS DATA.", 65X,"F228"/)
  229 ISEARCH=2
      IPA=ISIPA
  230 CONTINUE
!             SOLUTION HAS BEEN FOUND, BACKSPACE AND  FINISH CLIMBOUT.
      ISEARCH=2
      IR=1
      VROTKT(1)=SRVROT(ISK)
      IROT=0
      GO TO 1006
  240 CONTINUE
      GO TO 416
  412 IF (IPA.LE.49) C14(IPA)=DIST(K)
      IF (IGAMLIM.NE.0.AND.C14(IPA).NE.0.0) C14(IPA)=-1.0*C14(IPA)
      IF (ISOT.EQ.1) WRITE(6,414) VKOBS,DIST(K)
      IF (ISOT.EQ.2) WRITE(6,414) S4*VKOBS,S3*DIST(K)
  414 FORMAT(3X,"*** V OBS=",F6.1,45X,"ENGOUT DOBS=",F7.0,              &
     &4X,"*****",25X,"F414"/)
  416 IF (IPRINT.NE.0) GO TO 408
      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K), S1*W(K),S3*DIST(K),TIME(K)
  408 CONTINUE
      IF (H(K).EQ.0.0.OR.H(K).NE.C13(IPA)) GO TO 420
      KPRINT=1
  420 FLYPT=3.5*CONV1
      IF (DISL.GT.FLYPT.OR.DISK.LT.FLYPT) GO TO 421
!        INTERP. BETWEEN L AND K FOR ALT. ABOVE 3.5 NM. PT.
      SLP35=(ALTK-ALTL)/(DISK-DISL)
      S35H=ALTL+(FLYPT-DISL)*SLP35
      IF (IPA.LE.49) C10(IPA)=S35H
      KPRINT=1
  421 IF (DIST(K).GE.DTEST) KPRINT=1
      IF (ABS(DIST(K)-STOPDIS).LE.50.) KPRINT=1
      IF (KPRINT.EQ.0) GOTO418
      IF (IPFAIL.EQ.1.AND.ISPARE(5).EQ.0) GO TO 418
      TGOTG=CBPERT
      IF (CBPERT.EQ.1.0) TGOTG=TOT
      IF (IPRINT.NE.1) GO TO 409
      IF (ISOT.EQ.1)WRITE(6,30)K,VKT(K),MACH(K),H(K),W(K),DIST(K),TIME(K)&
     &,ICODE,LCODE,JCODE
      IF (ISOT.EQ.2) WRITE(6,30) K,S4*VKT(K),MACH(K),S3*H(K),            &
     &S1*W(K),S3*DIST(K),TIME,ICODE,LCODE,JCODE
      GO TO 410
  409 IF (IPRINT.LT.2) GO TO 410
      IF (ISOT.EQ.1) WRITE(6,49)K,VKT(K),MACH(K),H(K),W(K),DIST(K),      &
     &TIME(K),ICODE,LCODE,JCODE,VKEAS,Q(K),                             &
     &ALPHA(K),CL(K),CDD(K),DRAG(K),LOD(K),                             &
     &TT(K),TDR(K),TFF(K),TACC(K),NACC(K),GAMMA(K)
     
      IF (ISOT.EQ.2) WRITE(6,49)K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),     &
     &S3*DIST(K),TIME(K),ICODE,LCODE,JCODE,                             &
     &S4*VKEAS,S6*Q(K),S5*ALPHA(K),CL(K),CDD(K),                        &
     &S1*DRAG(K),LOD(K),S1*TT(K),S1*TDR(K),S7*TFF(K),TACC(K),           &
     &NACC(K),S5*GAMMA(K)
     
      IF (JSJTCD.EQ.0.AND.ISPARE(3).EQ.0) GO TO 330
      IF (JSJTCD.NE.0.AND.ISPARE(3).NE.0) GO TO 328
      IF (JSJTCD.NE.0) WRITE(6,314) TGOTG,TNOTN
      IF (ISPARE(3).NE.0) WRITE(6,316) FLAP
      GO TO 330
  328 WRITE(6,319) TGOTG,TNOTN,FLAP
  330 WRITE(6,322)
!
  410 DISTPLT=DIST(K)
      IF (H(K).GT.HOBS) GO TO 422
      DTEST=DISTPLT+PINT1
      GO TO 426
  422 IF (LCODE.GE.4) GO TO 424
      DTEST=DISTPLT+PINT2
      GO TO 426
  424 DTEST=DISTPLT+PINT3
  426 CONTINUE
  418 CONTINUE
!      THIS LOCATION USED TO SAVE TIME HISTORY DATA
!       *IF* NCODE.NE.0,  AND  IBAL.EQ.0  AND
!            DIST(K) IS IN DISTPLT LIMIT.
!      TYPICAL%  ET(NZP)=TIME(K),  EH(NZP)=H(K),  ETC.
!      THEN  NZP=NZP+1     (nzp also reqd. in subroutine Approach.)
!     DETERMINE LOGIC FOR NEXT STEP.
!     *************************************************
      IF (JCODE.NE.2.OR.V(K).NE.VFAIL) GO TO 411
      IF (ISOT.EQ.1) WRITE(6,417) VFAILKT,DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,417) S4*VFAILKT,S3*DIST(K),TIME(K)
  417 FORMAT(3X,"***** ENGINE FAILURE AT VEL=",F6.1,                    &
     &2X,"DIST=",F7.0,2X,"FAILTIME=",F6.2,49X,"F417"/)
      IF (IPRINT.NE.0) GO TO 411
      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K),S1*W(K),S3*DIST(K),TIME(K)
  411 IF (IBAL.EQ.0.OR.V(K).NE.VFAIL) GO TO 413
      SVEL=V(K)
      SMACH=MACH(K)
      SALT=H(K)
      SWT=W(K)
      SDIST=DIST(K)
      STIME=TIME(K)
      SALPH=ALPHA(K)
      STACC=TACC(K)
      SNACC=NACC(K)
      ATRTIME=STIME+DTREC
      IF (DTREC.EQ.0.0) ATRTIME=ATRTIME+.01
  413 IF (ABS(DIST(K)-STOPDIS).LE.50.) GO TO 301
      IF (H(K).GT.ALTLIM) GO TO 80
      IF (LCODE.EQ.4) GO TO 65
      IF (LCODE.EQ.3.AND.HSTEP1.GE.5000.) GO TO 83
      IF (LCODE.EQ.3.AND.ICODE.EQ.3) GO TO 62
      IF (LCODE.EQ.2)GO TO 50
      IF (V(K).LT.VROT) CYCLE   !!! GO TO 20
      IF (ALPHA(K).EQ.ALFROT) GO TO 40
!      ROTATE IN ALPHA INCREMENTS, SOLVE FOR DELVEL
      ICODE=2
      DTASS=SAVDTAS
!        FIRST GUESS FOR DELVEL
      DELVEL=DTASS*(TACC(K)*GZERO)
      DELH=DTASS*AVGHDOT
      CYCLE   !!! GO TO 20
   40 ICODE=3
!        HAVE REACHED ALFROT
      DELVEL=VSTEP2*CONV3
      IF (JCODE.EQ.1) CYCLE   !!! GO TO 20
      IF (TIME(K).LT.(FAILTIM+DTFAIL)) GO TO 86
      CYCLE   !!! GO TO 20
!      LCODE=2, EQUATIONS FOR DELVEL DURING LIFTOFF TO HOBS
   50 DELVEL=VSTEP2*CONV3
      IF (ICODE.EQ.1) GO TO 84
      IF (ICODE.EQ.3) GO TO 56
      IF (ALPHA(K).NE.ALFROT) GO TO 58
      ICODE=3
   58 DTASS=DELVEL/(TACC(K)*GZERO)
      IF (ICODE.EQ.3) GO TO 56
      RDTASS=(ALFROT-ALPHA(K))/ROTRATE
!       ESTIMATE WHICH EVENT (DELVEL OR ALFROT) WILL OCCUR FIRST
      IF (DTASS.LE.RDTASS) GO TO 56
      DTASS=RDTASS
!      FIRST GUESS FOR DELVEL
      DELVEL=DTASS*(TACC(K)*GZERO)
   56 IF (NACC(K).EQ.0.0.AND.H(K).EQ.0.0) GO TO 59
      CYCLE   !!! GO TO 20
   59 DELH=DTASS*AVGHDOT
      CYCLE   !!! GO TO 20
!      EQUATIONS FOR CLIMB-OUT AFTER HOBS
   62 IF (ABS(1.-(H(K)/HOBS)).GE.TLHOBS) GO TO 63
      H(K)=HOBS
      IF (VFAIL.LE.VROT.AND.IBAL.NE.0) GO TO 900
!        NOTE. 12/77  AT LATER DATE WILL MOVE THIS TEST TO BETTER LOC.
      V2=VKT(K)
      VKCLIMB=V2+DVCLIMB
      IF (DVCLIMB.GE.100.) VKCLIMB=DVCLIMB
      IF (VKCLIMB.LT.V2) VKCLIMB=V2
      IF (VKCLIMB.GT.V2) GO TO 74
      VKCLIMB=V2
      VCLIMB=V(K)
      LCODE=4
      GO TO 65
   74 VCLIMB=VKCLIMB*CONV3
      IF (IALPHA.EQ.0.AND.ISOT.EQ.1) WRITE(6,85) ALPHA(K)
      IF (IALPHA.EQ.0.AND.ISOT.EQ.2) WRITE(6,85) S5*ALPHA(K)
      IF (IALPHA.EQ.1.AND.ISOT.EQ.1) WRITE(6,79) THETA
      IF (IALPHA.EQ.1.AND.ISOT.EQ.2) WRITE(6,79) S5*THETA
      IF (IALPHA.EQ.2.AND.ISOT.EQ.1) WRITE(6,48) ALPHA(K)
      IF (IALPHA.EQ.2.AND.ISOT.EQ.2) WRITE(6,48) S5*ALPHA(K)
   79 FORMAT(3X,"*** CLIMB AND ACCEL. AT THETA=",F5.2,78X,"F79"/)
   85 FORMAT(3X,"*** CLIMB AND ACCEL. AT ALPHA=",F5.2,78X,"F85"/)
   48 FORMAT(3X,"*** CLIMB AND ACCEL., NEW LOGIC."/)
   63 DELVEL=V(K)-V(L)
      IGAMMA=1
      DELH=HSTEP1
      CYCLE   !!! GO TO 20
      
!        EQUATIONS FOR CONSTANT VEL. CLIMB    LCODE=4  ICODE=4
   65 IF (ICODE.EQ.4) GO TO 68
      IF (ISOT.EQ.1) WRITE(6,39) MACH(K),H(K),W(K),DIST(K),TIME(K)
      IF (ISOT.EQ.2) WRITE(6,39) MACH(K),S3*H(K), S1*W(K),S3*DIST(K),TIME(K)
      SVKCLMB=VKT(K)
      IF (ISOT.EQ.1) WRITE(6,87) H(K),VKT(K)
      IF (ISOT.EQ.2) WRITE(6,87) S3*H(K),S4*VKT(K)
   87 FORMAT(3X,"*** ABOVE H=",F6.0,2X," CLIMB AT VEL=",F6.1,74X, "F87"/)
      IF (VKT(K).LT.VKCLIMB) WRITE(6,122) VKCLIMB
  122 FORMAT(3X,"*** CLIMB VEL. LESS THAN DESIRED, (", F6.2, " )",71X,"F122"/)
      TEST=V2+9.95
      IF (VKT(K).GE.TEST) GO TO 68
      WRITE(6,97) VKT(K)
   97 FORMAT(3X,"*** CLIMB VEL. (",F6.2, ") IS LESS THAN V2+10.", 71X,"F97"/)
   68 ICODE=4
      IF (JCODE.NE.7) TACC(K)=0.0
      DELVEL=0.0
      DELH=HSTEP2
      CLSTART=CL(K)-.01
      IGAMMA=1
      IF (JCODE.EQ.7) CYCLE   !!! GO TO 20
      IF (V(K).EQ.V(L)) CYCLE   !!! GO TO 20
      IF (H(K).EQ.HOBS) CYCLE   !!! GO TO 20
      DH=H(K)-H(L)
      IF (DH.NE.HSTEP1) DELH=HSTEP2-DH
   20 END DO   !        STATEMENT 20 IS END OF MAIN LOOP  ***************
!     *************************************************

  900 JCODE=4
!        EQUATIONS FOR REFUSED T.O..  PICK UP DATA FROM VFAIL POINT
!        MAY WISH TO INDEX FICASE(+.01) FOR EACH TRIAL VFAIL.
      IF (IPRINT.EQ.0) GO TO 902
      IF (ISOT.EQ.1) WRITE(6,901) VFAILKT
      IF (ISOT.EQ.2) WRITE(6,901) S4*VFAILKT
  901 FORMAT(3X,"***** DETERMINE REFUSED TAKEOFF FOR V FAIL=",F8.3,62X,"F901"/)
  902 L=1
      ICUT=0
      JRJTCD=JTCODE
      V(1)=SVEL
      MACH(1)=SMACH
      H(1)=SALT
      W(1)=SWT
      DIST(1)=SDIST
      TIME(1)=STIME
      ALPHA(1)=SALPH
      TACC(1)=STACC
      NACC(1)=SNACC
      TFF(1)=0.0
      GAMMA(1)=0.0
      CALL REFUSED
      JTCODE=JRJTCD
      IF (LOC.EQ.1000) GO TO 146
      REFTOD=DIST(K)
      IF (REFTOD.NE.0.0.AND.IPA.LE.49) C10(IPA)=REFTOD
      IF (JCODE.EQ.10) GO TO 88
      IF (ISOT.EQ.1) WRITE(6,905) VFAILKT,CSPARE(2),REFTOD
      IF (ISOT.EQ.2) WRITE(6,905) S4*VFAILKT,S4*CSPARE(2),S3*REFTOD
  905 FORMAT(3X,"***** FOR VEFKT=",F8.3," (V1=",F8.3,")",                &
     & 10X,"THE REFUSED T.O. DIST.=",F7.0,4X,"*****",25X,"F905"//)
      IF (IBAL.EQ.1) GO TO 80
!        SOLVE FOR CRITICAL VFAIL PT.
      IF (IFAIL.GE.1) GO TO 908
      UNBAL1=REFTOD-DOBS
      IF (UNBAL1.LT.-50.) GO TO 914
      IF (ABS(UNBAL1).LE.50.) GO TO 910
      VK1=VFAILKT
      VFAILKT=VFAILKT-5.
      GO TO 907
  908 UNBAL2=REFTOD-DOBS
      IF (ABS(UNBAL2).LE.50.) GO TO 910
!        INTERPOLATE FOR VFAILKT FOR UNBAL=0.0
      VK2=VFAILKT
      VSLOPE=(VK2-VK1)/(UNBAL2-UNBAL1)
      VFAILKT=VK1-UNBAL1*VSLOPE
      IF (VFAILKT.GE.VROTKT(IR)) GO TO 914

      VK1=VK2
      UNBAL1=UNBAL2
  907 IFAIL=IFAIL+1
      IF (IFAIL.GT.10) GO TO 88

      IF (ISOT.EQ.1) WRITE(6,909) VFAILKT
      IF (ISOT.EQ.2) WRITE(6,909) S4*VFAILKT
  909 FORMAT(3X,"*** SELECT NEW VFAIL=",F8.3,85X,"F909"/)
      GO TO 103
  910 VEF=VFAILKT
      V1=CSPARE(2)
      BALFL=DOBS
      IF (IPA.GT.49) GO TO 911
      C11(IPA)=VFAILKT
      C19(IPA)=V1
      C12(IPA)=BALFL
      IF (IATR.NE.0) C12(IPA)=-1.0*BALFL

  911 CONTINUE
      IF (ISOT.EQ.1) WRITE(6,912) VROTKT(IR),VEF,V1,BALFL
      IF (ISOT.EQ.2) WRITE(6,912) S4*VROTKT(IR),S4*VEF,S4*V1,S3*BALFL
  912 FORMAT(3X,"***** FOR VROT=",F6.1,", AN ENGINE FAILURE AT VEF=",  &
     & F6.1," (V1=",F6.1,"), THE BALANCED F.L. IS=",F7.0,                 &
     & " *****",12X, "F912"/)
      ACON=1.0/CONV3
      IF (ISOT.EQ.1) WRITE(6,930) ACON*VLOF,ACON*VOBS
      IF (ISOT.EQ.2) WRITE(6,930) S3*VLOF,S3*VOBS
  930 FORMAT(11X,"THE ENGINE OUT VLOF=",F6.1, " AND THE VOBS=", F6.1)

      IF (ISOT.EQ.1) WRITE(6,931) ACON*SVLOF,ACON*SVOBS
      IF (ISOT.EQ.2) WRITE(6,931) S3*SVLOF,S3*SVOBS
  931 FORMAT(13X,"(ALL ENGINE DATA, VLOF=",F6.1,2X,"VOBS=",F6.1,1X,")"/)
      GO TO 80

  913 WRITE(6,923)
  923 FORMAT(3X,"**** CASE TERMINATED, NEG. DELV OR DELH.", 74X, "F923"/)
      GO TO 93

  914 WRITE(6,915) VROTKT(IR)
  915 FORMAT(3X, &
    "***** A BALANCED F.L. CANNOT BE DETERMINED AT VROTKT=",F6.1,54X,"F915"/)
      GO TO 315

   81 WRITE(6,921) VKT(L),H(L),GAMMA(L)
  921 FORMAT(9X,"**** CASE TERMINATED, GAMMA ITER. EXCEEDED."/         &
     &12X, "( AT LAST POINT, VKT=",F7.2,2X,"H=",F12.2,2X,                 &
     &"GAMMA=", F7.4,")",44X,"F921")
      GO TO 93

   82 WRITE(6,922) VKT(L),H(L),TACC(L)
  922 FORMAT(9X,"**** CASE TERMINATED, NEG. TMD DURING ITERATION."/    &
     &12X, "( AT LAST POINT, VKT=", F7.2,2X,"H=", F12.2, 2X,                 &
     & "TACC=",F9.5,")",43X,"F922")
      IF (ISEARCH.NE.0) ISEARCH=2
      GO TO 93

   83 WRITE(6,924)
  924 FORMAT(3X,"**** CASE TERMINATED AT OBSTACLE",81X,"F924"/)
!!!      IF (IPFAIL.EQ.1) GO TO 62
                               ! just a guess, RLC
      IF (IPFAIL.EQ.1) GO TO 93
      GO TO 920

  301 WRITE(6,305)
  305 FORMAT(3X,"**** CASE TERMINATED, BY STOPDIS", 81X,"F305"/)
      GO TO 920

   84 WRITE(6,925)
  925 FORMAT(3X,"**** CASE TERMINATED, LOWER VROTKT SUGGESTED",        &
     &69X,"F925"/)
      GO TO 93

   86 WRITE(6,926)
  926 FORMAT(3X,"**** CASE TERMINATED, ICODE=3 BEFORE JCODE=3.",2X,    &
     & "EVENT NOT PROGRAMMED."/8X,"SUGGEST DEC. ROTRATE OR INCR. ",    &
     &"DTFAIL.",70X,"F926"/)
      GO TO 93

   88 WRITE(6,927) VKT(K)
  927 FORMAT(3X,"**** CASE TERMINATED DURING REFUSED T.O. AT VKT=",    &
     &F8.3,57X,"F927"/)
      GO TO 93

   89 WRITE(6,928)
  928 FORMAT(3X,"**** CASE TERMINATED DURING CBDIST ITERATION",        &
     &69X,"F928"/)
      GO TO 93

  143 WRITE(6,144)
  144 FORMAT(3X,"**** CASE TERMINATED. DELH ITER. EXCEEDED.",          &
     &71X,"F144"/)
      GO TO 93

  308 WRITE(6,309)
  309 FORMAT(3X,"**** CASE TERMINATED. IN LOOP DURING BAL.F.L. CALCS.", &
     &61X,"F309"/)
      GO TO 93

  146 WRITE(6,147)
  147 FORMAT(3X,"**** CASE TERMINATED, FTLUP2 ERROR.",77X,"F147"/)
   93 IF (ICODE.GE.9) GO TO 810
      SEL=L
      IF (IPRINT.LE.1) GO TO 313
      WRITE(6,310)
  310 FORMAT(3X,"**** DETAILS OF TWO PREVIOUS POINTS.",                &
     & 4X,"(V EAS WILL BE FOR LAST POINT.)"/)
      K=SEL-1
      IF (K.LT.1) K=1

  311 IF (ISOT.EQ.1) WRITE(6,49)K,VKT(K),MACH(K),H(K),W(K),DIST(K),      &
     &TIME(K),ICODE,LCODE,JCODE,VKEAS,Q(K),                             &
     &ALPHA(K),CL(K),CDD(K),DRAG(K),LOD(K),                             &
     &TT(K),TDR(K),TFF(K),TACC(K),NACC(K),GAMMA(K)

      IF (ISOT.EQ.2) WRITE(6,49)K,S4*VKT(K),MACH(K),S3*H(K),S1*W(K),     &
     &S3*DIST(K),TIME(K),ICODE,LCODE,JCODE,                             &
     &S4*VKEAS,S6*Q(K),S5*ALPHA(K),CL(K),CDD(K),                        &
     &S1*DRAG(K),LOD(K),S1*TT(K),S1*TDR(K),S7*TFF(K),TACC(K),           &
     &NACC(K),S5*GAMMA(K)

      K=K+1
      IF (K.EQ.(SEL+1)) WRITE(6,312) ICODE,LCODE,JCODE
  312 FORMAT(6X,"** AT TIME OF PROBLEM",2X,"ICODE=",I3,2X,"LCODE=",I3,   &
     &2X,"JCODE=",I3/6X,"** ITERATION MAY NOT BE COMPLETED AT FOLLOWING POINT.")
      IF (K.GT.(SEL+1)) GO TO 313
      GO TO 311
  313 L=SEL
      K=L+1
      IF (IGAMLIM.NE.0.OR.LCODE.GE.3) GO TO 324
      WRITE(6,323)
  323 FORMAT(/5X,"*** MAY WISH TO LIMIT GAMMA WITH INPUT (GAMLIM). ",63X,"F323"/)
  324 IF (LCODE.GE.3) WRITE(6,325)
  325 FORMAT(15X,"*** MAY WISH TO TRY OTHER ANGLE OF ATTACK CONTROL",  &
     &" (ISPARE(1)).      F325")
      IF (IBAL.NE.2.OR.DOBX115.EQ.0.0) GO TO 315
      ITVFAIL=ITVFAIL+1
      IF (ITVFAIL.GE.6) GO TO 315
      IF (H(L).LE.HSVFAIL) GO TO 315
      HSVFAIL=H(L)
      VFAILKT=VFAILKT-5.0
      WRITE(6,317) VFAILKT
  317 FORMAT(/30X,"*** TRY LOWER VFAILKT=",F7.3, 57X,"F317"//)
      IFAIL=0
      IPFAIL=1
      GO TO 103

  315 IF (IPA.EQ.0) IPA=1
      IF (C10(IPA).NE.0.0) C10(IPA)=-1.0*C10(IPA)
      IF (C10(IPA).EQ.0.0) C10(IPA)=-1.0
      IF (ISEARCH.NE.2)GO TO 80
      C6(IPA)=0.0
      C7(IPA)=0.0
      C8(IPA)=0.0
      C9(IPA)=0.0
      C16(IPA)=0.0
      C11(IPA)=0.0
      C12(IPA)=0.0
      GO TO 810

  920 IF (IBAL.LE.1) GO TO 80

      WRITE(6,4015)
 4015 FORMAT("   DETERMINE THE CRITICAL V1, FOR A BALANCED FIELD LENGTH", &
      " BY ASSUMING AN ENGINE FAILURE AT SEVERAL VFAILKTS.")
      WRITE(6,4016)
 4016 FORMAT("      THEN DETERMINE THE F.A.R. FIELD LENGTH FOR THIS VR.",58X,"F4016"//)
      VFAILKT=VROTKT(IR)-ASPARE(9)
      IPFAIL=IPFAIL+1
      IF (IPFAIL.GT.1) GO TO 308
      GO TO 103
   80 CONTINUE
!     IF (IBAL.EQ.1) GO TO 704
      IF (ISEARCH.NE.0) GO TO 704
      IF (VK1ST.NE.0.0) GO TO 704
!     IF (H(K).LT.((1.-TLHOBS)*HOBS)) GO TO 704
      FLAP=TOFLP
      IF (BALFL.NE.0.0) GO TO 702
!        RE-INSTATE  ALL-ENGINE  RESULTS.
      VLOF=SVLOF
      ALFLOF=SALFLOF
      FPLOF=SFPLOF
      WLOF=SWLOF
      VOBS=SVOBS
      WOBS=SWOBS
      FPOBS=SFPOBS
  702 IF (IMIL.NE.0) GO TO 706
      CALL ClimbGradients()
      IF (LOC.EQ.1000) GO TO 146

      IF (MISGRAD.NE.0) C9(IPA)=-1.0*DOBX115
  704 IF (BALFL.EQ.0.0) GO TO 706
      FARFL=BALFL
      IF (DOBX115.GT.BALFL) FARFL=DOBX115
      C18(IPA)=FARFL
      IF (ISOT.EQ.1) WRITE(6,705) VROTKT(IR),FARFL
      IF (ISOT.EQ.2) WRITE(6,705) S4*VROTKT(IR),S3*FARFL
  705 FORMAT(//5X,"******* FOR A VROT=", F6.1, &
       " THE F.A.R. FIELD LENGTH=", F7.0,2X,"*******",44X,"F705"//)
  706 IF (VZERC.EQ.0.0) GO TO 708
      VKOBS=C16(IPA)
      IF (VKOBS.LT.(1.125*VZERC)) C16(IPA)=-1.0*C16(IPA)

  708 WRITE(6,206)
!        RUN SUMMARY IS NEAR .1972
  206 FORMAT(2X,"*****   CASE SUMMARY"/                                &
     &3X,"CASE  GW/K   WOS   TOW  TOFLP  VROT  ",                      &
     & "ALROT  VLOF  ALLOF  VOBS  ",                                    &
     &"TO F.L. VFAIL  BALF.L. FARF.L.  ",                              &
     & "HH,C/B  D,C/B  V,C/B T/TNOR H,3.5")
      IF (ISOT.EQ.2) GO TO 211
      WRITE(6,2001)
      WRITE(6,2004)C1(IPA),C2(IPA),C3(IPA),C4(IPA),C17(IPA),C5(IPA),    &
     &C6(IPA),C7(IPA),C8(IPA),C16(IPA),C9(IPA),C11(IPA),C12(IPA),       &
     &C18(IPA),C13(IPA),C14(IPA),C19(IPA),C15(IPA),C10(IPA)
      GO TO 214

  211 WRITE(6,2003)
  212 WRITE(6,2004)C1(IPA),S1*C2(IPA),S6*C3(IPA),C4(IPA),S5*C17(IPA),   &
     &S4*C5(IPA),S5*C6(IPA),S4*C7(IPA),S5*C8(IPA),S4*C16(IPA),          &
     &S3*C9(IPA),S4*C11(IPA),S3*C12(IPA),S3*C18(IPA),S3*C13(IPA),       &
     &S3*C14(IPA),S4*C19(IPA),C15(IPA),S3*C10(IPA)
  214 WRITE(6,207)
  207 FORMAT(////)
      GAMLIM=SGAMLIM
      GO TO 1005

  700 IF (ILAND.EQ.0) GO TO 730
      IF (ILAND.EQ.1) GO TO 710
      
      IF (ISOT.EQ.1) WRITE(6,720) FICASE,WTLAND
      IF (ISOT.EQ.2) WRITE(6,720) FICASE,S1*WTLAND
  720 FORMAT(//3X,"*** CASE=",F5.2,2X,"WTLAND=",F9.1/)
  
  710 CALL Approach()
      IF (LOC.EQ.1000) GO TO 146
      IF (ILAND.EQ.0.OR.IBAL.NE.0) GO TO 730
!       LANDING CALC. ONLY IF IBAL EQUAL 0.
      SDTFAIL=DTFAIL
      DTFAIL=0.0
      ENGSOUT=0.0
      IF (IPRINT.NE.0) WRITE(6,714)
  714 FORMAT(//4X,"CALC. GLIDE SLOPE FROM OBSTACLE AND GROUND ROLL DISTANCE."/)

      VLAND=APVKT2   !       CALC. FLARE TO ST. OF GROUND ROLL.
      V(1)=APVKT2*CONV3
!         AT THIS LOCATION, INSERT MORE DETAILED VERSION
!          (WITH FLARE, TOUCHDOWN, ROTATE)  LATER.
      APPRDX=0.0
      APPRDT=0.0
      APPRRS=0.0
      IF (GLIDE2.EQ.0.0) GO TO 715
      APPRDX=HOBSL/TAN(GLIDE2/57.2958)
      APPRRS=V(1)*SIN(GLIDE2/57.2958)
      APPRDT=HOBSL/APPRRS
  715 MACH(1)=APM2
      IF (IPRINT.NE.0) WRITE(6,713) GLIDE2,APPRDX,APPRDT
  713 FORMAT(8X,"FOR GLIDE ANGLE=",F5.2,2X,"GLIDE DIST=",F8.3,          &
     &2X,"GLIDE TIME=",F6.2/)
      H(1)=0.0
      W(1)=WTLAND
      DIST(1)=APPRDX
      TIME(1)=APPRDT
      ALPHA(1)=ALFROLL
      TACC(1)=0.0
      NACC(1)=0.0
      TFF(1)=0.0
      GAMMA(1)=0.0
      JCODE=1
      JTCODE=0
      ICODE=10
      CALL REFUSED()
      DISLAND=DIST(K)
      DTFAIL=SDTFAIL
      GRDIST=DISLAND-APPRDX
      GRTIME=TIME(K)-APPRDT

      WRITE(6,716) WTLAND,TREV
  716 FORMAT(//14X,"**  FOR LANDING WEIGHT=",F10.0,                     &
     &4X,"TR=",F6.3,65X," F716")

      IF (APPRDX.NE.0.0) WRITE(6,717) GLIDE2,APVKT2,APPRDT,APPRDX
  717 FORMAT(20X,"APPR. GLIDE ANGLE=",F5.2,2X,"VAPPR,KTS=",            &
     &F7.2,"APPRDT=",F6.2,2X,"APPRDX=",F8.2)
      WRITE(6,718) VLAND,GRTIME,GRDIST
  718 FORMAT(20X,"VTD,KTS=",F7.2,2X,"ROLL TIME=",F6.2,                   &
     &2X,"GROUND ROLL DIST.=",F8.2)
      IF (APPRDX.NE.0.0) WRITE(6,719) TIME(K),DISLAND
  719 FORMAT(17X,"** TOTAL TIME=",F6.2,4X,                              &
     & "TOTAL LANDING DIST.=",F8.2,50X," F719")

  730 CONTINUE   !        STORE  LAND ONLY  RESULTS FOR SUMMARY PRINTOUT.
  810 CONTINUE   !         ( PREVIOUS LOCATION OF NOISE TAPE WRITES )

      DO 812 LR=1,10
      VROTKT(LR)=SVRV(LR)
  812 END DO

      CBALT=SCBALT
      CBDIST=SCBDIST
      IF (ISEARCH.EQ.0) GO TO 1500
 
!!!      NCODE=SNCODE     !!! sncode never set????  disabled 19Jul09
      IROT=ISIROT
      IBAL=ISIBAL
      IPRINT=ISIPRNT
      ISEARCH=ISISRCH
!     IF (ISEARCH.EQ.2) ISEARCH=0
      ISPARE(2)=ISISPR2
      FICASE=SFICASE
      JTCODE=ISJTCOD

!... Restore original values of power and tlvar ................................
      DO 205 IL=1,25
      POWER(IL)=SPOWER(IL)
      TLVAR(IL)=STLVAR(IL)
  205 END DO

 1500 IF (ISIN.EQ.1) GO TO 1000
      ISIN=1
      CALL CONSI()
!    FOR REPEAT CASES CONVERT INTERNAL DATA BASE BACK TO SI UNITS.
      ISIN=2
      GO TO 1000   ! This case is complete. Go back and read another.
      
      
      
      
      
      
! Now all input records have been read and processed. Write the summary.      
 2000 IF (IPA.EQ.1) GO TO 2100
      IF (ISEARCH.GE.10) GO TO 2150
!        WRITE SUMMARY ARRAYS.
      WRITE(6,5) TRIM(mainTitle),TODAY
      WRITE(6,2002) FICASE-1.000
!        CASE SUMMARY IS NEAR .1858
 2002 FORMAT(25X,"SUMMARY OF TAKEOFF RESULTS FOR",F6.0," CASES"//      &
     &3X,"CASE   GW    W/S   T/W  TOFLP  VROT  ",                      &
     & "ALROT  VLOF  ALLOF  VOBS  ",                                    &
     & "TO F.L. VFAIL  BALF.L. FARF.L.  ",                              &
     & "H,C/B  D,C/B  V,C/B T/TNOR H,3.5")
      IF (ISOT.EQ.1) WRITE(6,2001)
 2001 FORMAT(8X,"LBF/K   PSF    -    DEG    KT    DEG    KT    DEG   ",&
     & " KT     FT     KT      FT      FT       FT    FT     KT     -     FT")
      IF (ISOT.EQ.2) WRITE(6,2003)
 2003 FORMAT(10X,"KN    KPA    -    DEG   M/S    DEG   M/S    DEG   M/S", &
     & "     M     M/S       M       M       M     M    M/S     -     M" )
      WRITE(6,2005)
 2005 FORMAT(22X,"(1)",23X,"(2)   (2)    (2)    (3)",                  &
     & "    (4)    (5)",41X, "(6)" )
!     IPB=IPA-1
      IPB=IPA
      IF (IPB.GT.49) IPB=49
      IF (ISOT.EQ.2) GO TO 820
      WRITE(6,2004) (C1(N),C2(N),C3(N),C4(N),                           &
     &C17(N),C5(N),C6(N),C7(N),C8(N),                                   &
     &C16(N),C9(N),C11(N),C12(N),C18(N),                                &
     &C13(N),C14(N),C19(N),C15(N),C10(N),N=1,IPB)
      GO TO 822

  820 WRITE(6,2004) (C1(IPA),S1*C2(IPA),S6*C3(IPA),C4(IPA),S5*C17(IPA), &
     &S4*C5(IPA),S5*C6(IPA),S4*C7(IPA),S5*C8(IPA),S4*C16(IPA),          &
     &S3*C9(IPA),S4*C11(IPA),S3*C12(IPA),S3*C18(IPA),S3*C13(IPA),       &
     &S3*C14(IPA),S4*C19(IPA),C15(IPA),S3*C10(IPA),IPA=1,IPB)

 2004 FORMAT(2X,F5.2,1X,F6.1,1X,F5.1,1X,F5.3,                           &
     &1X,F5.1,2X,F5.1,1X,F5.2,2X,F5.1,1X,F5.2,                          &
     &1X,F6.1,1X,F7.0,1X,F6.1,1X,F7.0,1X,F7.0,                          &
     &1X,F6.0,1X,F7.0,1X,F5.1,1X,F5.3,1X,F6.0)
 
 822 WRITE(6,2007)
 2007 FORMAT(/66X,"( FOR BAL. F.L. RUNS ONLY )      TOD",              &
     & "   EOTOD   V1         REFTOD"/                                  &
     &67X,"NEGS. UTILIZE GAMMA LIMIT"/)

      WRITE(6,2006)
 2006 FORMAT(///6X,"(1),  INITIAL VALUE. SHOWN NEG. IF SCHEDULED."/    &
     &6X,"(2),  ALL ENGINES OPERATING. VOBS NEG. IF BELOW VZERC LIM."/ &
     &6X,"(3),  TO F.L. SHOWN NEG. IF 1ST OR 2ND SEG. GRADS NOT MET."/ &
     &6X,"(4),  NEGS. ARE PERCENT TGREF, AT SLS. (THROTTLED T.O.)"/    &
     &6X,"(5),  NEGS. UTILIZE ATR DURING ENG. OUT T.O."/               &
     &6X,"(6),  NEGS. ARE FOR CASE TERMINATED, CHECK CASE LISTING.")

 2100 IF (NCODE.LT.2) GO TO 2150
      ENDFILE 8
      REWIND 8
 2150 STOP
 
END PROGRAM WEF1041   ! ========================================================


